@@ -1,5 +1,5 @@
 /* apps/apps.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,13 +47,19 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <string.h>
+#include <sys/types.h>
 #include <sys/stat.h>
+#define NON_MAIN
 #include "apps.h"
+#undef NON_MAIN
 
+#ifdef WIN16
+#define APPS_WIN16
+#include "../crypto/buffer/bss_file.c"
+#endif
+
+#ifdef undef /* never finished - probably never will be :-) */
 int args_from_file(file,argc,argv)
 char *file;
 int *argc;
@@ -133,6 +139,7 @@ char **argv[];
 	*argc=num;
 	return(1);
 	}
+#endif
 
 int str2fmt(s)
 char *s;
@@ -148,3 +155,61 @@ char *s;
 	else
 		return(FORMAT_UNDEF);
 	}
+
+#if defined(MSDOS) || defined(WIN32) || defined(WIN16)
+void program_name(in,out,size)
+char *in;
+char *out;
+int size;
+	{
+	int i,n;
+	char *p=NULL;
+
+	n=strlen(in);
+	/* find the last '/', '\' or ':' */
+	for (i=n-1; i>0; i--)
+		{
+		if ((in[i] == '/') || (in[i] == '\\') || (in[i] == ':'))
+			{
+			p= &(in[i+1]);
+			break;
+			}
+		}
+	if (p == NULL)
+		p=in;
+	n=strlen(p);
+	/* strip off trailing .exe if present. */
+	if ((n > 4) && (p[n-4] == '.') &&
+		((p[n-3] == 'e') || (p[n-3] == 'E')) &&
+		((p[n-2] == 'x') || (p[n-2] == 'X')) &&
+		((p[n-1] == 'e') || (p[n-1] == 'E')))
+		n-=4;
+	if (n > size-1)
+		n=size-1;
+
+	for (i=0; i<n; i++)
+		{
+		if ((p[i] >= 'A') && (p[i] <= 'Z'))
+			out[i]=p[i]-'A'+'a';
+		else
+			out[i]=p[i];
+		}
+	out[n]='\0';
+	}
+#else
+void program_name(in,out,size)
+char *in;
+char *out;
+int size;
+	{
+	char *p;
+
+	p=strrchr(in,'/');
+	if (p != NULL)
+		p++;
+	else
+		p=in;
+	strncpy(out,p,size-1);
+	out[size-1]='\0';
+	}
+#endif
