@@ -1,5 +1,5 @@
 /* apps/apps.h */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -45,42 +45,58 @@
  * [including the GNU Public Licence.]
  */
 
-#if defined(MSDOS) || defined(_Windows)
-#define EXIT(n)		return(n);
-#define LIST_SEPERATOR_CHAR ';'
-#include <io.h>
+#ifndef HEADER_APPS_H
+#define HEADER_APPS_H
 
-#define SSLEAY_CONF	"ssleay.cnf"
+#include "../e_os.h"
 
-#else
-
-#define SSLEAY_CONF	"ssleay.conf"
-
-#define LIST_SEPERATOR_CHAR ':'
-#ifndef MONOLITH
-#define	EXIT(n)		exit(n); return(n)
-#else
-#define EXIT(n)		return(n)
-#endif
+#include "buffer.h"
 
+#ifdef WIN16
+BIO_METHOD *BIO_s_file();
 #endif
 
 #ifndef MONOLITH
+
 #define MAIN(a,v)	main(a,v)
+
+#ifndef NON_MAIN
+BIO *bio_err=NULL;
+#endif
+
 #else
+
 #define MAIN(a,v)	PROG(a,v)
 #include "conf.h"
 extern LHASH *config;
+extern BIO *bio_err;
+
+#endif
+
+#if defined(MONOLITH) && !defined(SSLEAY)
+#  define apps_startup()
+#else
+#  if defined(MSDOS) || defined(WIN16) || defined(WIN32)
+#    ifdef _O_BINARY
+#      define apps_startup() _fmode=_O_BINARY
+#    else
+#      define apps_startup() _fmode=O_BINARY
+#    endif
+#  else
+#    define apps_startup()
+#  endif
 #endif
 
 #ifndef NOPROTO
 int should_retry(int i);
 int args_from_file(char *file, int *argc, char **argv[]);
 int str2fmt(char *s);
+void program_name(char *in,char *out,int size);
 #else
 int should_retry();
 int args_from_file();
 int str2fmt();
+void program_name();
 #endif
 
 #define FORMAT_UNDEF    0
@@ -88,3 +104,5 @@ int str2fmt();
 #define FORMAT_TEXT     2
 #define FORMAT_PEM      3
 #define FORMAT_NETSCAPE 4
+
+#endif
