@@ -1,5 +1,5 @@
 /* apps/ca.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,18 +47,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#define X_OK	0
-#define W_OK	2
-#define R_OK	4
-#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "apps.h"
+#include "buffer.h"
 #include "err.h"
 #include "bn.h"
 #include "rsa.h"
@@ -69,29 +62,35 @@
 #include "pem.h"
 #include "conf.h"
 
+#ifndef W_OK
+#include <sys/file.h>
+#endif
+
+#undef PROG
 #define PROG ca_main
 
 #define BASE_SECTION	"ca"
-#define CONFIG_FILE "/usr/local/ssl/lib/ssleay.conf"
+#define CONFIG_FILE "lib/ssleay.conf"
 
 #define ENV_DEFAULT_CA "default_ca"
 
-#define ENV_DIR		"dir"
-#define ENV_CERTS	"certs"
-#define ENV_CRL_DIR	"crl_dir"
-#define ENV_CA_DB	"CA_DB"
+#define ENV_DIR			"dir"
+#define ENV_CERTS		"certs"
+#define ENV_CRL_DIR		"crl_dir"
+#define ENV_CA_DB		"CA_DB"
 #define ENV_NEW_CERTS_DIR	"new_certs_dir"
-#define ENV_CERTIFICATE "certificate"
-#define ENV_SERIAL	"serial"
-#define ENV_CRL		"crl"
-#define ENV_PRIVATE_KEY	"private_key"
-#define ENV_RANDFILE	"RANDFILE"
-#define ENV_DEFAULT_DAYS "default_days"
-#define ENV_DEFAULT_CRL_DAYS "default_crl_days"
-#define ENV_DEFAULT_MD	"default_md"
-#define ENV_POLICY      "policy"
+#define ENV_CERTIFICATE 	"certificate"
+#define ENV_SERIAL		"serial"
+#define ENV_CRL			"crl"
+#define ENV_PRIVATE_KEY		"private_key"
+#define ENV_RANDFILE		"RANDFILE"
+#define ENV_DEFAULT_DAYS 	"default_days"
+#define ENV_DEFAULT_CRL_DAYS 	"default_crl_days"
+#define ENV_DEFAULT_CRL_HOURS 	"default_crl_hours"
+#define ENV_DEFAULT_MD		"default_md"
+#define ENV_POLICY      	"policy"
 
-#define ENV_DATABASE	"database"
+#define ENV_DATABASE		"database"
 
 #define DB_type         0
 #define DB_exp_date     1
@@ -105,13 +104,15 @@
 #define DB_TYPE_EXP	'E'
 #define DB_TYPE_VAL	'V'
 
-static char *usage[]={
+static char *ca_usage[]={
 "usage: ca args\n",
 "\n",
 " -verbose        - Talk alot while doing things\n",
 " -config file    - A config file\n",
 " -name arg       - The particular CA definition to use\n",
-" -gencrl days    - Generate a new CRL, days is when the next CRL is due\n",
+" -gencrl         - Generate a new CRL\n",
+" -crldays days   - Days is when the next CRL is due\n",
+" -crlhours hours - Hours is when the next CRL is due\n",
 " -days arg       - number of days to certify the certificate for\n",
 " -md arg         - md to use, one of md2, md5, sha or sha1\n",
 " -policy arg     - The CA 'policy' to support\n",
@@ -127,7 +128,7 @@ NULL
 
 #ifndef NOPROTO
 static void lookup_fail(char *name,char *tag);
-static int key_callback(char *buf,int len,int verify);
+static int MS_CALLBACK key_callback(char *buf,int len,int verify);
 static unsigned long index_serial_hash(char **a);
 static int index_serial_cmp(char **a, char **b);
 static unsigned long index_name_hash(char **a);
@@ -139,10 +140,10 @@ static int certify(X509 **xret, char *infile,RSA *rsa,X509 *x509,
 	EVP_MD *dgst,STACK *policy,TXT_DB *db,BIGNUM *serial,int days,
 	int batch, int verbose);
 static int cmp_on_nid(X509_NAME_ENTRY **a,X509_NAME_ENTRY **b);
-static void write_new_certificate(FILE *fp, X509 *x);
+static void write_new_certificate(BIO *bp, X509 *x);
 #else
 static void lookup_fail();
-static int key_callback();
+static int MS_CALLBACK key_callback();
 static unsigned long index_serial_hash();
 static int index_serial_cmp();
 static unsigned long index_name_hash();
@@ -171,8 +172,9 @@ char **argv;
 	int verbose=0;
 	int gencrl=0;
 	long crldays=0;
+	long crlhours=0;
 	long errorline=-1;
-	char *configfile=CONFIG_FILE;
+	char *configfile=NULL;
 	char *md=NULL;
 	char *policy=NULL;
 	char *keyfile=NULL;
@@ -187,22 +189,28 @@ char **argv;
 	RSA *rsa=NULL;
 	X509 *x509=NULL;
 	X509 *x=NULL;
-	FILE *in=NULL,*out=NULL,*DBout=NULL,*Cout=NULL;
+	BIO *in=NULL,*out=NULL,*Sout=NULL,*Cout=NULL;
 	char *dbfile=NULL;
 	TXT_DB *db=NULL;
 	X509_CRL *crl=NULL;
 	X509_CRL_INFO *ci=NULL;
 	X509_REVOKED *r=NULL;
 	char **pp,*p,*f;
-	unsigned int i;
-	int j,k,n,m;
+	int i,j,k,n,m;
 	long l;
 	EVP_MD *dgst=NULL;
 	STACK *attribs=NULL;
 	STACK *cert_sk=NULL;
+#undef BSIZE
 #define BSIZE 256
 	char buf[3][BSIZE];
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
 	argc--;
 	argv++;
 	while (argc >= 1)
@@ -274,6 +282,11 @@ char **argv;
 			if (--argc < 1) goto bad;
 			crldays= atol(*(++argv));
 			}
+		else if (strcmp(*argv,"-crlhours") == 0)
+			{
+			if (--argc < 1) goto bad;
+			crlhours= atol(*(++argv));
+			}
 		else if (strcmp(*argv,"-infiles") == 0)
 			{
 			argc--;
@@ -296,7 +309,7 @@ bad:
 		{
 		char **pp;
 
-		for (pp=usage; (*pp != NULL); pp++)
+		for (pp=ca_usage; (*pp != NULL); pp++)
 			fprintf(stderr,*pp);
 		goto err;
 		}
@@ -304,6 +317,15 @@ bad:
 	ERR_load_crypto_strings();
 
 	/*****************************************************************/
+	if (configfile == NULL)
+		{
+		/* We will just use 'buf[0]' as a temporary buffer.  */
+		strncpy(buf[0],X509_get_default_cert_area(),
+			sizeof(buf[0])-2-sizeof(CONFIG_FILE));
+		strcat(buf[0],"/");
+		strcat(buf[0],CONFIG_FILE);
+		configfile=buf[0];
+		}
 	if ((conf=CONF_load(NULL,configfile,&errorline)) == NULL)
 		{
 		if (errorline <= 0)
@@ -326,6 +348,16 @@ bad:
 			}
 		}
 
+	in=BIO_new(BIO_s_file());
+	out=BIO_new(BIO_s_file());
+	Sout=BIO_new(BIO_s_file());
+	Cout=BIO_new(BIO_s_file());
+	if ((in == NULL) || (out == NULL) || (Sout == NULL) || (Cout == NULL))
+		{
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+
 	/*****************************************************************/
 	/* we definitly need an RSA key, so lets get it */
 	if ((keyfile == NULL) && ((keyfile=CONF_get_string(conf,
@@ -334,20 +366,19 @@ bad:
 		lookup_fail(section,ENV_PRIVATE_KEY);
 		goto err;
 		}
-	if ((in=fopen(keyfile,"r")) == NULL)
+	if (BIO_read_filename(in,keyfile) <= 0)
 		{
 		perror(keyfile);
 		fprintf(stderr,"trying to load CA private key\n");
 		goto err;
 		}
 	if (key == NULL)
-		rsa=PEM_read_RSAPrivateKey(in,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
 	else
 		{
-		rsa=PEM_read_RSAPrivateKey(in,NULL,key_callback);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,key_callback);
 		memset(key,0,strlen(key));
 		}
-	fclose(in);
 	if (rsa == NULL)
 		{
 		fprintf(stderr,"unable to load CA private key\n");
@@ -362,15 +393,13 @@ bad:
 		lookup_fail(section,ENV_CERTIFICATE);
 		goto err;
 		}
-	if ((in=fopen(certfile,"r")) == NULL)
+        if (BIO_read_filename(in,certfile) <= 0)
 		{
 		perror(certfile);
 		fprintf(stderr,"trying to load CA certificate\n");
 		goto err;
 		}
-	x509=PEM_read_X509(in,NULL,NULL);
-	fclose(in);
-	in=NULL;
+	x509=PEM_read_bio_X509(in,NULL,NULL);
 	if (x509 == NULL)
 		{
 		fprintf(stderr,"unable to load CA certificate\n");
@@ -417,14 +446,13 @@ bad:
 		lookup_fail(section,ENV_DATABASE);
 		goto err;
 		}
-	if ((in=fopen(dbfile,"r")) == NULL)
+        if (BIO_read_filename(in,dbfile) <= 0)
 		{
 		perror(dbfile);
 		fprintf(stderr,"unable to open '%s'\n",dbfile);
 		goto err;
 		}
-	db=TXT_DB_load(in,DB_NUMBER);
-	fclose(in);
+	db=TXT_DB_read(in,DB_NUMBER);
 	if (db == NULL) goto err;
 
 	/* Lets check some fields */
@@ -472,7 +500,8 @@ bad:
 		}
 	if (verbose)
 		{
-		TXT_DB_write(stdout,db);
+		BIO_set_fp(out,stdout,BIO_NOCLOSE); /* cannot fail */
+		TXT_DB_write(out,db);
 		fprintf(stderr,"%d entries loaded from the database\n",
 			db->data->num);
 		fprintf(stderr,"generating indexs\n");
@@ -528,7 +557,7 @@ bad:
 
 		if (days == 0)
 			{
-			days=CONF_get_number(conf,section,
+			days=(int)CONF_get_number(conf,section,
 				ENV_DEFAULT_DAYS);
 			}
 		if (days == 0)
@@ -554,16 +583,18 @@ bad:
 			fprintf(stderr,"unable to find 'section' for %s\n",policy);
 			goto err;
 			}
+
 		if (outfile != NULL)
 			{
-			if ((out=fopen(outfile,"w")) == NULL)
+
+			if (BIO_write_filename(Sout,outfile) <= 0)
 				{
 				perror(outfile);
 				goto err;
 				}
 			}
 		else
-			out=stdout;
+			BIO_set_fp(Sout,stdout,BIO_NOCLOSE);
 
 		if ((cert_sk=sk_new_null()) == NULL)
 			{
@@ -588,7 +619,7 @@ bad:
 					}
 				}
 			}
-		for (i=0; i<(unsigned int)argc; i++)
+		for (i=0; i<argc; i++)
 			{
 			total++;
 			j=certify(&x,argv[i],rsa,x509,dgst,attribs,db,
@@ -635,14 +666,13 @@ bad:
 
 			strncpy(buf[1],dbfile,BSIZE-4);
 			strcat(buf[1],".new");
-			if ((DBout=fopen(buf[1],"w")) == NULL)
+			if (BIO_write_filename(out,buf[1]) <= 0)
 				{
 				perror(dbfile);
 				fprintf(stderr,"unable to open '%s'\n",dbfile);
 				goto err;
 				}
-			l=TXT_DB_write(DBout,db);
-			fclose(DBout);
+			l=TXT_DB_write(out,db);
 			if (l <= 0) goto err;
 			}
 	
@@ -661,25 +691,31 @@ bad:
 			strncpy(buf[2],outdir,BSIZE-(j*2)-6);
 			strcat(buf[2],"/");
 			n=(unsigned char *)&(buf[2][strlen(buf[2])]);
-			for (k=0; k<j; k++)
+			if (j > 0)
 				{
-				sprintf((char *)n,"%02X",*(in++));
-				n+=2;
+				for (k=0; k<j; k++)
+					{
+					sprintf((char *)n,"%02X",*(in++));
+					n+=2;
+					}
+				}
+			else
+				{
+				*(n++)='0';
+				*(n++)='0';
 				}
 			*(n++)='.'; *(n++)='p'; *(n++)='e'; *(n++)='m';
 			*n='\0';
 			if (verbose)
 				fprintf(stderr,"writing %s\n",buf[2]);
 
-			if ((Cout=fopen(buf[2],"w")) == NULL)
+			if (BIO_write_filename(Cout,buf[2]) <= 0)
 				{
 				perror(buf[2]);
 				goto err;
 				}
 			write_new_certificate(Cout,x);
-			fclose(Cout);
-			Cout=NULL;
-			write_new_certificate(out,x);
+			write_new_certificate(Sout,x);
 			}
 
 		if (sk_num(cert_sk))
@@ -727,12 +763,16 @@ bad:
 	/*****************************************************************/
 	if (gencrl)
 		{
-		if (!crldays)
+		EVP_PKEY pkey;
+
+		if (!crldays && !crlhours)
 			{
 			crldays=CONF_get_number(conf,section,
 				ENV_DEFAULT_CRL_DAYS);
+			crlhours=CONF_get_number(conf,section,
+				ENV_DEFAULT_CRL_HOURS);
 			}
-		if (crldays == 0)
+		if ((crldays == 0) && (crlhours == 0))
 			{
 			fprintf(stderr,"cannot lookup how long until the next CRL is issuer\n");
 			goto err;
@@ -745,8 +785,8 @@ bad:
 		ci->issuer=X509_NAME_dup(x509->cert_info->subject);
 		if (ci->issuer == NULL) goto err;
 
-		X509_gmtime(ci->lastUpdate,0);
-		X509_gmtime(ci->nextUpdate,crldays*60*60);
+		X509_gmtime_adj(ci->lastUpdate,0);
+		X509_gmtime_adj(ci->nextUpdate,(crldays*24+crlhours)*60*60);
 
 		for (i=0; i<sk_num(db->data); i++)
 			{
@@ -796,24 +836,39 @@ bad:
 
 		/* we how have a CRL */
 		if (verbose) fprintf(stderr,"signing CRL\n");
-		if (!X509_CRL_sign(crl,rsa,EVP_md5)) goto err;
+		pkey.type=EVP_PKEY_RSA;
+		pkey.pkey.rsa=rsa;
+		if (md != NULL)
+			{
+			if ((dgst=EVP_get_MDbyname(md)) == NULL)
+				{
+				fprintf(stderr,"%s is an unsuported message digest type\n",md);
+				goto err;
+				}
+			}
+		else
+			dgst=EVP_md5();
+		if (!X509_CRL_sign(crl,&pkey,dgst)) goto err;
 
-		PEM_write_X509_CRL(stdout,crl);
+		PEM_write_bio_X509_CRL(Sout,crl);
 		}
 	/*****************************************************************/
 	ret=0;
 err:
-	if (Cout != NULL) fclose(Cout);
+	if (Cout != NULL) BIO_free(Cout);
+	if (Sout != NULL) BIO_free(Sout);
+	if (out != NULL) BIO_free(out);
+	if (in != NULL) BIO_free(in);
+
 	if (cert_sk != NULL)
 		{
 		while ((x=(X509 *)sk_pop(cert_sk)) != NULL)
 			X509_free(x);
 		sk_free(cert_sk);
 		}
-	if (ret) ERR_print_errors(stderr);
+	if (ret) ERR_print_errors(bio_err);
 	if (serial != NULL) BN_free(serial);
 	if (db != NULL) TXT_DB_free(db);
-	if ((out != NULL) && (out != stdout)) fclose(out);
 	if (rsa != NULL) RSA_free(rsa);
 	if (x509 != NULL) X509_free(x509);
 	if (crl != NULL) X509_CRL_free(crl);
@@ -828,7 +883,7 @@ char *tag;
 	fprintf(stderr,"variable lookup failed for %s::%s\n",name,tag);
 	}
 
-static int key_callback(buf,len,verify)
+static int MS_CALLBACK key_callback(buf,len,verify)
 char *buf;
 int len,verify;
 	{
@@ -878,32 +933,38 @@ char **b;
 static BIGNUM *load_serial(serialfile)
 char *serialfile;
 	{
-	FILE *in=NULL;
+	BIO *in=NULL;
 	BIGNUM *ret=NULL;
-	char buf[1024];
+	MS_STATIC char buf[1024];
 	ASN1_INTEGER *ai=NULL;
 
-	if ((in=fopen(serialfile,"r")) == NULL)
+	if ((in=BIO_new(BIO_s_file())) == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+
+	if (BIO_read_filename(in,serialfile) <= 0)
 		{
 		perror(serialfile);
 		goto err;
 		}
 	ai=ASN1_INTEGER_new();
 	if (ai == NULL) goto err;
-	if (!f2i_ASN1_INTEGER(in,ai,buf,1024))
+	if (!a2i_ASN1_INTEGER(in,ai,buf,1024))
 		{
 		fprintf(stderr,"unable to load number from %s\n",
 			serialfile);
 		goto err;
 		}
-	ret=ASN1_INTEGER_to_BIGNUM(ai,NULL);
+	ret=ASN1_INTEGER_to_BN(ai,NULL);
 	if (ret == NULL)
 		{
 		fprintf(stderr,"error converting number from bin to BIGNUM");
 		goto err;
 		}
 err:
-	if (in != NULL) fclose(in);
+	if (in != NULL) BIO_free(in);
 	if (ai != NULL) ASN1_INTEGER_free(ai);
 	return(ret);
 	}
@@ -912,25 +973,32 @@ static int save_serial(serialfile,serial)
 char *serialfile;
 BIGNUM *serial;
 	{
-	FILE *out;
+	BIO *out;
 	int ret=0;
 	ASN1_INTEGER *ai=NULL;
 
-	if ((out=fopen(serialfile,"w")) == NULL)
+	out=BIO_new(BIO_s_file());
+	if (out == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+	if (BIO_write_filename(out,serialfile) <= 0)
 		{
 		perror(serialfile);
 		goto err;
 		}
-	if ((ai=BIGNUM_to_ASN1_INTEGER(serial,NULL)) == NULL)
+
+	if ((ai=BN_to_ASN1_INTEGER(serial,NULL)) == NULL)
 		{
 		fprintf(stderr,"error converting serial to ASN.1 format\n");
 		goto err;
 		}
-	i2f_ASN1_INTEGER(out,ai);
-	fprintf(out,"\n");
+	i2a_ASN1_INTEGER(out,ai);
+	BIO_puts(out,"\n");
 	ret=1;
 err:
-	if (out != NULL) fclose(out);
+	if (out != NULL) BIO_free(out);
 	if (ai != NULL) ASN1_INTEGER_free(ai);
 	return(ret);
 	}
@@ -949,7 +1017,7 @@ int days;
 int batch;
 int verbose;
 	{
-	FILE *in=NULL;
+	BIO *in=NULL,*b_err=NULL;
 	STACK *name=NULL,*CAname=NULL;
 	STACK *subject=NULL;
 	X509_REQ *req=NULL;
@@ -957,29 +1025,38 @@ int verbose;
 	X509_CINF *ci;
 	X509_NAME_ENTRY *ne;
 	X509_NAME_ENTRY tmp,*tne,*push;
-	int ok=-1,j;
-	unsigned int i;
+	int ok=-1,i,j;
 	char *p;
 	CONF_VALUE *cv;
 	char *row[DB_NUMBER],**rrow,**irow=NULL;
-	RSA *pkey=NULL;
+	EVP_PKEY *pkey=NULL;
+	EVP_PKEY static_pkey;
 
 	for (i=0; i<DB_NUMBER; i++)
 		row[i]=NULL;
 
-	if ((in=fopen(infile,"r")) == NULL)
+	in=BIO_new(BIO_s_file());
+	b_err=BIO_new(BIO_s_file());
+	if ((in == NULL) || (b_err == NULL))
+		{
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+	BIO_set_fp(b_err,stderr,BIO_NOCLOSE);
+
+	if (BIO_read_filename(in,infile) <= 0)
 		{
 		perror(infile);
 		goto err;
 		}
-	if ((req=PEM_read_X509_REQ(in,NULL,NULL)) == NULL)
+	if ((req=PEM_read_bio_X509_REQ(in,NULL,NULL)) == NULL)
 		{
 		fprintf(stderr,"Error reading certificate request in %s\n",
 			infile);
 		goto err;
 		}
 	if (verbose)
-		X509_REQ_print(stderr,req);
+		X509_REQ_print(b_err,req);
 
 	fprintf(stderr,"Check that the request matches the signature\n");
 
@@ -998,7 +1075,7 @@ int verbose;
 		goto err;
 		}
 	i=X509_REQ_verify(req,pkey);
-	RSA_free(pkey);
+	EVP_PKEY_free(pkey);
 	if (!i)
 		{
 		ok=0;
@@ -1013,7 +1090,7 @@ int verbose;
 	for (i=0; i<sk_num(name); i++)
 		{
 		ne=(X509_NAME_ENTRY *)sk_value(name,i);
-		j=i2f_ASN1_OBJECT(stderr,ne->object);
+		j=i2a_ASN1_OBJECT(b_err,ne->object);
 		for (j=22-j; j>0; j--)
 			fputc(' ',stderr);
 		fputc(':',stderr);
@@ -1232,16 +1309,16 @@ int verbose;
 		j=0;
 	/* else ci->version=NULL; */
 
-	if (BIGNUM_to_ASN1_INTEGER(serial,ci->serialNumber) == NULL)
+	if (BN_to_ASN1_INTEGER(serial,ci->serialNumber) == NULL)
 		goto err;
 	X509_NAME_free(ci->issuer);
 	if ((ci->issuer=X509_NAME_dup(x509->cert_info->subject)) == NULL)
 		goto err;
 
 	fprintf(stderr,"Certificate is to be certified until ");
-	X509_gmtime(ci->validity->notBefore,0);
-	X509_gmtime(ci->validity->notAfter,60*60*24*days);
-	ASN1_UTCTIME_print(stderr,ci->validity->notAfter);
+	X509_gmtime_adj(ci->validity->notBefore,0);
+	X509_gmtime_adj(ci->validity->notAfter,(long)60*60*24*days);
+	ASN1_UTCTIME_print(b_err,ci->validity->notAfter);
 	fprintf(stderr," (%d days)\n",days);
 
 	X509_NAME_free(ci->subject);
@@ -1275,7 +1352,9 @@ int verbose;
 			goto err;
 			}
 		}
-	if (!X509_sign(ret,rsa,dgst))
+	static_pkey.type=EVP_PKEY_RSA;
+	static_pkey.pkey.rsa=rsa;
+	if (!X509_sign(ret,&static_pkey,dgst))
 		goto err;
 
 	/* We now just add it to the database */
@@ -1321,7 +1400,8 @@ err:
 		if (row[i] != NULL) free(row[i]);
 
 	if (req != NULL) X509_REQ_free(req);
-	if (in != NULL) fclose(in);
+	if (in != NULL) BIO_free(in);
+	if (b_err != NULL) BIO_free(b_err);
 	if (CAname != NULL)
 		X509_NAME_free(CAname);
 	if (ok <= 0)
@@ -1349,21 +1429,26 @@ X509_NAME_ENTRY **a,**b;
 	return(OBJ_obj2nid((*a)->object)-OBJ_obj2nid((*b)->object));
 	}
 
-static void write_new_certificate(fp,x)
-FILE *fp;
+static void write_new_certificate(bp,x)
+BIO *bp;
 X509 *x;
 	{
 	char *f;
 
 	f=X509_NAME_oneline(X509_get_issuer_name(x));
-	fprintf(fp,"issuer :%s\n",f); free(f);
+	BIO_puts(bp,"issuer :");
+	BIO_puts(bp,f);
+	BIO_puts(bp,"\n"); free(f);
 	f=X509_NAME_oneline(X509_get_subject_name(x));
-	fprintf(fp,"subject:%s\n",f); free(f);
-	fprintf(fp,"serial :");
-	i2f_ASN1_INTEGER(fp,x->cert_info->serialNumber);
-	fprintf(fp,"\n\n");
-	X509_print(fp,x);
-	fprintf(fp,"\n");
-	PEM_write_X509(fp,x);
-	fprintf(fp,"\n");
+	BIO_puts(bp,"subject:");
+	BIO_puts(bp,f);
+	BIO_puts(bp,"\n"); free(f);
+	BIO_puts(bp,"serial :");
+	i2a_ASN1_INTEGER(bp,x->cert_info->serialNumber);
+	BIO_puts(bp,"\n\n");
+	X509_print(bp,x);
+	BIO_puts(bp,"\n");
+	PEM_write_bio_X509(bp,x);
+	BIO_puts(bp,"\n");
 	}
+
