@@ -1,5 +1,5 @@
 /* apps/req.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,13 +48,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <string.h>
+#include "apps.h"
+#include "buffer.h"
 #include "rsa.h"
 #include "envelope.h"
-#include "apps.h"
 #include "rand.h"
 #include "conf.h"
 #include "err.h"
@@ -73,6 +71,7 @@
 #define DEFAULT_KEY_LENGTH	512
 #define MIN_KEY_LENGTH		384
 
+#undef PROG
 #define PROG	req_main
 
 /* -inform arg	- input format - default PEM (one of DER, TXT or PEM)
@@ -98,18 +97,19 @@ static int add_attribute_object(X509_NAME *n, char *text, char *def,
 	char *value, int nid,int min,int max);
 static int add_DN_object(X509_NAME *n, char *text, char *def, char *value,
 	int nid,int min,int max);
-static void cb(int p,int n);
+static void MS_CALLBACK req_cb(int p,int n);
 static int fix_data(int nid,int *type,int len,int min,int max);
 #else
 static int make_REQ();
 static int add_object();
-static void cb();
+static void MS_CALLBACK req_cb();
 static int fix_data();
 #endif
 
 #ifndef MONOLITH
-LHASH *config=NULL;
+static LHASH *config=NULL;
 #endif
+static LHASH *conf=NULL;
 
 int MAIN(argc, argv)
 int argc;
@@ -120,13 +120,19 @@ char **argv;
 	X509_REQ *req=NULL;
 	RSA *rsa=NULL;
 	int i,badops=0,new=0,newkey= -1;
-	FILE *in=NULL,*out=NULL;
+	BIO *in=NULL,*out=NULL;
 	int informat,outformat,verify=0,noout=0,text=0,keyform=FORMAT_PEM;
 	int nodes=0,kludge=0;
 	char *infile,*outfile,*prog,*keyfile=NULL,*template=NULL,*keyout=NULL;
-	EVP_CIPHER *cipher=EVP_des_ede3_cbc;
+	EVP_CIPHER *cipher=EVP_des_ede3_cbc();
 	char *p;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
 	infile=NULL;
 	outfile=NULL;
 	informat=FORMAT_PEM;
@@ -245,27 +251,37 @@ bad:
 		{
 		long errline;
 
-		config=CONF_load(NULL,template,&errline);
-		if (config == NULL)
+		conf=CONF_load(NULL,template,&errline);
+		if (conf == NULL)
 			{
 			fprintf(stderr,"error on line %ld of %s\n",errline,template);
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
+	else
+		conf=config;
+
+	in=BIO_new(BIO_s_file());
+	out=BIO_new(BIO_s_file());
+	if ((in == NULL) || (out == NULL))
+		{
+		ERR_print_errors(bio_err);
+		goto end;
+		}
 
 	if (keyfile != NULL)
 		{
-		in=fopen(keyfile,"r");
-		if (in == NULL)
+		if (BIO_read_filename(in,keyfile) <= 0)
 			{
 			perror(keyfile);
 			goto end;
 			}
+
 		if (keyform == FORMAT_ASN1)
-			rsa=d2i_RSAPrivateKey_fp(in,NULL);
+			rsa=d2i_RSAPrivateKey_bio(in,NULL);
 		else if (keyform == FORMAT_PEM)
-			rsa=PEM_read_RSAPrivateKey(in,NULL,NULL);
+			rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
 		else
 			{
 			fprintf(stderr,"bad input format specified for X509 request\n");
@@ -275,11 +291,9 @@ bad:
 		if (rsa == NULL)
 			{
 			fprintf(stderr,"unable to load RSA key\n");
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
-		fclose(in);
-		in=NULL;
 		}
 
 	if (new && (rsa == NULL))
@@ -287,7 +301,7 @@ bad:
 		char *randfile;
 		char buffer[200];
 
-		if ((randfile=CONF_get_string(config,SECTION,"RANDFILE")) == NULL)
+		if ((randfile=CONF_get_string(conf,SECTION,"RANDFILE")) == NULL)
 			randfile=RAND_file_name(buffer,200);
 		if ((randfile == NULL) || !RAND_load_file(randfile,1024L*1024L))
 			{
@@ -299,7 +313,7 @@ bad:
 			}
 		if (newkey <= 0)
 			{
-			newkey=CONF_get_number(config,SECTION,BITS);
+			newkey=(int)CONF_get_number(conf,SECTION,BITS);
 			if (newkey <= 0)
 				newkey=DEFAULT_KEY_LENGTH;
 			}
@@ -310,13 +324,13 @@ bad:
 			goto end;
 			}
 		fprintf(stderr,"Generating a %d bit private key\n",newkey);
-		rsa=RSA_generate_key(newkey,0x10001,cb);
+		rsa=RSA_generate_key(newkey,0x10001,req_cb);
 		if ((randfile == NULL) || (RAND_write_file(randfile) == 0))
 			fprintf(stderr,"unable to write 'random state'\n");
 
 		if (rsa == NULL)
 			{
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 
@@ -328,32 +342,31 @@ bad:
 			}
 
 		if (keyout == NULL)
-			keyout=CONF_get_string(config,SECTION,KEYFILE);
+			keyout=CONF_get_string(conf,SECTION,KEYFILE);
 
 		if (keyout == NULL)
 			{
 			fprintf(stderr,"writing new private key to stdout\n");
-			out=stdout;
+			BIO_set_fp(out,stdout,BIO_NOCLOSE);
 			}
 		else
 			{
 			fprintf(stderr,"writing new private key to '%s'\n",keyout);
-			out=fopen(keyout,"w");
-			if (out == NULL)
+			if (BIO_write_filename(out,keyout) <= 0)
 				{
 				perror(keyout);
 				goto end;
 				}
 			}
 
-		p=CONF_get_string(config,SECTION,"encrypt_rsa_key");
+		p=CONF_get_string(conf,SECTION,"encrypt_rsa_key");
 		if ((p != NULL) && (strcmp(p,"no") == 0))
 			cipher=NULL;
 		if (nodes) cipher=NULL;
 		
 		i=0;
 loop:
-		if (!PEM_write_RSAPrivateKey(out,rsa,cipher,
+		if (!PEM_write_bio_RSAPrivateKey(out,rsa,cipher,
 			NULL,0,NULL))
 			{
 			if ((ERR_GET_REASON(ERR_peek_error()) ==
@@ -363,11 +376,10 @@ loop:
 				i++;
 				goto loop;
 				}
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		fprintf(stderr,"-----\n");
-		if (out != stdout) fclose(out);
 		}
 
 	if (!new)
@@ -377,11 +389,10 @@ loop:
 		 * changed. */
 		kludge= -1;
 		if (infile == NULL)
-			in=stdin;
+			BIO_set_fp(in,stdin,BIO_NOCLOSE);
 		else
 			{
-			in=fopen(infile,"r");
-			if (in == NULL)
+			if (BIO_read_filename(in,infile) <= 0)
 				{
 				perror(infile);
 				goto end;
@@ -389,9 +400,9 @@ loop:
 			}
 
 		if	(informat == FORMAT_ASN1)
-			req=d2i_X509_REQ_fp(in,NULL);
+			req=d2i_X509_REQ_bio(in,NULL);
 		else if (informat == FORMAT_PEM)
-			req=PEM_read_X509_REQ(in,NULL,NULL);
+			req=PEM_read_bio_X509_REQ(in,NULL,NULL);
 		else
 			{
 			fprintf(stderr,"bad input format specified for X509 request\n");
@@ -400,19 +411,24 @@ loop:
 		if (req == NULL)
 			{
 			fprintf(stderr,"unable to load X509 request\n");
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
 
 	if (new || x509)
 		{
+		if (rsa == NULL)
+			{
+			fprintf(stderr,"you need to specify a private key\n");
+			goto end;
+			}
 		if (req == NULL)
 			{
 			req=X509_REQ_new();
 			if (req == NULL)
 				{
-				ERR_print_errors(stderr);
+				ERR_print_errors(bio_err);
 				goto end;
 				}
 
@@ -422,12 +438,13 @@ loop:
 			if (!i)
 				{
 				fprintf(stderr,"problems making Certificate Request\n");
-				ERR_print_errors(stderr);
+				ERR_print_errors(bio_err);
 				goto end;
 				}
 			}
 		if (x509)
 			{
+			EVP_PKEY pkey;
 			X509_CINF *ci;
 			X509_REQ_INFO *ri;
 
@@ -446,8 +463,9 @@ loop:
 			ci->issuer=ri->subject;
 			ri->subject=NULL;
 
-			X509_gmtime(ci->validity->notBefore,0);
-			X509_gmtime(ci->validity->notAfter,60*60*24*days);
+			X509_gmtime_adj(ci->validity->notBefore,0);
+			X509_gmtime_adj(ci->validity->notAfter,
+				(long)60*60*24*days);
 
 			X509_NAME_free(ci->subject);
 			ci->subject=X509_NAME_dup(ci->issuer);
@@ -456,43 +474,55 @@ loop:
 			ci->key=ri->pubkey;
 			ri->pubkey=NULL;
 
-			if (!(i=X509_sign(x509ss,rsa,EVP_md5)))
+			pkey.type=EVP_PKEY_RSA;
+			pkey.pkey.rsa=rsa;
+			if (!(i=X509_sign(x509ss,&pkey,EVP_md5())))
 				goto end;
 			}
 		else
 			{
-			if (!(i=X509_REQ_sign(req,rsa,EVP_md5)))
+			EVP_PKEY pkey;
+
+			pkey.type=EVP_PKEY_RSA;
+			pkey.pkey.rsa=rsa;
+
+			if (!(i=X509_REQ_sign(req,&pkey,EVP_md5())))
 				goto end;
 			}
 		}
 
 	if (verify && !x509)
 		{
-		RSA *r;
+		EVP_PKEY p;
+		EVP_PKEY *pkey=NULL;
 
 		if (rsa == NULL)
 			{
-			r=X509_REQ_extract_key(req);
-			if (r == NULL)
+			pkey=X509_REQ_extract_key(req);
+			if (pkey == NULL)
 				{
-				ERR_print_errors(stderr);
+				ERR_print_errors(bio_err);
 				goto end;
 				}
 			}
 		else
-			r=rsa;
+			{
+			p.type=EVP_PKEY_RSA;
+			p.pkey.rsa=rsa;
+			pkey= &p;
+			}
 
-		i=X509_REQ_verify(req,r);
-		if (r != rsa)
-			RSA_free(r);
+		i=X509_REQ_verify(req,pkey);
+		if (rsa == NULL)
+			EVP_PKEY_free(pkey);
 		if (i < 0)
 			{
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		else if (i == 0)
 			{
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			fprintf(stderr,"verify failure\n");
 			}
 		else /* if (i > 0) */
@@ -506,14 +536,14 @@ loop:
 		}
 
 	if (outfile == NULL)
-		out=stdout;
+		BIO_set_fp(out,stdout,BIO_NOCLOSE);
 	else
 		{
 		if ((keyout != NULL) && (strcmp(outfile,keyout) == 0))
-			out=fopen(outfile,"a");
+			i=(int)BIO_append_filename(out,outfile);
 		else
-			out=fopen(outfile,"w");
-		if (out == NULL)
+			i=(int)BIO_write_filename(out,outfile);
+		if (!i)
 			{
 			perror(outfile);
 			goto end;
@@ -531,9 +561,9 @@ loop:
 	if (!noout && !x509)
 		{
 		if 	(outformat == FORMAT_ASN1)
-			i=i2d_X509_REQ_fp(out,req);
+			i=i2d_X509_REQ_bio(out,req);
 		else if (outformat == FORMAT_PEM)
-			i=PEM_write_X509_REQ(out,req);
+			i=PEM_write_bio_X509_REQ(out,req);
 		else	{
 			fprintf(stderr,"bad output format specified for outfile\n");
 			goto end;
@@ -541,16 +571,16 @@ loop:
 		if (!i)
 			{
 			fprintf(stderr,"unable to write X509 request\n");
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
 	if (!noout && x509 && (x509ss != NULL))
 		{
 		if 	(outformat == FORMAT_ASN1)
-			i=i2d_X509_fp(out,x509ss);
+			i=i2d_X509_bio(out,x509ss);
 		else if (outformat == FORMAT_PEM)
-			i=PEM_write_X509(out,x509ss);
+			i=PEM_write_bio_X509(out,x509ss);
 		else	{
 			fprintf(stderr,"bad output format specified for outfile\n");
 			goto end;
@@ -558,14 +588,15 @@ loop:
 		if (!i)
 			{
 			fprintf(stderr,"unable to write X509 certificate\n");
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
 	ex=0;
 end:
-	if ((in != NULL) && (in != stdin)) fclose(in);
-	if ((out != NULL) && (out != stdout)) fclose(out);
+	if ((conf != NULL) && (conf != config)) CONF_free(conf);
+	if (in != NULL) BIO_free(in);
+	if (out != NULL) BIO_free(out);
 	if (rsa != NULL) RSA_free(rsa);
 	if (req != NULL) X509_REQ_free(req);
 	if (x509ss != NULL) X509_free(x509ss);
@@ -588,26 +619,26 @@ int attribs;
 	STACK *sk,*attr=NULL;
 	CONF_VALUE *v;
 	
-	tmp=CONF_get_string(config,SECTION,DISTINGUISHED_NAME);
+	tmp=CONF_get_string(conf,SECTION,DISTINGUISHED_NAME);
 	if (tmp == NULL)
 		{
 		fprintf(stderr,"unable to find '%s' in config\n",
 			DISTINGUISHED_NAME);
 		goto err;
 		}
-	sk=CONF_get_section(config,tmp);
+	sk=CONF_get_section(conf,tmp);
 	if (sk == NULL)
 		{
 		fprintf(stderr,"unable to get '%s' section\n",tmp);
 		goto err;
 		}
 
-	tmp_attr=CONF_get_string(config,SECTION,ATTRIBUTES);
+	tmp_attr=CONF_get_string(conf,SECTION,ATTRIBUTES);
 	if (tmp_attr == NULL)
 		attr=NULL;
 	else
 		{
-		attr=CONF_get_section(config,tmp_attr);
+		attr=CONF_get_section(conf,tmp_attr);
 		if (attr == NULL)
 			{
 			fprintf(stderr,"unable to get '%s' section\n",tmp_attr);
@@ -619,9 +650,9 @@ int attribs;
 
 	fprintf(stderr,"You are about to be asked to enter information that will be incorperated\n");
 	fprintf(stderr,"into your certificate request.\n");
-	fprintf(stderr,"What you are about to enteris what is called a Distinguished Name or a DN.\n");
-	fprintf(stderr,"There are quite a few field but you can leave some blank\n");
-	fprintf(stderr,"For some field there will be a default value,\n");
+	fprintf(stderr,"What you are about to enter is what is called a Distinguished Name or a DN.\n");
+	fprintf(stderr,"There are quite a few fields but you can leave some blank\n");
+	fprintf(stderr,"For some fields there will be a default value,\n");
 	fprintf(stderr,"If you enter '.', the field will be left blank.\n");
 	fprintf(stderr,"-----\n");
 
@@ -642,18 +673,18 @@ start:		for (;;)
 				goto start;
 
 			sprintf(buf,"%s_default",type);
-			if ((def=CONF_get_string(config,tmp,buf)) == NULL)
+			if ((def=CONF_get_string(conf,tmp,buf)) == NULL)
 				def="";
 				
 			sprintf(buf,"%s_value",type);
-			if ((value=CONF_get_string(config,tmp,buf)) == NULL)
+			if ((value=CONF_get_string(conf,tmp,buf)) == NULL)
 				value=NULL;
 
 			sprintf(buf,"%s_min",type);
-			min=CONF_get_number(config,tmp,buf);
+			min=(int)CONF_get_number(conf,tmp,buf);
 
 			sprintf(buf,"%s_max",type);
-			max=CONF_get_number(config,tmp,buf);
+			max=(int)CONF_get_number(conf,tmp,buf);
 
 			if (!add_DN_object(ri->subject,v->value,def,value,nid,
 				min,max))
@@ -686,20 +717,20 @@ start2:			for (;;)
 					goto start2;
 
 				sprintf(buf,"%s_default",type);
-				if ((def=CONF_get_string(config,tmp_attr,buf))
+				if ((def=CONF_get_string(conf,tmp_attr,buf))
 					== NULL)
 					def="";
 				
 				sprintf(buf,"%s_value",type);
-				if ((value=CONF_get_string(config,tmp_attr,buf))
+				if ((value=CONF_get_string(conf,tmp_attr,buf))
 					== NULL)
 					value=NULL;
 
 				sprintf(buf,"%s_min",type);
-				min=CONF_get_number(config,tmp_attr,buf);
+				min=(int)CONF_get_number(conf,tmp_attr,buf);
 
 				sprintf(buf,"%s_max",type);
-				max=CONF_get_number(config,tmp_attr,buf);
+				max=(int)CONF_get_number(conf,tmp_attr,buf);
 
 				if (!add_attribute_object(ri->attributes,
 					v->value,def,value,nid,min,max))
@@ -741,6 +772,8 @@ start2:			for (;;)
 	p=s;
 	i2d_RSAPublicKey(rsa,&p);
 	ri->pubkey->public_key->length=i;
+	if (ri->pubkey->public_key->data != NULL)
+		free(ri->pubkey->public_key->data);
 	ri->pubkey->public_key->data=s;
 
 	ret=1;
@@ -759,7 +792,7 @@ int max;
 	{
 	int i,j,z;
 	X509_NAME_ENTRY *ne=NULL;
-	static char buf[1024];
+	MS_STATIC char buf[1024];
 
 start:
 	fprintf(stderr,"%s [%s]:",text,def);
@@ -795,7 +828,6 @@ start:
 	buf[--i]='\0';
 
 	/* add object plus value */
-	j=0; /* set the set number */
 	if (sk_num(n) == 0)
 		j=0;
 	else
@@ -922,7 +954,7 @@ err:
 	return(0);
 	}
 
-static void cb(p, n)
+static void MS_CALLBACK req_cb(p, n)
 int p;
 int n;
 	{
