@@ -1,5 +1,5 @@
 /* apps/rsa.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,15 +47,17 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 #include "apps.h"
+#include "buffer.h"
 #include "err.h"
 #include "rsa.h"
-#include "buffer.h"
 #include "envelope.h"
 #include "x509.h"
 #include "pem.h"
 
+#undef PROG
 #define PROG	rsa_main
 
 /* -inform arg	- input format - default PEM (one of DER, NET or PEM)
@@ -76,10 +78,16 @@ char **argv;
 	RSA *rsa=NULL;
 	int i,badops=0;
 	EVP_CIPHER *enc=NULL;
-	FILE *in=NULL,*out=NULL;
+	BIO *in=NULL,*out=NULL;
 	int informat,outformat,text=0,noout=0;
 	char *infile,*outfile,*prog;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
 	infile=NULL;
 	outfile=NULL;
 	informat=FORMAT_PEM;
@@ -115,12 +123,12 @@ char **argv;
 		else if (strcmp(*argv,"-text") == 0)
 			text=1;
 		else if (strcmp(*argv,"-des") == 0)
-			enc=EVP_des_cbc;
+			enc=EVP_des_cbc();
 		else if (strcmp(*argv,"-des3") == 0)
-			enc=EVP_des_ede3_cbc;
-#ifndef NOIDEA
+			enc=EVP_des_ede3_cbc();
+#ifndef NO_IDEA
 		else if (strcmp(*argv,"-idea") == 0)
-			enc=EVP_idea_cbc;
+			enc=EVP_idea_cbc();
 #endif
 		else
 			{
@@ -143,7 +151,7 @@ bad:
 		fprintf(stderr," -out arg      output file\n");
 		fprintf(stderr," -des          encrypt PEM output with cbc des\n");
 		fprintf(stderr," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
-#ifndef NOIDEA
+#ifndef NO_IDEA
 		fprintf(stderr," -idea         encrypt PEM output with cbc idea\n");
 #endif
 		fprintf(stderr," -text         print the key in text\n");
@@ -153,12 +161,19 @@ bad:
 
 	ERR_load_crypto_strings();
 
+	in=BIO_new(BIO_s_file());
+	out=BIO_new(BIO_s_file());
+	if ((in == NULL) || (out == NULL))
+		{
+		ERR_print_errors(bio_err);
+		goto end;
+		}
+
 	if (infile == NULL)
-		in=stdin;
+		BIO_set_fp(in,stdin,BIO_NOCLOSE);
 	else
 		{
-		in=fopen(infile,"r");
-		if (in == NULL)
+		if (BIO_read_filename(in,infile) <= 0)
 			{
 			perror(infile);
 			goto end;
@@ -167,34 +182,36 @@ bad:
 
 	fprintf(stderr,"read RSA private key\n");
 	if	(informat == FORMAT_ASN1)
-		rsa=d2i_RSAPrivateKey_fp(in,NULL);
+		rsa=d2i_RSAPrivateKey_bio(in,NULL);
+#ifndef NO_RC4
 	else if (informat == FORMAT_NETSCAPE)
 		{
-		BUFFER *buf=NULL;
+		BUF_MEM *buf=NULL;
 		unsigned char *p;
 		int size=0,i;
 
-		buf=BUF_new();
+		buf=BUF_MEM_new();
 		for (;;)
 			{
-			if ((buf == NULL) || (!BUF_grow(buf,size+1024*10)))
+			if ((buf == NULL) || (!BUF_MEM_grow(buf,size+1024*10)))
 				goto end;
-			i=fread(&(buf->data[size]),1,1024*10,in);
+			i=BIO_read(in,&(buf->data[size]),1024*10);
 			size+=i;
 			if (i == 0) break;
 			if (i < 0)
 				{
 				perror("reading private key");
-				BUF_free(buf);
+				BUF_MEM_free(buf);
 				goto end;
 				}
 			}
 		p=(unsigned char *)buf->data;
 		rsa=(RSA *)d2i_Netscape_RSA(NULL,&p,(long)size,NULL);
-		BUF_free(buf);
+		BUF_MEM_free(buf);
 		}
+#endif
 	else if (informat == FORMAT_PEM)
-		rsa=PEM_read_RSAPrivateKey(in,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
 	else
 		{
 		fprintf(stderr,"bad input format specified for key\n");
@@ -203,28 +220,33 @@ bad:
 	if (rsa == NULL)
 		{
 		fprintf(stderr,"unable to load Private Key\n");
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		goto end;
 		}
 
 	if (outfile == NULL)
-		out=stdout;
+		BIO_set_fp(out,stdout,BIO_NOCLOSE);
 	else
 		{
-		out=fopen(outfile,"w");
-		if (out == NULL)
+		if (BIO_write_filename(out,outfile) <= 0)
 			{
 			perror(outfile);
 			goto end;
 			}
 		}
 
-	if (text) RSA_print(stdout,rsa);
+	if (text) 
+		if (!RSA_print(out,rsa,0))
+			{
+			perror(outfile);
+			goto end;
+			}
 
 	if (noout) goto end;
 	fprintf(stderr,"writing RSA private key\n");
 	if 	(outformat == FORMAT_ASN1)
-		i=i2d_RSAPrivateKey_fp(out,rsa);
+		i=i2d_RSAPrivateKey_bio(out,rsa);
+#ifndef NO_RC4
 	else if (outformat == FORMAT_NETSCAPE)
 		{
 		unsigned char *p,*pp;
@@ -239,11 +261,12 @@ bad:
 			}
 		pp=p;
 		i2d_Netscape_RSA(rsa,&p,NULL);
-		fwrite(pp,size,1,out);
+		BIO_write(out,(char *)pp,size);
 		free(pp);
 		}
+#endif
 	else if (outformat == FORMAT_PEM)
-		i=PEM_write_RSAPrivateKey(out,rsa,enc,NULL,0,NULL);
+		i=PEM_write_bio_RSAPrivateKey(out,rsa,enc,NULL,0,NULL);
 	else	{
 		fprintf(stderr,"bad output format specified for outfile\n");
 		goto end;
@@ -251,13 +274,14 @@ bad:
 	if (!i)
 		{
 		fprintf(stderr,"unable to write private key\n");
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		}
 	else
 		ret=0;
 end:
-	if ((in != NULL) && (in != stdin)) fclose(in);
-	if ((out != NULL) && (out != stdout)) fclose(out);
+	if (in != NULL) BIO_free(in);
+	if (out != NULL) BIO_free(out);
 	if (rsa != NULL) RSA_free(rsa);
 	EXIT(ret);
 	}
+
