@@ -1,5 +1,5 @@
 /* apps/s_filter.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,21 +47,20 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <string.h>
+#define USE_SOCKETS
 #include "apps.h"
-#include "s_net.h"
 #include "err.h"
 #include "x509.h"
 #include "ssl.h"
 #include "s_apps.h"
 
+#undef PROG
 #define PROG s_filter_main
 
 /*#define TEST_CERT "client.pem" */ /* no default cert. */
 
+#undef BUFSIZZ
 #define BUFSIZZ	10*1024
 
 extern int verify_depth;
@@ -91,8 +90,9 @@ typedef struct connection_st
 	int writefd[2];  /* fd for normal write */
 	} CON;
 
-SSL_CTX *ctx=NULL;
+SSL_CTX *f_ctx=NULL;
 
+#ifdef undef
 #ifndef NOPROTO
 static int setup_connections (CON *c, int argc, char **argv);
 static int process (CON *c);
@@ -128,7 +128,7 @@ CON *con;
 
 	for (i=0; i<2; i++)
 		{
-		con->ssl[i]=	(SSL *)SSL_new(ctx);
+		con->ssl[i]=	(SSL *)SSL_new(f_ctx);
 		con->host[i]=	NULL;
 		con->port[i]=	-1;
 		con->socket[i]=	-1;
@@ -137,6 +137,8 @@ CON *con;
 		}
 	}
 
+#endif /* undef */
+
 #ifdef undef
 static CON *glob_con;
 static int accept_sock;
@@ -146,14 +148,23 @@ int MAIN(argc, argv)
 int argc;
 char **argv;
 	{
+	EXIT(0);
+	}
+#ifdef undef
 	CON con;
 	char *cipher=NULL;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
 	verify_depth=0;
 	verify_error=VERIFY_OK;
 
-	ctx=SSL_CTX_new();
-	if (ctx == NULL) { EXIT(1); }
+	f_ctx=SSL_CTX_new();
+	if (f_ctx == NULL) { EXIT(1); }
 
 	zero_con(&con);
 
@@ -240,11 +251,11 @@ char **argv;
 		argv++;
 		}
 
-	if ((!SSL_load_verify_locations(ctx,con.CAfile,con.CApath)) ||
-		(!SSL_set_default_verify_paths(ctx)))
+	if ((!SSL_load_verify_locations(f_ctx,con.CAfile,con.CApath)) ||
+		(!SSL_set_default_verify_paths(f_ctx)))
 		{
 		fprintf(stderr,"error seting default verify locations\n");
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		EXIT(1);
 		}
 
@@ -258,9 +269,9 @@ char **argv;
 
 	
 	if (cipher == NULL)
-		SSL_CTX_set_cipher_list(ctx,getenv("SSL_CIPHER"));
+		SSL_CTX_set_cipher_list(f_ctx,getenv("SSL_CIPHER"));
 	else
-		SSL_CTX_set_cipher_list(ctx,cipher);
+		SSL_CTX_set_cipher_list(f_ctx,cipher);
 
 	if (con.ssl[0] != NULL)
 		if (!set_cert_stuff(con.ssl[0],con.cert_file,con.key_file))
@@ -310,8 +321,10 @@ char **argv;
 		c->socket[1]=-1;
 		if (argc > 0)
 			{
+#ifndef WINDOWS
 			i=spawn(argc,argv,&(c->readfd[1]),&(c->writefd[1]));
 			if (i <= 0)
+#endif
 				{
 				perror("spawn");
 				return(0);
@@ -346,9 +359,11 @@ char **argv;
 		c->socket[zero]= -1;
 		if (argc > 0)
 			{
+#ifndef WINDOWS
 			i=spawn(argc,argv,&(c->readfd[zero]),
 				&(c->writefd[zero]));
 			if (i <= 0)
+#endif
 				{
 				perror("spawn");
 				return(0);
@@ -496,6 +511,7 @@ CON *con;
 int port;
 int *ret;
 	{
+#ifndef WINDOWS
 	int sock;
 	char *name;
 	int accept_socket;
@@ -517,5 +533,9 @@ int *ret;
 		}
 	*ret=sock;
 	return(1);
+#else
+	return(0);
+#endif
 	}
 
+#endif /* undef */
