@@ -1,5 +1,5 @@
 /* apps/s_server.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,36 +48,35 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include "lhash.h"
-#include "s_net.h"
-#include "x509.h"
-#include "ssl.h"
+#define USE_SOCKETS
 #include "apps.h"
-#include "s_apps.h"
 #include "err.h"
 #include "pem.h"
+#include "x509.h"
+#include "ssl.h"
+#include "s_apps.h"
 
 #ifndef NOPROTO
-static int body(char *hostname, int s);
+static int sv_body(char *hostname, int s);
 static void close_accept_socket(void );
-static void usage(void);
+static void sv_usage(void);
 static int init_ssl_connection(SSL *s);
 static void print_stats(FILE *fp,SSL_CTX *ctx);
 #else
-static int body();
+static int sv_body();
 static void close_accept_socket();
-static void usage();
+static void sv_usage();
 static int init_ssl_connection();
 static void print_stats();
 #endif
 
-#define BUFSIZZ	10*1024
+#undef BUFSIZZ
+#define BUFSIZZ	8*1024
 static int accept_socket=-1;
 
 #define TEST_CERT	"server.pem"
+#undef PROG
 #define PROG		s_server_main
 
 extern int verify_depth;
@@ -85,14 +84,15 @@ extern int verify_error;
 
 static char *cipher=NULL;
 int verify=SSL_VERIFY_NONE;
-char *cert_file=TEST_CERT,*key_file=NULL;
+char *s_cert_file=TEST_CERT,*s_key_file=NULL;
 #ifdef FIONBIO
-static int nbio=0;
+static int s_nbio=0;
 #endif
 static SSL_CTX *ctx=NULL;
 
+static BIO *bio_s_out=NULL;
 
-static void usage()
+static void sv_usage()
 	{
 	fprintf(stderr,"usage: s_server [args ...]\n");
 	fprintf(stderr,"\n");
@@ -124,6 +124,15 @@ char *argv[];
 	int badop=0;
 	int ret=1;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+	if (bio_s_out == NULL)
+		if ((bio_s_out=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_s_out,stdout,BIO_NOCLOSE);
+
 	ctx=SSL_CTX_new();
 	if (ctx == NULL)
 		goto end;
@@ -131,7 +140,7 @@ char *argv[];
 	verify_depth=0;
 	verify_error=VERIFY_OK;
 #ifdef FIONBIO
-	nbio=0;
+	s_nbio=0;
 #endif
 
 	argc--;
@@ -162,12 +171,12 @@ char *argv[];
 		else if	(strcmp(*argv,"-cert") == 0)
 			{
 			if (--argc < 1) goto bad;
-			cert_file= *(++argv);
+			s_cert_file= *(++argv);
 			}
 		else if	(strcmp(*argv,"-key") == 0)
 			{
 			if (--argc < 1) goto bad;
-			key_file= *(++argv);
+			s_key_file= *(++argv);
 			}
 		else if	(strcmp(*argv,"-CApath") == 0)
 			{
@@ -186,7 +195,7 @@ char *argv[];
 			}
 #ifdef FIONBIO	
 		else if	(strcmp(*argv,"-nbio") == 0)
-			{ nbio=1; }
+			{ s_nbio=1; }
 #endif
 		else
 			{
@@ -200,13 +209,13 @@ char *argv[];
 	if (badop)
 		{
 bad:
-		usage();
+		sv_usage();
 		goto end;
 		}
 
 	if (cipher == NULL) cipher=getenv("SSL_CIPHER");
 
-	if (cert_file == NULL)
+	if (s_cert_file == NULL)
 		{
 		fprintf(stderr,"You must specify a certificate file for the server to use\n");
 		goto end;
@@ -218,17 +227,17 @@ bad:
 		(!SSL_set_default_verify_paths(ctx)))
 		{
 		fprintf(stderr,"X509_load_verify_locations\n");
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		goto end;
 		}
 
 	if (cipher != NULL)
 		SSL_CTX_set_cipher_list(ctx,cipher);
 	fprintf(stderr,"ACCEPT\n");
-	do_server(port,&accept_socket,body);
+	do_server(port,&accept_socket,sv_body);
+	print_stats(stderr,ctx);
 	ret=0;
 end:
-	print_stats(stderr,ctx);
 	if (ctx != NULL) SSL_CTX_free(ctx);
 	EXIT(ret);
 	}
@@ -252,21 +261,29 @@ SSL_CTX *ctx;
 	fprintf(fp,"%4ld session cache timeouts\n",SSL_CTX_sess_timeouts(ctx));
 	}
 
-static int body(hostname, s)
+static int sv_body(hostname, s)
 char *hostname;
 int s;
 	{
-	char buf[BUFSIZZ];
+	char *buf=NULL;
 	fd_set readfds;
 	int ret=1,width;
-	int l,k,i=1;
+	int k,i;
 	static SSL *con=NULL;
+	unsigned long l=1;
 
-#ifdef FIONBIO	
-	if (nbio)
+	if ((buf=malloc(BUFSIZZ)) == NULL)
 		{
+		fprintf(stderr,"out of memory\n");
+		goto err;
+		}
+#ifdef FIONBIO	
+	if (s_nbio)
+		{
+		unsigned long l=1;
+
 		fprintf(stderr,"turning on non blocking io\n");
-		ioctl(s,FIONBIO,&i);
+		socket_ioctl(s,FIONBIO,&l);
 		}
 #endif
 
@@ -275,7 +292,7 @@ int s;
 		{
 		con=(SSL *)SSL_new(ctx);
 		SSL_set_verify(con,verify,verify_callback);
-		if (!set_cert_stuff(con,cert_file,key_file))
+		if (!set_cert_stuff(con,s_cert_file,s_key_file))
 			{
 			ret=-1;
 			goto err;
@@ -289,7 +306,9 @@ int s;
 	for (;;)
 		{
 		FD_ZERO(&readfds);
+#ifndef WINDOWS
 		FD_SET(fileno(stdin),&readfds);
+#endif
 		FD_SET(s,&readfds);
 		select(width,&readfds,NULL,NULL,NULL);
 		if (FD_ISSET(fileno(stdin),&readfds))
@@ -314,7 +333,7 @@ int s;
 				k=SSL_write(con,&(buf[l]),(unsigned int)i);
 				if (
 #ifdef FIONBIO
-					nbio &&
+					s_nbio &&
 #endif
 					should_retry(k))
 					{
@@ -323,7 +342,7 @@ int s;
 					}
 				if (k <= 0)
 					{
-					ERR_print_errors(stderr);
+					ERR_print_errors(bio_err);
 					fprintf(stderr,"DONE\n");
 					ret=1;
 					goto err;
@@ -355,7 +374,7 @@ int s;
 				i=SSL_read(con,(char *)buf,BUFSIZZ);
 				if (
 #ifdef FIONBIO
-					nbio &&
+					s_nbio &&
 #endif
 					should_retry(i))
 					{
@@ -363,7 +382,7 @@ int s;
 					}
 				else if (i <= 0)
 					{
-					ERR_print_errors(stderr);
+					ERR_print_errors(bio_err);
 					fprintf(stderr,"DONE\n");
 					ret=1;
 					goto err;
@@ -377,6 +396,11 @@ int s;
 err:
 /*	if (con != NULL) SSL_free(con);  */
 	fprintf(stderr,"CONNECTION CLOSED\n");
+	if (buf != NULL)
+		{
+		memset(buf,0,BUFSIZZ);
+		free(buf);
+		}
 	if (ret >= 0)
 		fprintf(stderr,"ACCEPT\n");
 	return(ret);
@@ -411,17 +435,17 @@ SSL *con;
 				X509_cert_verify_error_string(verify_error));
 			}
 		else
-			ERR_print_errors(stderr);
+			ERR_print_errors(bio_err);
 		return(0);
 		}
 
-	PEM_write_SSL_SESSION(stdout,SSL_get_session(con));
+	PEM_write_bio_SSL_SESSION(bio_s_out,SSL_get_session(con));
 
 	peer=SSL_get_peer_certificate(con);
 	if (peer != NULL)
 		{
 		printf("Client certificate\n");
-		PEM_write_X509(stdout,peer);
+		PEM_write_bio_X509(bio_s_out,peer);
 		str=X509_NAME_oneline(X509_get_subject_name(peer));
 		printf("subject=%s\n",str);
 		free(str);
