@@ -1,8 +1,55 @@
+/* apps/s_time.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+ * All rights reserved.
+ * 
+ * This file is part of an SSL implementation written
+ * by Eric Young (eay@mincom.oz.au).
+ * The implementation was written so as to conform with Netscapes SSL
+ * specification.  This library and applications are
+ * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
+ * as long as the following conditions are aheared to.
+ * 
+ * Copyright remains Eric Young's, and as such any Copyright notices in
+ * the code are not to be removed.  If this code is used in a product,
+ * Eric Young should be given attribution as the author of the parts used.
+ * This can be in the form of a textual message at program startup or
+ * in documentation (online or textual) provided with the package.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ * 
+ * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * 
+ * The licence and distribution terms for any publically available version or
+ * derivative of this code cannot be changed.  i.e. this code cannot simply be
+ * copied and put under another distribution licence
+ * [including the GNU Public Licence.]
+ */
+
 /*-----------------------------------------
    cntime - SSL client connection timer program
    Written and donated by Larry Streepy <streepy@healthcare.com>
   -----------------------------------------*/
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -50,27 +97,22 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <string.h>
 
-#include "../crypto/crypto.h"
-#include "s_net.h"
 #include "x509.h"
 #include "ssl.h"
 #include "pem.h"
+#define USE_SOCKETS
 #include "apps.h"
 #include "s_apps.h"
+#include "err.h"
 #ifdef WIN32_STUFF
 #include "winmain.h"
 #include "wintext.h"
 #endif
 
+#ifndef MSDOS
 #define TIMES
-
-#ifdef WIN32
-#undef TIMES
 #endif
 
 #ifndef VMS
@@ -117,18 +159,22 @@ struct tms {
 #endif
 #endif
 
+#undef PROG
 #define PROG s_time_main
+
 #define ioctl ioctlsocket
 
 #define SSL_HOST_NAME	"localhost"
 
 /*#define TEST_CERT "client.pem" */ /* no default cert. */
 
+#undef BUFSIZZ
 #define BUFSIZZ 1024*10
 
 #define min(a,b) (((a) < (b)) ? (a) : (b))
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
+#undef SECONDS
 #define SECONDS	30
 extern int verify_depth;
 extern int verify_error;
@@ -150,8 +196,8 @@ static SSL *doConnection();
 
 static int port=PORT;
 static char *host=SSL_HOST_NAME;
-static char *cert_file=NULL;
-static char *key_file=NULL;
+static char *t_cert_file=NULL;
+static char *t_key_file=NULL;
 static char *CApath=NULL;
 static char *CAfile=NULL;
 static char *cipher=NULL;
@@ -174,7 +220,7 @@ static void usage()
 {
 	static char umsg[] = "\
 -time arg     - max number of seconds to collect data, default %d\n\
--verify arg   - turn on peer certificate verification\n\
+-verify arg   - turn on peer certificate verification, arg == depth\n\
 -cert arg     - certificate file to use, PEM format assumed\n\
 -key arg      - RSA file to use, PEM format assumed, in cert file if\n\
                 not specified but cert fill is.\n\
@@ -214,6 +260,12 @@ char **argv;
     nbio=0;
 #endif
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
     argc--;
     argv++;
 
@@ -234,17 +286,17 @@ char **argv;
 	    verify=SSL_VERIFY_PEER;
 	    if (--argc < 1) goto bad;
 	    verify_depth=atoi(*(++argv));
-	    Fprintf(stderr,"verify depth is %d\n",verify_depth);
+	    fprintf(stderr,"verify depth is %d\n",verify_depth);
 
 	} else if( strcmp(*argv,"-cert") == 0) {
 
 	    if (--argc < 1) goto bad;
-	    cert_file= *(++argv);
+	    t_cert_file= *(++argv);
 
 	} else if( strcmp(*argv,"-key") == 0) {
 
 	    if (--argc < 1) goto bad;
-	    key_file= *(++argv);
+	    t_key_file= *(++argv);
 
 	} else if( strcmp(*argv,"-CApath") == 0) {
 
@@ -272,7 +324,7 @@ char **argv;
 	    maxTime= atoi(*(++argv));
 	}
 	else {
-	    Fprintf(stderr,"unknown option %s\n",*argv);
+	    fprintf(stderr,"unknown option %s\n",*argv);
 	    badop=1;
 	    break;
 	}
@@ -339,7 +391,7 @@ char **argv;
 	double totalTime = 0.0;
 	int nConn = 0;
 	SSL *scon, *savecon;
-	int finishtime=0;
+	long finishtime=0;
 
 	if ((ctx=SSL_CTX_new()) == NULL) return(1);
 
@@ -352,8 +404,8 @@ char **argv;
 
 	if ((!SSL_load_verify_locations(ctx,CAfile,CApath)) ||
 		(!SSL_set_default_verify_paths(ctx))) {
-		Fprintf(stderr,"error seting default verify locations\n");
-		ERR_print_errors(stderr);
+		fprintf(stderr,"error seting default verify locations\n");
+		ERR_print_errors(bio_err);
 		EXIT(1);
 	}
 
@@ -361,7 +413,7 @@ char **argv;
 		cipher = getenv("SSL_CIPHER");
 
 	if( cipher == NULL ) {
-		Fprintf( stderr, "No CIPHER specified\n" );
+		fprintf( stderr, "No CIPHER specified\n" );
 /*		EXIT(1); */
 	}
 
@@ -369,7 +421,7 @@ char **argv;
 
 	/* Loop and time how long it takes to make connections */
 
-	finishtime=time(NULL)+SECONDS;
+	finishtime=(long)time(NULL)+SECONDS;
 	for (;;)
 		{
 		if (finishtime < time(NULL)) break;
@@ -394,7 +446,7 @@ char **argv;
 
 		totalTime += Time_F(STOP); /* Add the time for this iteration */
 		nConn += 1;
-		Fputc( '#', stdout ); fflush(stdout);
+		fputc( '#', stdout ); fflush(stdout);
 	}
 
 	printf( "\n\n%d connections in %.2fs; %.2f connections/user sec\n",
@@ -408,7 +460,7 @@ char **argv;
 
 	/* Get an SSL object so we can reuse the session id */
 	if( (savecon = doConnection( NULL )) == NULL ) {
-		Fprintf( stderr, "Unable to get connection\n" );
+		fprintf( stderr, "Unable to get connection\n" );
 		EXIT(1);
 	}
 	nConn = 0;
@@ -441,7 +493,7 @@ char **argv;
 	
 		totalTime += Time_F(STOP); /* Add the time for this iteration*/
 		nConn += 1;
-		Fputc( '#', stdout ); fflush(stdout);
+		fputc( '#', stdout ); fflush(stdout);
 	}
 
 	SSL_free( savecon );
@@ -464,7 +516,7 @@ char **argv;
 static SSL *
 doConnection(scon)
 SSL *scon;
-{
+	{
 	SSL *serverCon;
 	int s;
 	int width, i;
@@ -472,7 +524,7 @@ SSL *scon;
 
 	if(init_client(&s,host,port) == 0) {
 		perror( "couldn't connect" );
-		SHUTDOWN(s);
+		/* SHUTDOWN(s); */
 		return NULL;
 	}
 
@@ -485,42 +537,42 @@ SSL *scon;
 /*	SSL_set_verify(serverCon,verify,verify_callback);*/
 	SSL_CTX_set_cipher_list(ctx,cipher);
 
-	if(!set_cert_stuff(serverCon,cert_file,key_file)) {
+	if(!set_cert_stuff(serverCon,t_cert_file,t_key_file)) {
 		SHUTDOWN(SSL_get_fd(serverCon));
 		return NULL;
 	}
 
 	/* ok, lets connect */
-	for(;;) {
+	do	{
 		width=SSL_get_fd(serverCon)+1;
 		for(;;) {
 			i=SSL_connect(serverCon);
-			if (should_retry(i)) {
-				Fprintf(stderr,"DELAY\n");
+			if (should_retry(i))
+				{
+				fprintf(stderr,"DELAY\n");
 
 				FD_ZERO(&readfds);
 				FD_SET(SSL_get_fd(serverCon),&readfds);
 				select(width,&readfds,NULL,NULL,NULL);
 				continue;
-			}
+				}
 			break;
-		}
-		if(i <= 0) {
-			Fprintf(stderr,"ERROR\n");
-			if (verify_error != VERIFY_OK) {
-				Fprintf(stderr,"verify error:%s\n",
-						X509_cert_verify_error_string(verify_error));
-			} else {
-				ERR_print_errors(stderr);
 			}
+		if(i <= 0)
+			{
+			fprintf(stderr,"ERROR\n");
+			if (verify_error != VERIFY_OK)
+				fprintf(stderr,"verify error:%s\n",
+						X509_cert_verify_error_string(verify_error));
+			else
+				ERR_print_errors(bio_err);
 			return NULL;
-		}
-		break;
-	}
+			}
+		} while (0);
 													
 	SHUTDOWN(SSL_get_fd(serverCon));
 
 	return serverCon;
-}
+	}
 
 
