@@ -1,5 +1,5 @@
 /* apps/speed.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,25 +47,28 @@
 
 /* most of this code has been pilfered from my libdes speed.c program */
 
+#undef SECONDS
 #define SECONDS		3	
 #define RSA_SECONDS	10	
 
 /* 11-Sep-92 Andrew Daviel   Support for Silicon Graphics IRIX added */
 /* 06-Apr-92 Luke Brennan    Support for VMS and add extra signal calls */
 
-#ifndef MSDOS
-#define TIMES
-#endif
-
+#undef PROG
 #define PROG speed_main
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <signal.h>
 #include <string.h>
+#include "apps.h"
+#include "crypto.h"
+#include "err.h"
+
+#ifndef MSDOS
+#define TIMES
+#endif
+
 #ifndef VMS
 #ifndef _IRIX
 #include <time.h>
@@ -96,12 +99,17 @@ struct tms {
 #include "md2.h"
 #include "md5.h"
 #include "sha.h"
+#ifndef NO_RC4
 #include "rc4.h"
+#endif
+#ifndef NO_RC2
 #include "rc2.h"
+#endif
+#ifndef NO_IDEA
 #include "idea.h"
+#endif
 #include "rsa.h"
 #include "x509.h"
-#include "apps.h"
 #include "./testcert.h"
 
 /* The following if from times(3) man page.  It may need to be changed */
@@ -117,6 +125,7 @@ struct tms {
 #endif
 #endif
 
+#undef BUFSIZE
 #define BUFSIZE	((long)1024*8)
 long run=0;
 
@@ -173,7 +182,7 @@ int s;
 		{
 		times(&tend);
 		ret=((double)(tend.tms_utime-tstart.tms_utime))/HZ;
-		return((ret == 0.0)?1e-6:ret);
+		return((ret < 0.001)?0.001:ret);
 		}
 #else /* !times() */
 	static struct timeb tstart,tend;
@@ -189,17 +198,16 @@ int s;
 		ftime(&tend);
 		i=(long)tend.millitm-(long)tstart.millitm;
 		ret=((double)(tend.time-tstart.time))+((double)i)/1000.0;
-		return((ret == 0.0)?1e-6:ret);
+		return((ret < 0.001)?0.001:ret);
 		}
 #endif
 	}
 
-static unsigned char buf[BUFSIZE];
-
 int MAIN(argc,argv)
 int argc;
 char **argv;
 	{
+	unsigned char *buf=NULL;
 	int ret=1;
 #define ALGOR_NUM	12
 #define SIZE_NUM	5
@@ -209,12 +217,18 @@ char **argv;
 	unsigned char md2[MD2_DIGEST_LENGTH];
 	unsigned char md5[MD5_DIGEST_LENGTH];
 	unsigned char sha[SHA_DIGEST_LENGTH];
+#ifndef NO_RC4
 	RC4_KEY rc4_ks;
+#endif
+#ifndef NO_RC2
 	RC2_KEY rc2_ks;
+#endif
+#ifndef NO_IDEA
+	IDEA_KEY_SCHEDULE idea_ks;
+#endif
 	static unsigned char key16[16]=
 		{0x12,0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0,
 		 0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0,0x12};
-	IDEA_KEY_SCHEDULE idea_ks;
 	int cfb_state=0;
 	des_cblock iv;
 	static des_cblock key ={0x12,0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0};
@@ -237,7 +251,7 @@ char **argv;
 	static int lengths[SIZE_NUM]={8,64,256,1024,8192};
 	long c[ALGOR_NUM][SIZE_NUM];
 	static char *names[ALGOR_NUM]={
-		"md2","md5","sha","sha-1","rc4",
+		"md2","md5","sha","sha1","rc4",
 		"des cfb","des cbc","des ede3","idea cfb","idea cbc",
 		"rc2 cfb","rc2 cbc"};
 #define	R_RSA_512	0
@@ -258,12 +272,24 @@ char **argv;
 	int rsa_doit[RSA_NUM];
 	int pr_header=0;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
 	for (i=0; i<RSA_NUM; i++)
 		rsa_key[i]=NULL;
-#ifdef PURIFY
+
+	if ((buf=(unsigned char *)malloc(BUFSIZE)) == NULL)
+		{
+		fprintf(stderr,"out of memory\n");
+		goto end;
+		}
+
 	memset(c,0,sizeof(c));
 	memset(iv,0,sizeof(iv));
-#endif
+
 	for (i=0; i<ALGOR_NUM; i++)
 		doit[i]=0;
 	for (i=0; i<RSA_NUM; i++)
@@ -277,29 +303,35 @@ char **argv;
 		if	(strcmp(*argv,"md2") == 0) doit[D_MD2]=1;
 		else if (strcmp(*argv,"md5") == 0) doit[D_MD5]=1;
 		else if (strcmp(*argv,"sha") == 0) doit[D_SHA]=1;
-		else if (strcmp(*argv,"sha-1") == 0) doit[D_SHA1]=1;
+		else if (strcmp(*argv,"sha1") == 0) doit[D_SHA1]=1;
+#ifndef NO_RC4
 		else if (strcmp(*argv,"rc4") == 0) doit[D_RC4]=1;
+#endif
 		else if (strcmp(*argv,"des-cfb") == 0) doit[D_CFB_DES]=1;
 		else if (strcmp(*argv,"des-cbc") == 0) doit[D_CBC_DES]=1;
 		else if (strcmp(*argv,"des-ede3") == 0) doit[D_EDE3_DES]=1;
-		else if (strcmp(*argv,"idea-cfb") == 0) doit[D_CFB_IDEA]=1;
-		else if (strcmp(*argv,"idea-cbc") == 0) doit[D_CBC_IDEA]=1;
-		else if (strcmp(*argv,"rc2-cfb") == 0) doit[D_CFB_RC2]=1;
-		else if (strcmp(*argv,"rc2-cbc") == 0) doit[D_CBC_RC2]=1;
 		else if (strcmp(*argv,"rsa512") == 0) rsa_doit[R_RSA_512]=2;
 		else if (strcmp(*argv,"rsa1024") == 0) rsa_doit[R_RSA_1024]=2;
 		else if (strcmp(*argv,"rsa2048") == 0) rsa_doit[R_RSA_2048]=2;
 		else if (strcmp(*argv,"rsa4096") == 0) rsa_doit[R_RSA_4096]=2;
+#ifndef NO_RC2
+		else if (strcmp(*argv,"rc2-cfb") == 0) doit[D_CFB_RC2]=1;
+		else if (strcmp(*argv,"rc2-cbc") == 0) doit[D_CBC_RC2]=1;
 		else if (strcmp(*argv,"rc2") == 0)
 			{
 			doit[D_CFB_RC2]=1;
 			doit[D_CBC_RC2]=1;
 			}
+#endif
+#ifndef NO_IDEA
+		else if (strcmp(*argv,"idea-cfb") == 0) doit[D_CFB_IDEA]=1;
+		else if (strcmp(*argv,"idea-cbc") == 0) doit[D_CBC_IDEA]=1;
 		else if (strcmp(*argv,"idea") == 0)
 			{
 			doit[D_CFB_IDEA]=1;
 			doit[D_CBC_IDEA]=1;
 			}
+#endif
 		else if (strcmp(*argv,"des") == 0)
 			{
 			doit[D_CFB_DES]=1;
@@ -316,10 +348,21 @@ char **argv;
 		else
 			{
 			fprintf(stderr,"bad value, pick one of\n");
-			fprintf(stderr,"md2      md5      sha      sha-1\n");
-			fprintf(stderr,"idea-cfb idea-cbc rcs-cfb  rc2-cbc\n");
-			fprintf(stderr,"des-cfb  des-cbc  des-ede3 rc4\n");
-			fprintf(stderr,"rsa512   rsa1024  rsa2048  rsa4096\n");
+			fprintf(stderr,"md2      md5      sha      sha1\n");
+#ifndef NO_IDEA
+			fprintf(stderr,"idea-cfb idea-cbc ");
+#endif
+#ifndef NO_RC2
+			fprintf(stderr,"rc2-cfb  rc2-cbc");
+#endif
+#if !(defined(NO_IDEA) && defined(NO_RC2))
+			fprintf(stderr,"\n");
+#endif
+			fprintf(stderr,"des-cfb  des-cbc  des-ede3 ");
+#ifndef NO_RC4
+			fprintf(stderr,"rc4");
+#endif
+			fprintf(stderr,"\nrsa512   rsa1024  rsa2048  rsa4096\n");
 			fprintf(stderr,"idea     rc2      des      rsa\n");
 			goto end;
 			}
@@ -339,8 +382,8 @@ char **argv;
 		if (doit[i]) pr_header++;
 
 #ifndef TIMES
-	printf("To get the most acurate results, try to run this\n");
-	printf("program when this computer is idle.\n");
+	fprintf(stderr,"To get the most accurate results, try to run this\n");
+	fprintf(stderr,"program when this computer is idle.\n");
 #endif
 
 	for (i=0; i<RSA_NUM; i++)
@@ -356,12 +399,18 @@ char **argv;
 	des_set_key((C_Block *)key,sch);
 	des_set_key((C_Block *)key2,sch2);
 	des_set_key((C_Block *)key3,sch3);
+#ifndef NO_IDEA
 	idea_set_encrypt_key(key16,&idea_ks);
+#endif
+#ifndef NO_RC4
 	RC4_set_key(&rc4_ks,16,key16);
+#endif
+#ifndef NO_RC2
 	RC2_set_key(&rc2_ks,16,key16);
+#endif
 
 #ifndef SIGALRM
-	printf("First we calculate the aproximate speed ...\n");
+	fprintf(stderr,"First we calculate the approximate speed ...\n");
 	count=10;
 	do	{
 		long i;
@@ -382,8 +431,8 @@ char **argv;
 	c[D_EDE3_DES][0]=count/3;
 	c[D_CBC_IDEA][0]=count;
 	c[D_CFB_IDEA][0]=count;
-	c[D_CBC_RC2][0]=count*3;
-	c[D_CFB_RC2][0]=count*3;
+	c[D_CBC_RC2][0]=count;
+	c[D_CFB_RC2][0]=count;
 
 	for (i=1; i<SIZE_NUM; i++)
 		{
@@ -394,14 +443,18 @@ char **argv;
 		}
 	for (i=1; i<SIZE_NUM; i++)
 		{
-		c[D_RC4][i]=c[D_RC4][i-1]*lengths[i-1]/lengths[i];
-		c[D_CFB_DES][i]=c[D_CFB_DES][i-1]*lengths[i-1]/lengths[i];
-		c[D_CBC_DES][i]=c[D_CBC_DES][i-1]*lengths[i-1]/lengths[i];
-		c[D_EDE3_DES][i]=c[D_EDE3_DES][i-1]*lengths[i-1]/lengths[i];
-		c[D_CBC_IDEA][i]=c[D_CBC_IDEA][i-1]*lengths[i-1]/lengths[i];
-		c[D_CFB_IDEA][i]=c[D_CFB_IDEA][i-1]*lengths[i-1]/lengths[i];
-		c[D_CBC_RC2][i]=c[D_CBC_RC2][i-1]*lengths[i-1]/lengths[i];
-		c[D_CFB_RC2][i]=c[D_CFB_RC2][i-1]*lengths[i-1]/lengths[i];
+		long l0,l1;
+
+		l0=(long)lengths[i-1];
+		l1=(long)lengths[i];
+		c[D_RC4][i]=c[D_RC4][i-1]*l0/l1;
+		c[D_CFB_DES][i]=c[D_CFB_DES][i-1]*l0/l1;
+		c[D_CBC_DES][i]=c[D_CBC_DES][i-1]*l0/l1;
+		c[D_EDE3_DES][i]=c[D_EDE3_DES][i-1]*l0/l1;
+		c[D_CBC_IDEA][i]=c[D_CBC_IDEA][i-1]*l0/l1;
+		c[D_CFB_IDEA][i]=c[D_CFB_IDEA][i-1]*l0/l1;
+		c[D_CBC_RC2][i]=c[D_CBC_RC2][i-1]*l0/l1;
+		c[D_CFB_RC2][i]=c[D_CFB_RC2][i-1]*l0/l1;
 		}
 	rsa_c[R_RSA_512]=count/2000;
 	for (i=1; i<RSA_NUM; i++)
@@ -429,7 +482,8 @@ char **argv;
 			for (count=0,run=1; COND(c[D_MD2][j]); count++)
 				MD2(buf,(unsigned long)lengths[j],&(md2[0]));
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_MD2],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_MD2],d);
 			results[D_MD2][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -443,7 +497,8 @@ char **argv;
 			for (count=0,run=1; COND(c[D_MD5][j]); count++)
 				MD5(buf,(unsigned long)lengths[j],&(md5[0]));
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_MD5],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_MD5],d);
 			results[D_MD5][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -457,7 +512,8 @@ char **argv;
 			for (count=0,run=1; COND(c[D_SHA][j]); count++)
 				SHA(buf,(unsigned long)lengths[j],&(sha[0]));
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_SHA],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_SHA],d);
 			results[D_SHA][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -471,11 +527,13 @@ char **argv;
 			for (count=0,run=1; COND(c[D_SHA1][j]); count++)
 				SHA1(buf,(unsigned long)lengths[j],&(sha[0]));
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_SHA1],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_SHA1],d);
 			results[D_SHA1][j]=((double)count)/d*lengths[j];
 			}
 		}
 
+#ifndef NO_RC4
 	if (doit[D_RC4])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -486,10 +544,12 @@ char **argv;
 				RC4(&rc4_ks,(unsigned int)lengths[j],
 					buf,buf);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_RC4],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_RC4],d);
 			results[D_RC4][j]=((double)count)/d*lengths[j];
 			}
 		}
+#endif
 
 	if (doit[D_CFB_DES])
 		{
@@ -501,7 +561,8 @@ char **argv;
 				des_cfb64_encrypt(buf,buf,(long)lengths[j],sch,
 					(C_Block *)&(iv[0]),&cfb_state,DES_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CFB_DES],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CFB_DES],d);
 			results[D_CFB_DES][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -518,7 +579,8 @@ char **argv;
 					(long)lengths[j],sch,
 					(C_Block *)&(iv[0]),DES_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CBC_DES],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CBC_DES],d);
 			results[D_CBC_DES][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -535,11 +597,13 @@ char **argv;
 					(long)lengths[j],sch,sch2,sch3,
 					(C_Block *)&(iv[0]),DES_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_EDE3_DES],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_EDE3_DES],d);
 			results[D_EDE3_DES][j]=((double)count)/d*lengths[j];
 			}
 		}
 
+#ifndef NO_IDEA
 	if (doit[D_CFB_IDEA])
 		{
 		int v=0;
@@ -554,7 +618,8 @@ char **argv;
 					(unsigned char *)&(iv[0]),
 					&v,IDEA_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CFB_IDEA],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CFB_IDEA],d);
 			results[D_CFB_IDEA][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -570,11 +635,13 @@ char **argv;
 					(unsigned long)lengths[j],&idea_ks,
 					(unsigned char *)&(iv[0]),IDEA_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CBC_IDEA],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CBC_IDEA],d);
 			results[D_CBC_IDEA][j]=((double)count)/d*lengths[j];
 			}
 		}
-
+#endif
+#ifndef NO_RC2
 	if (doit[D_CFB_RC2])
 		{
 		int v=0;
@@ -589,7 +656,8 @@ char **argv;
 					(unsigned char *)&(iv[0]),
 					&v,RC2_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CFB_RC2],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CFB_RC2],d);
 			results[D_CFB_RC2][j]=((double)count)/d*lengths[j];
 			}
 		}
@@ -605,10 +673,12 @@ char **argv;
 					(unsigned long)lengths[j],&rc2_ks,
 					(unsigned char *)&(iv[0]),RC2_ENCRYPT);
 			d=Time_F(STOP);
-			printf("%ld %s's in %.2fs\n",count,names[D_CBC_RC2],d);
+			fprintf(stderr,"%ld %s's in %.2fs\n",
+				count,names[D_CBC_RC2],d);
 			results[D_CBC_RC2][j]=((double)count)/d*lengths[j];
 			}
 		}
+#endif
 
 	for (j=0; j<RSA_NUM; j++)
 		{
@@ -620,11 +690,13 @@ char **argv;
 			if (RSA_private_encrypt(16,buf,buf,rsa_key[j]) < 0)
 				{
 				fprintf(stderr,"RSA private encrypt failure\n");
+				ERR_print_errors(bio_err);
+				count=1;
 				break;
 				}
 			}
 		d=Time_F(STOP);
-		printf("%ld %d bit RSA's in %.2fs\n",
+		fprintf(stderr,"%ld %d bit RSA's in %.2fs\n",
 			count,rsa_bits[j],d);
 		rsa_results[j]=d/(double)count;
 		if (count <= 1)
@@ -635,39 +707,47 @@ char **argv;
 			}
 		}
 
+	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_VERSION));
+        fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_BUILT_ON));
+	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_OPTIONS));
+	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_CFLAGS));
+
 	if (pr_header)
 		{
-		fprintf(stderr,"type     ");
+		fprintf(stdout,"The 'numbers' are in 1000s of bytes per second processed.\n"); 
+		fprintf(stdout,"type     ");
 		for (j=0;  j<SIZE_NUM; j++)
-			fprintf(stderr,"%7d bytes",lengths[j]);
-		fprintf(stderr,"\n");
+			fprintf(stdout,"%7d bytes",lengths[j]);
+		fprintf(stdout,"\n");
 		}
+
 	for (k=0; k<ALGOR_NUM; k++)
 		{
 		if (!doit[k]) continue;
-		fprintf(stderr,"%-9s",names[k]);
+		fprintf(stdout,"%-9s",names[k]);
 		for (j=0; j<SIZE_NUM; j++)
 			{
 			if (results[k][j] > 10000)
-				fprintf(stderr," %11.2fk",results[k][j]/1000.0);
+				fprintf(stdout," %11.2fk",results[k][j]/1000.0);
 			else
-				fprintf(stderr," %11.2f ",results[k][j]);
+				fprintf(stdout," %11.2f ",results[k][j]);
 			}
-		fprintf(stderr,"\n");
+		fprintf(stdout,"\n");
 		}
 	for (k=0; k<RSA_NUM; k++)
 		{
 		if (!rsa_doit[k]) continue;
-		fprintf(stderr,"rsa %4d bits %7.3fs",
+		fprintf(stdout,"rsa %4d bits %7.3fs",
 			rsa_bits[k],rsa_results[k]);
-		fprintf(stderr,"\n");
+		fprintf(stdout,"\n");
 		}
 	ret=0;
 end:
+	if (buf != NULL) free(buf);
 	for (i=0; i<RSA_NUM; i++)
 		if (rsa_key[i] != NULL)
 			RSA_free(rsa_key[i]);
-	return(ret);
+	EXIT(ret);
 	}
 
 static void print_message(s,num,length)
@@ -676,12 +756,12 @@ long num;
 int length;
 	{
 #ifdef SIGALRM
-	printf("Doing %s for %ds on %d size blocks: ",s,SECONDS,length);
-	fflush(stdout);
+	fprintf(stderr,"Doing %s for %ds on %d size blocks: ",s,SECONDS,length);
+	fflush(stderr);
 	alarm(SECONDS);
 #else
-	printf("Doing %s %ld times on %d size blocks: ",s,num,length);
-	fflush(stdout);
+	fprintf(stderr,"Doing %s %ld times on %d size blocks: ",s,num,length);
+	fflush(stderr);
 #endif
 #ifdef LINT
 	num=num;
@@ -693,12 +773,12 @@ long num;
 int bits;
 	{
 #ifdef SIGALRM
-	printf("Doing %d bit rsa's for %ds: ",bits,RSA_SECONDS);
-	fflush(stdout);
+	fprintf(stderr,"Doing %d bit rsa's for %ds: ",bits,RSA_SECONDS);
+	fflush(stderr);
 	alarm(RSA_SECONDS);
 #else
-	printf("Doing %ld %d bit rsa's: ",num,bits);
-	fflush(stdout);
+	fprintf(stderr,"Doing %ld %d bit rsa's: ",num,bits);
+	fflush(stderr);
 #endif
 #ifdef LINT
 	num=num;
