@@ -1,5 +1,5 @@
 /* apps/x509.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,23 +47,22 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
 #include <string.h>
 #include "apps.h"
+#include "buffer.h"
 #include "asn1.h"
 #include "err.h"
 #include "bn.h"
 #include "rsa.h"
-#include "buffer.h"
 #include "envelope.h"
 #include "x509.h"
 #include "objects.h"
 #include "pem.h"
 
+#undef PROG
 #define PROG x509_main
 
+#undef POSTFIX
 #define	POSTFIX	".srl"
 #define DEF_DAYS	30
 
@@ -74,7 +73,7 @@
 
 #define CERT_HDR	"certificate"
 
-char *usage[]={
+static char *usage[]={
 "usage: x509 args\n",
 " -inform arg     - input format - default PEM (one of DER, NET or PEM)\n",
 " -outform arg    - output format - default PEM (one of DER, NET or PEM\n",
@@ -105,14 +104,14 @@ NULL
 };
 
 #ifndef NOPROTO
-static int callb(int ok, X509 *xs, X509 *xi, int depth, int error);
+static int MS_CALLBACK callb(int ok, X509 *xs, X509 *xi, int depth, int error);
 static RSA *load_key(char *file, int format);
 static X509 *load_cert(char *file, int format);
-static int sign (X509 *x, RSA *rsa,int days);
+static int sign (X509 *x, EVP_PKEY *pkey,int days);
 static int certify (CERTIFICATE_CTX *ctx,char *CAfile, X509 *x,
-	X509 *xca, RSA *rsa,char *serial, int create, int days);
+	X509 *xca, EVP_PKEY *pkey,char *serial, int create, int days);
 #else
-static int callb();
+static int MS_CALLBACK callb();
 static RSA *load_key();
 static X509 *load_cert();
 static int sign ();
@@ -128,7 +127,8 @@ char **argv;
 	X509 *x=NULL,*xca=NULL;
 	RSA *rsa=NULL,*CArsa=NULL;
 	int i,num,badops=0;
-	FILE *out=NULL;
+	BIO *out=NULL;
+	BIO *STDout=NULL;
 	int informat,outformat,keyformat,CAformat,CAkeyformat;
 	char *infile=NULL,*outfile=NULL,*keyfile=NULL,*CAfile=NULL;
 	char *CAkeyfile=NULL,*str=NULL,*CAserial=NULL;
@@ -140,6 +140,15 @@ char **argv;
 	X509_REQ *rq=NULL;
 	RSA *r=NULL;
 
+	apps_startup();
+
+	if (bio_err == NULL)
+		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+
+	if ((STDout=BIO_new(BIO_s_file())) != NULL)
+		BIO_set_fp(STDout,stdout,BIO_NOCLOSE);
+
 	informat=FORMAT_PEM;
 	outformat=FORMAT_PEM;
 	keyformat=FORMAT_PEM;
@@ -234,6 +243,11 @@ char **argv;
 			subject= ++num;
 		else if (strcmp(*argv,"-issuer") == 0)
 			issuer= ++num;
+		else if (strcmp(*argv,"-dates") == 0)
+			{
+			startdate= ++num;
+			enddate= ++num;
+			}
 		else if (strcmp(*argv,"-startdate") == 0)
 			startdate= ++num;
 		else if (strcmp(*argv,"-enddate") == 0)
@@ -264,7 +278,7 @@ bad:
 
 	if (!X509_set_default_verify_paths(ctx))
 		{
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		goto end;
 		}
 
@@ -278,30 +292,35 @@ bad:
 
 	if (reqfile)
 		{
-		RSA *pkey;
+		EVP_PKEY *pkey;
 		char *c;
 		X509_CINF *ci;
-		FILE *in;
-		BIGNUM *bn;
+		BIO *in;
 
 		if (!sign_flag && !CA_flag)
 			{
 			fprintf(stderr,"We need a private key to sign with\n");
 			goto end;
 			}
+		in=BIO_new(BIO_s_file());
+		if (in == NULL)
+			{
+			ERR_print_errors(bio_err);
+			goto end;
+			}
+
 		if (infile == NULL)
-			in=stdin;
+			BIO_set_fp(in,stdin,BIO_NOCLOSE);
 		else
 			{
-			if ((in=fopen(infile,"r")) == NULL)
+			if (BIO_read_filename(in,infile) <= 0)
 				{
 				perror(infile);
 				goto end;
 				}
 			}
-
-		req=PEM_read_X509_REQ(in,NULL,NULL);
-		if (in != stdin) fclose(in);
+		req=PEM_read_bio_X509_REQ(in,NULL,NULL);
+		BIO_free(in);
 
 		if (req == NULL) { perror(infile); goto end; }
 
@@ -320,7 +339,7 @@ bad:
 	                goto end;
 	                }
 		i=X509_REQ_verify(req,pkey);
-		RSA_free(pkey);
+		EVP_PKEY_free(pkey);
 	        if (!i)
 			{
 			fprintf(stderr,"Signature did not match the certificate request\n");
@@ -335,8 +354,6 @@ bad:
 		if ((x=X509_new()) == NULL) goto end;
 		ci=x->cert_info;
 
-		if ((bn=BN_new()) == NULL) goto end;
-
 		ASN1_INTEGER_set(ci->serialNumber,0);
 
 		X509_NAME_free(ci->issuer);
@@ -347,8 +364,8 @@ bad:
 	        ci->subject=req->req_info->subject;
 	        req->req_info->subject=NULL;;
 
-		X509_gmtime(ci->validity->notBefore,0);
-	        X509_gmtime(ci->validity->notAfter,60*60*24*days);
+		X509_gmtime_adj(ci->validity->notBefore,0);
+	        X509_gmtime_adj(ci->validity->notAfter,(long)60*60*24*days);
 
 		X509_PUBKEY_free(ci->key);
 		ci->key=req->req_info->pubkey;
@@ -364,6 +381,26 @@ bad:
 		if (xca == NULL) goto end;
 		}
 
+	if (!noout || text)
+		{
+		out=BIO_new(BIO_s_file());
+		if (out == NULL)
+			{
+			ERR_print_errors(bio_err);
+			goto end;
+			}
+		if (outfile == NULL)
+			BIO_set_fp(out,stdout,BIO_NOCLOSE);
+		else
+			{
+			if (BIO_write_filename(out,outfile) <= 0)
+				{
+				perror(outfile);
+				goto end;
+				}
+			}
+		}
+
 	if (num)
 		{
 		for (i=1; i<=num; i++)
@@ -374,7 +411,7 @@ bad:
 				if (str == NULL)
 					{
 					fprintf(stderr,"unable to get issuer Name from certificate\n");
-					ERR_print_errors(stderr);
+					ERR_print_errors(bio_err);
 					goto end;
 					}
 				fprintf(stdout,"issuer= %s\n",str);
@@ -387,7 +424,7 @@ bad:
 				if (str == NULL)
 					{
 					fprintf(stderr,"unable to get subject Name from certificate\n");
-					ERR_print_errors(stderr);
+					ERR_print_errors(bio_err);
 					goto end;
 					}
 				fprintf(stdout,"subject=%s\n",str);
@@ -396,7 +433,7 @@ bad:
 			if (serial == i)
 				{
 				fprintf(stdout,"serial ");
-				i2f_ASN1_INTEGER(stdout,x->cert_info->serialNumber);
+				i2a_ASN1_INTEGER(STDout,x->cert_info->serialNumber);
 				}
 			if (hash == i)
 				{
@@ -405,44 +442,58 @@ bad:
 				}
 			if (text == i)
 				{
-				X509_print(stdout,x);
+				X509_print(out,x);
 				}
 			if (startdate == i)
 				{
-				fprintf(stdout,"notBefore=%s\n",
+				BIO_puts(STDout,"notBefore=");
+				ASN1_UTCTIME_print(STDout,
 					x->cert_info->validity->notBefore);
+				BIO_puts(STDout,"\n");
 				}
 			if (enddate == i)
 				{
-				fprintf(stdout,"notAfter =%s\n",
+				BIO_puts(STDout,"notAfter=");
+				ASN1_UTCTIME_print(STDout,
 					x->cert_info->validity->notAfter);
+				BIO_puts(STDout,"\n");
 				}
 
 			/* should be in the library */
 			if (sign_flag == i)
 				{
+				EVP_PKEY pkey;
+
 				fprintf(stderr,"Getting Private key\n");
 				if (rsa == NULL)
 					{
 					rsa=load_key(keyfile,keyformat);
 					if (rsa == NULL) goto end;
 					}
-				if (!sign(x,rsa,days)) goto end;
+				pkey.type=EVP_PKEY_RSA;
+				pkey.pkey.rsa=rsa;
+				if (!sign(x,&pkey,days)) goto end;
 				}
 			if (CA_flag == i)
 				{
+				EVP_PKEY pkey;
+
 				fprintf(stderr,"Getting CA Private Key\n");
 				if (CAkeyfile != NULL)
 					{
 					CArsa=load_key(CAkeyfile,CAkeyformat);
 					if (CArsa == NULL) goto end;
 					}
-				if (!certify(ctx,CAfile,x,xca,CArsa,CAserial,
+				pkey.type=EVP_PKEY_RSA;
+				pkey.pkey.rsa=CArsa;
+				if (!certify(ctx,CAfile,x,xca,&pkey,CAserial,
 					CA_createserial,days))
 					goto end;
 				}
 			if (x509req == i)
 				{
+				EVP_PKEY pkey;
+
 				fprintf(stderr,"Getting request Private Key\n");
 				if (keyfile == NULL)
 					{
@@ -457,27 +508,18 @@ bad:
 
 				fprintf(stderr,"Generating certificate request\n");
 
-				rq=(X509_REQ *)X509_X509_TO_req(x,r);
+				pkey.type=EVP_PKEY_RSA;
+				pkey.pkey.rsa=r;
+				rq=(X509_REQ *)X509_to_X509_REQ(x,&pkey);
 				if (rq == NULL)
 					{
-					ERR_print_errors(stderr);
+					ERR_print_errors(bio_err);
 					goto end;
 					}
 				if (!noout)
 					{
-					if (outfile == NULL)
-						out=stdout;
-					else
-						{
-						out=fopen(outfile,"w");
-						if (out == NULL)
-							{
-							perror(outfile);
-							goto end;
-							}
-						}
 					X509_REQ_print(out,rq);
-					PEM_write_X509_REQ(out,rq);
+					PEM_write_bio_X509_REQ(out,rq);
 					}
 				noout=1;
 				}
@@ -490,18 +532,10 @@ bad:
 		goto end;
 		}
 
-	if (outfile == NULL)
-		out=stdout;
-	else
-		{
-		out=fopen(outfile,"w");
-		if (out == NULL) { perror(outfile); goto end; }
-		}
-
 	if 	(outformat == FORMAT_ASN1)
-		i=i2d_X509_fp(out,x);
+		i=i2d_X509_bio(out,x);
 	else if (outformat == FORMAT_PEM)
-		i=PEM_write_X509(out,x);
+		i=PEM_write_bio_X509(out,x);
 	else if (outformat == FORMAT_NETSCAPE)
 		{
 		ASN1_HEADER ah;
@@ -511,10 +545,10 @@ bad:
 		os.length=strlen(CERT_HDR);
 		ah.header= &os;
 		ah.data=(char *)x;
-		ah.meth=ASN1_X509_meth;
+		ah.meth=ASN1_X509_meth();
 
 		/* no macro for this one yet */
-		i=ASN1_i2d_fp(i2d_ASN1_HEADER,out,(unsigned char *)&ah);
+		i=ASN1_i2d_bio(i2d_ASN1_HEADER,out,(unsigned char *)&ah);
 		}
 	else	{
 		fprintf(stderr,"bad output format specified for outfile\n");
@@ -522,12 +556,13 @@ bad:
 		}
 	if (!i) {
 		fprintf(stderr,"unable to write certificate\n");
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		goto end;
 		}
 	ret=0;
 end:
-	if ((out != NULL) && (out != stdout)) fclose(out);
+	if (out != NULL) BIO_free(out);
+	if (STDout != NULL) BIO_free(STDout);
 	if (ctx != NULL) CERTIFICATE_CTX_free(ctx);
 	if (req != NULL) X509_REQ_free(req);
 	if (x != NULL) X509_free(x);
@@ -539,26 +574,27 @@ end:
 	EXIT(ret);
 	}
 
-static int certify(ctx,CAfile, x, xca, rsa, serialfile, create,days)
+static int certify(ctx,CAfile, x, xca, pkey, serialfile, create,days)
 CERTIFICATE_CTX *ctx;
 char *CAfile;
 X509 *x;
 X509 *xca;
-RSA *rsa;
+EVP_PKEY *pkey;
 char *serialfile;
 int create;
 int days;
 	{
-	FILE *io;
-	char *buf,buf2[1024];
-	BIGNUM *serial;
-	ASN1_INTEGER *bs,bs2;
+	int ret=0;
+	BIO *io=NULL;
+	MS_STATIC char *buf=NULL,buf2[1024];
+	BIGNUM *serial=NULL;
+	ASN1_INTEGER *bs=NULL,bs2;
 
-	buf=(char *)malloc(RSA_size(rsa)*2+
+	buf=(char *)malloc(EVP_PKEY_size(pkey)*2+
 		((serialfile == NULL)
 			?(strlen(CAfile)+strlen(POSTFIX)+1)
 			:(strlen(serialfile)))+1);
-	if (buf == NULL) { fprintf(stderr,"out of mem\n"); return(0); }
+	if (buf == NULL) { fprintf(stderr,"out of mem\n"); goto end; }
 	if (serialfile == NULL)
 		{
 		strcpy(buf,CAfile);
@@ -570,17 +606,23 @@ int days;
 	bs=ASN1_INTEGER_new();
 	if ((serial == NULL) || (bs == NULL))
 		{
-		ERR_print_errors(stderr);
-		return(0);
+		ERR_print_errors(bio_err);
+		goto end;
 		}
 
-	io=fopen(buf,"r");
+	io=BIO_new(BIO_s_file());
 	if (io == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto end;
+		}
+	
+	if (BIO_read_filename(io,buf) <= 0)
 		{
 		if (!create)
 			{
 			perror(buf);
-			return(0);
+			goto end;
 			}
 		else
 			{
@@ -590,12 +632,11 @@ int days;
 		}
 	else 
 		{
-		if (!f2i_ASN1_INTEGER(io,bs,buf2,1024))
+		if (!a2i_ASN1_INTEGER(io,bs,buf2,1024))
 			{
 			fprintf(stderr,"unable to load serial number from %s\n",buf);
-			ERR_print_errors(stderr);
-			fclose(io);
-			return(0);
+			ERR_print_errors(bio_err);
+			goto end;
 			}
 		else
 			{
@@ -603,61 +644,69 @@ int days;
 			if (serial == NULL)
 				{
 				fprintf(stderr,"error converting bin 2 bn");
-				fclose(io);
-				return(0);
+				goto end;
 				}
 			}
-		fclose(io);
 		}
 
 	if (!BN_add_word(serial,1))
-		{ fprintf(stderr,"add_word failure\n"); return(0); }
+		{ fprintf(stderr,"add_word failure\n"); goto end; }
 	bs2.data=(unsigned char *)buf2;
 	bs2.length=BN_bn2bin(serial,bs2.data);
 
-	io=fopen(buf,"w");
-	if (io == NULL)
+	if (BIO_write_filename(io,buf) <= 0)
 		{
 		fprintf(stderr,"error attempting to write serial number file\n");
 		perror(buf);
-		return(0);
+		goto end;
 		}
-	i2f_ASN1_INTEGER(io,&bs2);
-	fprintf(io,"\n");
-	fclose(io);
+	i2a_ASN1_INTEGER(io,&bs2);
+	BIO_puts(io,"\n");
+	BIO_free(io);
+	io=NULL;
 	
-	if (!X509_add_cert(ctx,x)) return(0);
+	if (!X509_add_cert(ctx,x)) goto end;
 	/* Must up the references since X509_add_cert thinks it can keep
 	 * the cert for it's very own :-) */
 	x->references++;
 	/* NOTE: this certificate can/should be self signed */
-	if (!X509_cert_verify(ctx,x,callb)) return(0);
+	if (!X509_cert_verify(ctx,x,callb))
+		goto end;
 
 	X509_NAME_free(x->cert_info->issuer);
 	x->cert_info->issuer=X509_NAME_dup(xca->cert_info->subject);
 	if (x->cert_info->validity->notBefore != NULL)
 		free(x->cert_info->validity->notBefore);
-	x->cert_info->validity->notBefore=(char *)malloc(100);
+	if ((x->cert_info->validity->notBefore=(char *)malloc(100)) == NULL)
+		goto end;
+	ASN1_INTEGER_free(x->cert_info->serialNumber);
 	x->cert_info->serialNumber=bs;
+	bs=NULL;
 
 	if (x->cert_info->validity->notBefore == NULL)
 		{
 		fprintf(stderr,"out of mem\n");
-		return(0);
+		goto end;
 		}
 
-	X509_gmtime(x->cert_info->validity->notBefore,0);
+	X509_gmtime_adj(x->cert_info->validity->notBefore,0);
 	/* hardwired expired */
-	X509_gmtime(x->cert_info->validity->notAfter,60*60*24*days);
-	if (!X509_sign(x,rsa,EVP_md5))
+	X509_gmtime_adj(x->cert_info->validity->notAfter,(long)60*60*24*days);
+	if (!X509_sign(x,pkey,EVP_md5()))
 		{
-		ERR_print_errors(stderr);
-		return(0);
+		ERR_print_errors(bio_err);
+		goto end;
 		}
-	return(1);
+	ret=1;
+end:
+	if (buf != NULL) free(buf);
+	if (bs != NULL) ASN1_INTEGER_free(bs);
+	if (io != NULL)	BIO_free(io);
+	if (serial != NULL) BN_free(serial);
+	return(ret);
 	}
 
-static int callb(ok, xs, xi, depth, error)
+static int MS_CALLBACK callb(ok, xs, xi, depth, error)
 int ok;
 X509 *xs;
 X509 *xi;
@@ -691,31 +740,38 @@ static RSA *load_key(file, format)
 char *file;
 int format;
 	{
-	FILE *key;
-	RSA *rsa;
+	BIO *key=NULL;
+	RSA *rsa=NULL;
 
 	if (file == NULL)
 		{
 		fprintf(stderr,"no keyfile specified\n");
-		return(NULL);
+		goto end;
+		}
+	key=BIO_new(BIO_s_file());
+	if (key == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto end;
+		}
+	if (BIO_read_filename(key,file) <= 0)
+		{
+		perror(file);
+		goto end;
 		}
-	key=fopen(file,"r");
-	if (key == NULL) { perror(file); return(NULL); }
 	if	(format == FORMAT_ASN1)
-		rsa=d2i_RSAPrivateKey_fp(key,NULL);
+		rsa=d2i_RSAPrivateKey_bio(key,NULL);
 	else if (format == FORMAT_PEM)
-		rsa=PEM_read_RSAPrivateKey(key,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(key,NULL,NULL);
 	else
 		{
 		fprintf(stderr,"bad input format specified for key\n");
-		return(NULL);
+		goto end;
 		}
-	fclose(key);
+end:
+	if (key != NULL) BIO_free(key);
 	if (rsa == NULL)
-		{
 		fprintf(stderr,"unable to load Private Key\n");
-		return(NULL);
-		}
 	return(rsa);
 	}
 
@@ -724,43 +780,48 @@ char *file;
 int format;
 	{
 	ASN1_HEADER *ah=NULL;
+	BUF_MEM *buf=NULL;
 	X509 *x=NULL;
-	FILE *cert;
+	BIO *cert;
+
+	if ((cert=BIO_new(BIO_s_file())) == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto end;
+		}
 
 	if (file == NULL)
-		cert=stdin;
+		BIO_set_fp(cert,stdin,BIO_NOCLOSE);
 	else
 		{
-		cert=fopen(file,"r");
-		if (cert == NULL) { perror(file); return(NULL); }
+		if (BIO_read_filename(cert,file) <= 0)
+			{
+			perror(file);
+			goto end;
+			}
 		}
 	if 	(format == FORMAT_ASN1)
-		x=d2i_X509_fp(cert,NULL);
+		x=d2i_X509_bio(cert,NULL);
 	else if (format == FORMAT_NETSCAPE)
 		{
-		BUFFER *buf=NULL;
 		unsigned char *p,*op;
 		int size=0,i;
 
 		/* We sort of have to do it this way because it is sort of nice
 		 * to read the header first and check it, then
 		 * try to read the certificate */
-		buf=BUF_new();
+		buf=BUF_MEM_new();
 		for (;;)
 			{
-			if ((buf == NULL) || (!BUF_grow(buf,size+1024*10)))
-				{
-				BUF_free(buf);
-				return(NULL);
-				}
-			i=fread(&(buf->data[size]),1,1024*10,cert);
+			if ((buf == NULL) || (!BUF_MEM_grow(buf,size+1024*10)))
+				goto end;
+			i=BIO_read(cert,&(buf->data[size]),1024*10);
 			size+=i;
 			if (i == 0) break;
 			if (i < 0)
 				{
 				perror("reading certificate");
-				BUF_free(buf);
-				return(NULL);
+				goto end;
 				}
 			}
 		p=(unsigned char *)buf->data;
@@ -778,35 +839,33 @@ int format;
 			}
 		/* header is ok, so now read the object */
 		p=op;
-		ah->meth=ASN1_X509_meth;
+		ah->meth=ASN1_X509_meth();
 		if ((ah=d2i_ASN1_HEADER(&ah,&p,(long)size)) == NULL)
 			goto end;
 		x=(X509 *)ah->data;
 		ah->data=NULL;
-
-		BUF_free(buf);
 		}
 	else if (format == FORMAT_PEM)
-		x=PEM_read_X509(cert,NULL,NULL);
+		x=PEM_read_bio_X509(cert,NULL,NULL);
 	else	{
 		fprintf(stderr,"bad input format specified for input cert\n");
-		return(NULL);
+		goto end;
 		}
 end:
 	if (x == NULL)
 		{
 		fprintf(stderr,"unable to load certificate\n");
-		ERR_print_errors(stderr);
-		return(NULL);
+		ERR_print_errors(bio_err);
 		}
 	if (ah != NULL) ASN1_HEADER_free(ah);
-	if (cert != stdin) fclose(cert);
+	if (cert != NULL) BIO_free(cert);
+	if (buf != NULL) BUF_MEM_free(buf);
 	return(x);
 	}
 
-static int sign(x, rsa, days)
+static int sign(x, pkey, days)
 X509 *x;
-RSA *rsa;
+EVP_PKEY *pkey;
 int days;
 	{
 	int j;
@@ -814,23 +873,28 @@ int days;
 
 	X509_NAME_free(x->cert_info->issuer);
 	x->cert_info->issuer=X509_NAME_dup(x->cert_info->subject);
-	X509_gmtime(x->cert_info->validity->notBefore,0);
+	X509_gmtime_adj(x->cert_info->validity->notBefore,0);
 	/* Lets just make it 12:00am GMT, Jan 1 1970 */
 	memcpy(x->cert_info->validity->notBefore,"700101120000Z",13);
 	/* 28 days to be certified */
-	X509_gmtime(x->cert_info->validity->notAfter,60*60*24*days);
+	X509_gmtime_adj(x->cert_info->validity->notAfter,(long)60*60*24*days);
 
-	j=i2d_RSAPublicKey(rsa,NULL);
+	j=i2d_PublicKey(pkey,NULL);
+	if (x->cert_info->key->public_key->data != NULL)
+		free(x->cert_info->key->public_key->data);
 	p=x->cert_info->key->public_key->data=(unsigned char *)malloc(
 		(unsigned int)j+10);
 	if (p == NULL) { fprintf(stderr,"out of memory\n"); return(0); }
 	x->cert_info->key->public_key->length=j;
-	if (!i2d_RSAPublicKey(rsa,&p))
-                { ERR_print_errors(stderr); return(0); }
+	if (!i2d_PublicKey(pkey,&p))
+                {
+		ERR_print_errors(bio_err);
+		return(0);
+		}
 
-	if (!X509_sign(x,rsa,EVP_md5))
+	if (!X509_sign(x,pkey,EVP_md5()))
 		{
-		ERR_print_errors(stderr);
+		ERR_print_errors(bio_err);
 		return(0);
 		}
 	return(1);
