@@ -1,5 +1,5 @@
-/* lib/asn1/a_print.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/asn1/a_print.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,7 +46,7 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "asn1.h"
 
 /* ASN1err(ASN1_F_D2I_ASN1_PRINT_TYPE,ASN1_R_WRONG_PRINTABLE_TYPE);
@@ -107,7 +107,7 @@ long length;
 int type;
 	{
 	ASN1_BIT_STRING *ret=NULL;
-	unsigned char *p,*s,*max;
+	unsigned char *p,*s;
 	long len;
 	int inf,tag,xclass;
 	int i=0;
@@ -120,7 +120,6 @@ int type;
 		ret=(*a);
 
 	p= *pp;
-	max=(p+length);
 	inf=ASN1_get_object(&p,&len,&tag,&xclass,length);
 	if (inf == 0xff) goto err;
 
@@ -143,16 +142,17 @@ int type;
 	if (!(tag2bit[tag] & type))
 		{
 		i=ASN1_R_WRONG_PRINTABLE_TYPE;
+		goto err;
 		}
 	if (len != 0)
 		{
-		s=(unsigned char *)malloc(len+1);
+		s=(unsigned char *)malloc((int)len+1);
 		if (s == NULL)
 			{
 			i=ERR_R_MALLOC_FAILURE;
 			goto err;
 			}
-		memcpy(s,p,len);
+		memcpy(s,p,(int)len);
 		s[len]='\0';
 		p+=len;
 		}
@@ -160,7 +160,7 @@ int type;
 		s=NULL;
 
 	if (ret->data != NULL) free(ret->data);
-	ret->length=len;
+	ret->length=(int)len;
 	ret->data=s;
 	ret->type=tag;
 	if (a != NULL) (*a)=ret;
