@@ -1,5 +1,5 @@
-/* lib/asn1/asn1_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/asn1/asn1_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,7 +46,7 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "asn1.h"
 
 #ifndef NOPROTO
@@ -57,7 +57,7 @@ static int asn1_get_length();
 static void asn1_put_length();
 #endif
 
-char *ASN1_version="ASN1 part of SSLeay 0.5.2 29-Apr-1996";
+char *ASN1_version="ASN1 part of SSLeay 0.6.0 21-Jun-1996";
 
 int ASN1_check_infinite_end(p,len)
 unsigned char **p;
@@ -114,7 +114,7 @@ long omax;
 		}
 	*ptag=tag;
 	*pclass=xclass;
-	if (!asn1_get_length(&p,&inf,plength,max)) goto err;
+	if (!asn1_get_length(&p,&inf,plength,(int)max)) goto err;
 	if ((p+ *plength) > (omax+ *pp))
 		{
 		ASN1err(ASN1_F_ASN1_GET_OBJECT,ASN1_R_TOO_LONG);
