@@ -1,5 +1,5 @@
-/* lib/asn1/asn1_par.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/asn1/asn1_par.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,110 +46,116 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
+#include "buffer.h"
 #include "objects.h"
 #include "x509.h"
 
 #ifndef NOPROTO
-static void asn1_print_info(FILE *file, int tag, int xclass,int constructed);
-static int asn1_parse2(FILE *fp, unsigned char **pp, long length,
+static int asn1_print_info(BIO *bp, int tag, int xclass,int constructed);
+static int asn1_parse2(BIO *bp, unsigned char **pp, long length,
 	int offset, int depth);
 #else
-static void asn1_print_info();
+static int asn1_print_info();
 static int asn1_parse2();
 #endif
 
-static void asn1_print_info(file, tag, xclass, constructed)
-FILE *file;
+static int asn1_print_info(bp, tag, xclass, constructed)
+BIO *bp;
 int tag;
 int xclass;
 int constructed;
 	{
 	static char *fmt="%-18s";
 	static char *fmt2="%2d %-15s";
+	char *p,str[128],*p2=NULL;
 
 	if (constructed & V_ASN1_CONSTRUCTED)
-		Fprintf(file,"cons: ");
+		p="cons: ";
 	else
-		Fprintf(file,"prim: ");
+		p="prim: ";
+	if (BIO_puts(bp,p) <= 0) goto err;
+	p=str;
 	if ((xclass & V_ASN1_PRIVATE) == V_ASN1_PRIVATE)
-		{
-		Fprintf(file,"priv [ %d ] ",tag);
-		return;
-		}
+		sprintf(str,"priv [ %d ] ",tag);
 	else if ((xclass & V_ASN1_CONTEXT_SPECIFIC) == V_ASN1_CONTEXT_SPECIFIC)
-		{
-		Fprintf(file,"cont [ %d ]",tag);
-		return;
-		}
+		sprintf(str,"cont [ %d ]",tag);
 	else if ((xclass & V_ASN1_APPLICATION) == V_ASN1_APPLICATION)
-		{
-		Fprintf(file,"appl [ %d ]",tag);
-		return;
-		}
+		sprintf(str,"appl [ %d ]",tag);
 	else if ((tag == V_ASN1_EOC) /* && (xclass == V_ASN1_UNIVERSAL) */)
-		Fprintf(file,fmt,"EOC");
+		p="EOC";
 	else if (tag == V_ASN1_INTEGER)
-		Fprintf(file,fmt,"INTEGER");
+		p="INTEGER";
 	else if (tag == V_ASN1_BIT_STRING)
-		Fprintf(file,fmt,"BIT STRING");
+		p="BIT STRING";
 	else if (tag == V_ASN1_OCTET_STRING)
-		Fprintf(file,fmt,"OCTET STRING");
+		p="OCTET STRING";
 	else if (tag == V_ASN1_NULL)
-		Fprintf(file,fmt,"NULL");
+		p="NULL";
 	else if (tag == V_ASN1_OBJECT)
-		Fprintf(file,fmt,"OBJECT");
+		p="OBJECT";
 	else if (tag == V_ASN1_SEQUENCE)
-		Fprintf(file,fmt,"SEQUENCE");
+		p="SEQUENCE";
 	else if (tag == V_ASN1_SET)
-		Fprintf(file,fmt,"SET");
+		p="SET";
 	else if (tag == V_ASN1_PRINTABLESTRING)
-		Fprintf(file,fmt,"PRINTABLESTRING");
+		p="PRINTABLESTRING";
 	else if (tag == V_ASN1_T61STRING)
-		Fprintf(file,fmt,"T61STRING");
+		p="T61STRING";
 	else if (tag == V_ASN1_IA5STRING)
-		Fprintf(file,fmt,"IA5STRING");
+		p="IA5STRING";
 	else if (tag == V_ASN1_UTCTIME)
-		Fprintf(file,fmt,"UTCTIME");
+		p="UTCTIME";
 
 	/* extras */
 	else if (tag == V_ASN1_NUMERICSTRING)
-		Fprintf(file,fmt,"NUMERICSTRING");
+		p="NUMERICSTRING";
 	else if (tag == V_ASN1_VIDEOTEXSTRING)
-		Fprintf(file,fmt,"VIDEOTEXSTRING");
+		p="VIDEOTEXSTRING";
 	else if (tag == V_ASN1_GENERALIZEDTIME)
-		Fprintf(file,fmt,"GENERALIZEDTIME");
+		p="GENERALIZEDTIME";
 	else if (tag == V_ASN1_GRAPHICSTRING)
-		Fprintf(file,fmt,"GRAPHICSTRING");
+		p="GRAPHICSTRING";
 	else if (tag == V_ASN1_ISO64STRING)
-		Fprintf(file,fmt,"ISO64STRING");
+		p="ISO64STRING";
 	else if (tag == V_ASN1_GENERALSTRING)
-		Fprintf(file,fmt,"GENERALSTRING");
+		p="GENERALSTRING";
 	else if (tag == V_ASN1_UNIVERSALSTRING)
-		Fprintf(file,fmt,"UNIVERSALSTRING");
+		p="UNIVERSALSTRING";
 
-	else	Fprintf(file,fmt2,tag,"(unknown)");
+	else
+		p2="(unknown)";
+		
+	if (p2 != NULL)
+		sprintf(str,fmt2,tag,p2);
+	else
+		sprintf(str,fmt,p);
+	if (BIO_puts(bp,str) <= 0) goto err;
+	return(1);
+err:
+	return(0);
 	}
 
-void ASN1_parse(fp, pp, len)
-FILE *fp;
+int ASN1_parse(bp, pp, len)
+BIO *bp;
 unsigned char *pp;
 long len;
 	{
-	asn1_parse2(fp,&pp,len,0,0);
+	return(asn1_parse2(bp,&pp,len,0,0));
 	}
 
-static int asn1_parse2(fp, pp, length, offset, depth)
-FILE *fp;
+static int asn1_parse2(bp, pp, length, offset, depth)
+BIO *bp;
 unsigned char **pp;
 long length;
 int offset;
 int depth;
 	{
+	char str[128];
 	unsigned char *p,*ep,*tot,*op,*opp;
 	long len;
-	int tag,xclass,ret=1;
-	int nl,i,hl,j;
+	int tag,xclass,ret=0;
+	int nl,hl,j,r;
 	ASN1_OBJECT *o=NULL;
 
 	p= *pp;
@@ -164,58 +170,60 @@ int depth;
 #endif
 		if (j == 0xff)
 			{
-			Fprintf(fp,"Error in encoding\n");
+			if (BIO_puts(bp,"Error in encoding\n") <= 0) goto end;
 			ret=0;
 			goto end;
 			}
 		hl=(p-op);
 		/* if j == 0x21 it is a constructed indefinite length object */
-		Fprintf(fp,"%5ld:",(long)offset+(long)(op- *pp));
+		sprintf(str,"%5ld:",(long)offset+(long)(op- *pp));
+		if (BIO_puts(bp,str) <= 0) goto end;
+
 		if (j != 0x21)
-			Fprintf(fp,"d=%d hl=%ld l=%3ld ",
-				depth,(long)hl,len);
+			{
+			sprintf(str,"d=%-2d hl=%ld l=%4ld ",depth,(long)hl,len);
+			if (BIO_puts(bp,str) <= 0) goto end;
+			}
 		else
-			Fprintf(fp,"d=%d hl=%ld l=inf ",depth,(long)hl);
-		asn1_print_info(fp,tag,xclass,j);
+			{
+			sprintf(str,"d=%-2d hl=%ld l=inf  ",depth,(long)hl);
+			if (BIO_puts(bp,str) <= 0) goto end;
+			}
+		if (!asn1_print_info(bp,tag,xclass,j)) goto end;
 		if (j & V_ASN1_CONSTRUCTED)
 			{
 			ep=p+len;
-			Fputc('\n',fp);
+			if (BIO_puts(bp,"\n") <= 0) goto end;
 			if (len > length)
 				{
+				sprintf(str,"length is greater than %ld\n",
+					length);
+				if (BIO_puts(bp,str) <= 0) goto end;
 				ret=0;
-				Fprintf(fp,"length is greater than %ld\n",length);
 				goto end;
 				}
 			if ((j == 0x21) && (len == 0))
 				{
 				for (;;)
 					{
-					if (!asn1_parse2(fp,&p,
-						(long)(tot-p),
-						offset+(p - *pp),depth+1))
-						{
-						ret=0;
-						goto end;
-						}
-					if (p >= tot) break;
+					r=asn1_parse2(bp,&p,(long)(tot-p),
+						offset+(p - *pp),depth+1);
+					if (r == 0) { ret=0; goto end; }
+					if ((r == 2) || (p >= tot)) break;
 					}
 				}
 			else
 				while (p < ep)
 					{
-					if (!asn1_parse2(fp,&p,(long)len,
-						offset+(p - *pp),depth+1))
-						{
-						ret=0;
-						goto end;
-						}
+					r=asn1_parse2(bp,&p,(long)len,
+						offset+(p - *pp),depth+1);
+					if (r == 0) { ret=0; goto end; }
 					}
 			}
 		else if (xclass != 0)
 			{
 			p+=len;
-			Fputc('\n',fp);
+			if (BIO_puts(bp,"\n") <= 0) goto end;
 			}
 		else
 			{
@@ -225,20 +233,24 @@ int depth;
 				(tag == V_ASN1_IA5STRING) ||
 				(tag == V_ASN1_UTCTIME))
 				{
-				Fputc(':',fp);
-				for (i=0; i<len; i++)
-					Fputc(p[i],fp);
+				if (BIO_puts(bp,":") <= 0) goto end;
+				if (BIO_write(bp,(char *)p,(int)len)
+					!= (int)len)
+					goto end;
 				}
 			else if (tag == V_ASN1_OBJECT)
 				{
 				opp=op;
 				if (d2i_ASN1_OBJECT(&o,&opp,len+hl) != NULL)
 					{
-					Fprintf(fp,":");
-					i2f_ASN1_OBJECT(fp,o);
+					if (BIO_puts(bp,":") <= 0) goto end;
+					i2a_ASN1_OBJECT(bp,o);
 					}
 				else
-					Fprintf(fp,":BAD OBJECT");
+					{
+					if (BIO_puts(bp,":BAD OBJECT") <= 0)
+						goto end;
+					}
 				}
 			else if (tag == V_ASN1_INTEGER)
 				{
@@ -249,28 +261,43 @@ int depth;
 				bs=d2i_ASN1_INTEGER(NULL,&opp,len+hl);
 				if (bs != NULL)
 					{
-					Fputc(':',fp);
+					if (BIO_puts(bp,":") <= 0) goto end;
 					if (bs->type == V_ASN1_NEG_INTEGER)
-						Fputc('-',fp);
+						if (BIO_puts(bp,"-") <= 0)
+							goto end;
 					for (i=0; i<bs->length; i++)
-						Fprintf(fp,"%02X",bs->data[i]);
+						{
+						sprintf(str,"%02X",bs->data[i]);
+						if (BIO_puts(bp,str) <= 0)
+							goto end;
+						}
 					if (bs->length == 0)
-						Fprintf(fp,"00");
+						{
+						if (BIO_puts(bp,"00") <= 0)
+							goto end;
+						}
 					}
 				else
-					Fprintf(fp,"BAD INTEGER");
+					{
+					if (BIO_puts(bp,"BAD INTEGER") <= 0)
+						goto end;
+					}
 				ASN1_INTEGER_free(bs);
 				}
 
-			if (!nl) Fputc('\n',fp);
+			if (!nl) 
+				{
+				if (BIO_puts(bp,"\n") <= 0) goto end;
+				}
 			p+=len;
 			if ((tag == V_ASN1_EOC) && (xclass == 0))
 				{
-				ret=0;
+				ret=2; /* End of sequence */
 				goto end;
 				}
 			}
 		}
+	ret=1;
 end:
 	if (o != NULL) ASN1_OBJECT_free(o);
 	*pp=p;
