@@ -1,5 +1,5 @@
-/* lib/bn/bn_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/bn/bn_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,15 +46,34 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "bn.h"
 
-char *BN_version="Big Number part of SSLeay 0.5.2 29-Apr-1996";
+char *BN_version="Big Number part of SSLeay 0.6.0 21-Jun-1996";
 
 static BN_ULONG data_one=1L;
 static BIGNUM const_one={&data_one,1,1,0};
 BIGNUM *BN_value_one=&const_one;
 
+char *BN_options()
+	{
+	static init=0;
+	static char data[16];
+
+	if (!init)
+		{
+		init++;
+#ifdef BN_LLONG
+		sprintf(data,"bn(%d,%d)",sizeof(BN_ULLONG)*8,
+			sizeof(BN_ULONG)*8);
+#else
+		sprintf(data,"bn(%d,%d)",sizeof(BN_ULONG)*8,
+			sizeof(BN_ULONG)*8);
+#endif
+		}
+	return(data);
+	}
+
 int BN_num_bits_word(l)
 BN_ULONG l;
 	{
@@ -131,7 +150,9 @@ BIGNUM *a;
 	i=(a->top-1)*BN_BITS2;
 	if (l == 0)
 		{
+#ifndef WIN16
 		fprintf(stderr,"BAD TOP VALUE\n");
+#endif
 		abort();
 		}
 	return(i+BN_num_bits_word(l));
@@ -229,7 +250,7 @@ int bits;
 		p=b->d=(BN_ULONG *)realloc(b->d,sizeof(BN_ULONG)*(n+1));
 		if (p == NULL)
 			{
-			BNerr(BN_F_BN_EXPAND,ERR_R_MALLOC_FAILURE);
+			BNerr(BN_F_BN_EXPAND2,ERR_R_MALLOC_FAILURE);
 			return(NULL);
 			}
 		memset(&(p[b->max]),0,((n+1)-b->max)*sizeof(BN_ULONG));
@@ -268,15 +289,46 @@ BIGNUM *a;
 	a->neg=0;
 	}
 
+unsigned long BN_get_word(a)
+BIGNUM *a;
+	{
+	int i,n;
+	unsigned long ret=0;
+
+	n=BN_num_bytes(a);
+	if (n > sizeof(unsigned long))
+		return(0xFFFFFFFF);
+	for (i=a->top-1; i>=0; i--)
+		{
+		ret<<=BN_BITS4; /* stops the compiler complaining */
+		ret<<=BN_BITS4;
+		ret|=a->d[i];
+		}
+	return(ret);
+	}
+
 int BN_set_word(a,w)
 BIGNUM *a;
 unsigned long w;
 	{
+	int i,n;
 	if (bn_expand(a,sizeof(unsigned long)*8) == NULL) return(0);
 
-	a->d[0]=w;
-	a->top=(w == 0)?0:1;
+	n=sizeof(unsigned long)/BN_BYTES;
 	a->neg=0;
+	a->top=0;
+	a->d[0]=(BN_ULONG)w&BN_MASK2;
+	if (a->d[0] != 0) a->top=1;
+	for (i=1; i<n; i++)
+		{
+		/* the following is done instead of
+		 * w>>=BN_BITS2 so compilers don't complain
+		 * on builds where sizeof(long) == BN_TYPES */
+		w>>=BN_BITS4;
+		w>>=BN_BITS4;
+		a->d[i]=(BN_ULONG)w&BN_MASK2;
+		if (a->d[i] != 0) a->top=i+1;
+		}
 	return(1);
 	}
 
