@@ -1,5 +1,5 @@
-/* lib/des/ecb_enc.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/des/ecb_enc.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -49,7 +49,23 @@
 #include "spr.h"
 
 char *libdes_version="libdes v 3.24 - 20-Apr-1996 - eay";
-char *DES_version="DES part of SSLeay 0.5.2 25-Apr-1996";
+char *DES_version="DES part of SSLeay 0.6.0 21-Jun-1996";
+
+char *des_options()
+	{
+#ifdef DES_PTR
+	if (sizeof(DES_LONG) != sizeof(long))
+		return("des(ptr,int)");
+	else
+		return("des(ptr,long)");
+#else
+	if (sizeof(DES_LONG) != sizeof(long))
+		return("des(idx,int)");
+	else
+		return("des(idx,long)");
+#endif
+	}
+		
 
 void des_ecb_encrypt(input, output, ks, encrypt)
 des_cblock (*input);
@@ -109,21 +125,26 @@ int encrypt;
 
 	s=(DES_LONG *)ks;
 	/* I don't know if it is worth the effort of loop unrolling the
-	 * inner loop */
+	 * inner loop
+	 */
 	if (encrypt)
 		{
-		for (i=0; i<32; i+=4)
+		for (i=0; i<32; i+=8)
 			{
 			D_ENCRYPT(l,r,i+0); /*  1 */
 			D_ENCRYPT(r,l,i+2); /*  2 */
+			D_ENCRYPT(l,r,i+4); /*  3 */
+			D_ENCRYPT(r,l,i+6); /*  4 */
 			}
 		}
 	else
 		{
-		for (i=30; i>0; i-=4)
+		for (i=30; i>0; i-=8)
 			{
 			D_ENCRYPT(l,r,i-0); /* 16 */
 			D_ENCRYPT(r,l,i-2); /* 15 */
+			D_ENCRYPT(l,r,i-4); /* 14 */
+			D_ENCRYPT(r,l,i-6); /* 13 */
 			}
 		}
 	l=(l>>1)|(l<<31);
@@ -178,18 +199,22 @@ int encrypt;
 	 * inner loop */
 	if (encrypt)
 		{
-		for (i=0; i<32; i+=4)
+		for (i=0; i<32; i+=8)
 			{
 			D_ENCRYPT(l,r,i+0); /*  1 */
 			D_ENCRYPT(r,l,i+2); /*  2 */
+			D_ENCRYPT(l,r,i+4); /*  3 */
+			D_ENCRYPT(r,l,i+6); /*  4 */
 			}
 		}
 	else
 		{
-		for (i=30; i>0; i-=4)
+		for (i=30; i>0; i-=8)
 			{
 			D_ENCRYPT(l,r,i-0); /* 16 */
 			D_ENCRYPT(r,l,i-2); /* 15 */
+			D_ENCRYPT(l,r,i-4); /* 14 */
+			D_ENCRYPT(r,l,i-6); /* 13 */
 			}
 		}
 	l=(l>>1)|(l<<31);
