@@ -1,5 +1,5 @@
-/* lib/des/fcrypt.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/des/fcrypt.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -54,26 +54,14 @@
  * eay@mincom.oz.au or eay@psych.psy.uq.oz.au
  */
 
+#ifndef HEADER_DES_LOCL_H
+
 #if defined(NOCONST)
 #define const
 #endif
 
-typedef unsigned char des_cblock[8];
-
-typedef struct des_ks_struct
-	{
-	union	{
-		des_cblock _;
-		/* make sure things are correct size on machines with
-		 * 8 byte longs */
-		unsigned long pad[2];
-		} ks;
-#define _	ks._
-	} des_key_schedule[16];
-
-#define DES_KEY_SZ 	(sizeof(des_cblock))
-#define DES_ENCRYPT	1
-#define DES_DECRYPT	0
+/* I now assume that you have des.h available */
+#include "des.h"
 
 #define ITERATIONS 16
 #define HALF_ITERATIONS 8
@@ -88,7 +76,7 @@ typedef struct des_ks_struct
 			 *((c)++)=(unsigned char)(((l)>>16)&0xff), \
 			 *((c)++)=(unsigned char)(((l)>>24)&0xff))
 
-static const unsigned long SPtrans[8][64]={
+static const unsigned long des_SPtrans[8][64]={
 {
 /* nibble 0 */
 0x00820200L, 0x00020000L, 0x80800000L, 0x80820200L,
@@ -388,21 +376,31 @@ static const unsigned long skb[8][64]={
 #define HPERM_OP(a,t,n,m) ((t)=((((a)<<(16-(n)))^(a))&(m)),\
 	(a)=(a)^(t)^(t>>(16-(n))))\
 
-static const int shifts2[16]={0,0,1,1,1,1,1,1,0,1,1,1,1,1,1,0};
+#endif 
 
 #ifndef NOPROTO
 static int body(unsigned long *out0, unsigned long *out1,
 	des_key_schedule ks, unsigned long Eswap0, unsigned long Eswap1);
-static int des_set_key(des_cblock (*key), des_key_schedule schedule);
-#else
-static int body();
-static int des_set_key();
+#ifndef HEADER_DES_LOCL_H
+static int fcrypt_set_key(des_cblock (*key), des_key_schedule schedule);
 #endif
 
-static int des_set_key(key, schedule)
+#else
+
+static int body();
+#ifndef HEADER_DES_LOCL_H
+static int fcrypt_set_key();
+#endif
+#endif
+
+#ifdef HEADER_DES_LOCL_H
+#define fcrypt_set_key(a,b)	des_set_key(a,b)
+#else
+static int fcrypt_set_key(key, schedule)
 des_cblock (*key);
 des_key_schedule schedule;
 	{
+	static const int shifts2[16]={0,0,1,1,1,1,1,1,0,1,1,1,1,1,1,0};
 	register unsigned long c,d,t,s;
 	register unsigned char *in;
 	register unsigned long *k;
@@ -456,6 +454,7 @@ des_key_schedule schedule;
 		}
 	return(0);
 	}
+#endif
 
 /******************************************************************
  * modified stuff for crypt.
@@ -466,8 +465,8 @@ des_key_schedule schedule;
  * Inspired by Dana How <how@isl.stanford.edu>
  * DO NOT use the alternative version on machines with 8 byte longs.
  */
-#ifdef DES_USE_PTR
-#define D_ENCRYPT(L,R,S) \
+#ifdef DES_PTR
+#define CR_ENCRYPT(L,R,S) \
 	t=(R^(R>>16)); \
 	u=(t&E0); \
 	t=(t&E1); \
@@ -475,30 +474,30 @@ des_key_schedule schedule;
 	t=(t^(t<<16))^R^s[S+1]; \
 	t=(t>>2)|(t<<30); \
 	L^= \
-	*(unsigned long *)(des_SP+0x0100+((t    )&0xfc))+ \
-	*(unsigned long *)(des_SP+0x0300+((t>> 8)&0xfc))+ \
-	*(unsigned long *)(des_SP+0x0500+((t>>16)&0xfc))+ \
-	*(unsigned long *)(des_SP+0x0700+((t>>24)&0xfc))+ \
-	*(unsigned long *)(des_SP+       ((u    )&0xfc))+ \
-  	*(unsigned long *)(des_SP+0x0200+((u>> 8)&0xfc))+ \
-  	*(unsigned long *)(des_SP+0x0400+((u>>16)&0xfc))+ \
- 	*(unsigned long *)(des_SP+0x0600+((u>>24)&0xfc));
+	*(DES_LONG *)((unsigned char *)des_SP+0x0100+((t    )&0xfc))+ \
+	*(DES_LONG *)((unsigned char *)des_SP+0x0300+((t>> 8)&0xfc))+ \
+	*(DES_LONG *)((unsigned char *)des_SP+0x0500+((t>>16)&0xfc))+ \
+	*(DES_LONG *)((unsigned char *)des_SP+0x0700+((t>>24)&0xfc))+ \
+	*(DES_LONG *)((unsigned char *)des_SP+       ((u    )&0xfc))+ \
+  	*(DES_LONG *)((unsigned char *)des_SP+0x0200+((u>> 8)&0xfc))+ \
+  	*(DES_LONG *)((unsigned char *)des_SP+0x0400+((u>>16)&0xfc))+ \
+ 	*(DES_LONG *)((unsigned char *)des_SP+0x0600+((u>>24)&0xfc));
 #else /* original version */
-#define D_ENCRYPT(L,R,S)	\
+#define CR_ENCRYPT(L,R,S)	\
 	t=(R^(R>>16)); \
 	u=(t&E0); \
 	t=(t&E1); \
 	u=(u^(u<<16))^R^s[S  ]; \
 	t=(t^(t<<16))^R^s[S+1]; \
 	t=(t>>4)|(t<<28); \
-	L^=	SPtrans[1][(t    )&0x3f]| \
-		SPtrans[3][(t>> 8)&0x3f]| \
-		SPtrans[5][(t>>16)&0x3f]| \
-		SPtrans[7][(t>>24)&0x3f]| \
-		SPtrans[0][(u    )&0x3f]| \
-		SPtrans[2][(u>> 8)&0x3f]| \
-		SPtrans[4][(u>>16)&0x3f]| \
-		SPtrans[6][(u>>24)&0x3f];
+	L^=	des_SPtrans[1][(t    )&0x3f]| \
+		des_SPtrans[3][(t>> 8)&0x3f]| \
+		des_SPtrans[5][(t>>16)&0x3f]| \
+		des_SPtrans[7][(t>>24)&0x3f]| \
+		des_SPtrans[0][(u    )&0x3f]| \
+		des_SPtrans[2][(u>> 8)&0x3f]| \
+		des_SPtrans[4][(u>>16)&0x3f]| \
+		des_SPtrans[6][(u>>24)&0x3f];
 #endif
 
 /* Added more values to handle illegal salt values the way normal
@@ -539,7 +538,7 @@ static unsigned const char cov_2char[64]={
 #ifdef PERL5
 char *des_crypt(char *buf,char *salt);
 #else
-char *crypt(char *buf,char *salt);
+char *crypt(const char *buf,const char *salt);
 #endif
 #else
 #ifdef PERL5
@@ -554,11 +553,11 @@ char *des_crypt(buf,salt)
 #else
 char *crypt(buf,salt)
 #endif
-char *buf;
-char *salt;
+const char *buf;
+const char *salt;
 	{
 	unsigned int i,j,x,y;
-	unsigned long Eswap0=0,Eswap1=0;
+	unsigned long Eswap0,Eswap1;
 	unsigned long out[2],ll;
 	des_cblock key;
 	des_key_schedule ks;
@@ -589,7 +588,7 @@ char *salt;
 	for (; i<8; i++)
 		key[i]=0;
 
-	des_set_key((des_cblock *)(key),ks);
+	fcrypt_set_key((des_cblock *)(key),ks);
 	body(&(out[0]),&(out[1]),ks,Eswap0,Eswap1);
 
 	ll=out[0]; l2c(ll,b);
@@ -625,8 +624,8 @@ unsigned long Eswap0;
 unsigned long Eswap1;
 	{
 	register unsigned long l,r,t,u;
-#ifdef DES_USE_PTR
-	register unsigned char *des_SP=(unsigned char *)SPtrans;
+#ifdef DES_PTR
+	register unsigned char *des_SP=(unsigned char *)des_SPtrans;
 #endif
 	register unsigned long *s;
 	register int i,j;
@@ -643,8 +642,8 @@ unsigned long Eswap1;
 		{
 		for (i=0; i<(ITERATIONS*2); i+=4)
 			{
-			D_ENCRYPT(l,r,  i);	/*  1 */
-			D_ENCRYPT(r,l,  i+2);	/*  2 */
+			CR_ENCRYPT(l,r,  i);	/*  1 */
+			CR_ENCRYPT(r,l,  i+2);	/*  2 */
 			}
 		t=l;
 		l=r;
