@@ -1,5 +1,5 @@
-/* lib/des/read_pwd.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/des/read_pwd.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -45,6 +45,12 @@
  * [including the GNU Public Licence.]
  */
 
+#ifdef WIN16TTY
+#undef WIN16
+#undef _WINDOWS
+#include <graph.h>
+#endif
+
 /* 06-Apr-92 Luke Brennan    Support for VMS */
 #include "des_locl.h"
 #include <signal.h>
@@ -55,9 +61,9 @@
  * TERMIO, TERMIOS, VMS, MSDOS and SGTTY
  */
 
-#if defined(sgi) || defined(__sgi)
-#define TERMIOS
-#undef TERMIO
+#if defined(linux) && !defined(TERMIO)
+#undef TERMIOS
+#define TERMIO
 #undef SGTTY
 #endif
 
@@ -124,7 +130,7 @@ static int read_pw(char *buf, char *buff, int size, char *prompt, int verify);
 static void recsig(int);
 static void pushsig(void);
 static void popsig(void);
-#ifdef MSDOS
+#if defined(MSDOS) && !defined(WIN16)
 static int noecho_fgets(char *buf, int size, FILE *tty);
 #endif
 #else
@@ -133,7 +139,7 @@ static int read_pw();
 static void recsig();
 static void pushsig();
 static void popsig();
-#ifdef MSDOS
+#if defined(MSDOS) && !defined(WIN16)
 static int noecho_fgets();
 #endif
 #endif
@@ -190,6 +196,8 @@ int verify;
 	return(ret);
 	}
 
+#ifndef WIN16
+
 static void read_till_nl(in)
 FILE *in;
 	{
@@ -201,6 +209,7 @@ FILE *in;
 		} while (strchr(buf,'\n') == NULL);
 	}
 
+
 /* return 0 if ok, 1 (or -1) otherwise */
 static int read_pw(buf, buff, size, prompt, verify)
 char *buf;
@@ -282,12 +291,13 @@ int verify;
 		buf[0]='\0';
 		fgets(buf,size,tty);
 		if (feof(tty)) goto error;
+		if (ferror(tty)) goto error;
 		if ((p=(char *)strchr(buf,'\n')) != NULL)
 			*p='\0';
 		else	read_till_nl(tty);
 		if (verify)
 			{
-			fprintf(stderr,"\nVerifying password %s",prompt);
+			fprintf(stderr,"\nVerifying password - %s",prompt);
 			fflush(stderr);
 			buff[0]='\0';
 			fgets(buff,size,tty);
@@ -309,6 +319,9 @@ int verify;
 
 error:
 	fprintf(stderr,"\n");
+#ifdef DEBUG
+	perror("fgets(tty)");
+#endif
 	/* What can we do if there is an error? */
 #if defined(TTY_set) && !defined(VMS) 
 	if (ps >= 2) TTY_set(fileno(tty),&tty_orig);
@@ -327,6 +340,22 @@ error:
 	return(!ok);
 	}
 
+#else /* WIN16 */
+
+static int read_pw(buf, buff, size, prompt, verify)
+char *buf;
+char *buff;
+int size;
+char *prompt;
+int verify;
+	{ 
+	memset(buf,0,size);
+	memset(buff,0,size);
+	return(0);
+	}
+
+#endif
+
 static void pushsig()
 	{
 	int i;
@@ -352,7 +381,7 @@ int i;
 #endif
 	}
 
-#ifdef MSDOS
+#if defined(MSDOS) && !defined(WIN16)
 static int noecho_fgets(buf,size,tty)
 char *buf;
 int size;
@@ -370,7 +399,11 @@ FILE *tty;
 			break;
 			}
 		size--;
+#ifdef WIN16TTY
+		i=_inchar();
+#else
 		i=getch();
+#endif
 		if (i == '\r') i='\n';
 		*(p++)=i;
 		if (i == '\n')
