@@ -1,5 +1,5 @@
-/* lib/dh/dh_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/dh/dh_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,11 +46,11 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "bn.h"
 #include "dh.h"
 
-char *DH_version="Diffie-Hellman part of SSLeay 0.5.2 29-Apr-1996";
+char *DH_version="Diffie-Hellman part of SSLeay 0.6.0 21-Jun-1996";
 
 DH *DH_new()
 	{
@@ -62,6 +62,8 @@ DH *DH_new()
 		DHerr(DH_F_DH_NEW,ERR_R_MALLOC_FAILURE);
 		return(NULL);
 		}
+	ret->pad=0;
+	ret->version=0;
 	ret->p=NULL;
 	ret->g=NULL;
 	ret->length=0;
