@@ -1,5 +1,5 @@
-/* lib/dh/dhtest.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/dh/dhtest.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,31 +47,50 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include "buffer.h"
 #include "bn.h"
 #include "dh.h"
 
-#ifndef NOPROTO
-static void cb(int p, int n);
+#ifdef WIN16
+#define MS_CALLBACK	_far _loadds
 #else
-static void cb();
+#define MS_CALLBACK
+#endif
+
+#ifndef NOPROTO
+static void MS_CALLBACK cb(int p, int n);
+#else
+static void MS_CALLBACK cb();
+#endif
+
+#ifdef WIN16
+#define APPS_WIN16
+#include "../buffer/bss_file.c"
 #endif
 
 int main(argc,argv)
 int argc;
 char *argv[];
 	{
+	BIO *out=NULL;
 	DH *a,*b;
+	char buf[12];
 	unsigned char *abuf,*bbuf;
 	int i,alen,blen,aout,bout,ret=1;
 
+	out=BIO_new(BIO_s_file());
+	if (out == NULL) exit(1);
+	BIO_set_fp(out,stdout,BIO_NOCLOSE);
+
 	a=DH_generate_parameters(64,DH_GENERATOR_5,cb);
 	if (a == NULL) goto err;
 
-	printf("\np    =");
-	BN_print(stdout,a->p);
-	printf("\ng    =");
-	BN_print(stdout,a->g);
-	printf("\n");
+	BIO_puts(out,"\np    =");
+	BN_print(out,a->p);
+	BIO_puts(out,"\ng    =");
+	BN_print(out,a->g);
+	BIO_puts(out,"\n");
 
 	b=DH_new();
 	if (b == NULL) goto err;
@@ -81,36 +100,42 @@ char *argv[];
 	if ((b->p == NULL) || (b->g == NULL)) goto err;
 
 	if (!DH_generate_key(a)) goto err;
-	printf("pri 1=");
-	BN_print(stdout,a->private);
-	printf("\npub 1=");
-	BN_print(stdout,a->public);
-	printf("\n");
+	BIO_puts(out,"pri 1=");
+	BN_print(out,a->private);
+	BIO_puts(out,"\npub 1=");
+	BN_print(out,a->public);
+	BIO_puts(out,"\n");
 
 	if (!DH_generate_key(b)) goto err;
-	printf("pri 2=");
-	BN_print(stdout,b->private);
-	printf("\npub 2=");
-	BN_print(stdout,b->public);
-	printf("\n");
+	BIO_puts(out,"pri 2=");
+	BN_print(out,b->private);
+	BIO_puts(out,"\npub 2=");
+	BN_print(out,b->public);
+	BIO_puts(out,"\n");
 
 	alen=DH_size(a);
 	abuf=(unsigned char *)malloc(alen);
 	aout=DH_compute_key(abuf,b->public,a);
 
-	printf("key1 =");
+	BIO_puts(out,"key1 =");
 	for (i=0; i<aout; i++)
-		printf("%02X",abuf[i]);
-	printf("\n");
+		{
+		sprintf(buf,"%02X",abuf[i]);
+		BIO_puts(out,buf);
+		}
+	BIO_puts(out,"\n");
 
 	blen=DH_size(b);
 	bbuf=(unsigned char *)malloc(blen);
 	bout=DH_compute_key(bbuf,a->public,b);
 
-	printf("key2 =");
+	BIO_puts(out,"key2 =");
 	for (i=0; i<bout; i++)
-		printf("%02X",bbuf[i]);
-	printf("\n");
+		{
+		sprintf(buf,"%02X",bbuf[i]);
+		BIO_puts(out,buf);
+		}
+	BIO_puts(out,"\n");
 	if ((aout < 4) || (bout != aout) || (memcmp(abuf,bbuf,aout) != 0))
 		{
 		fprintf(stderr,"Error in DH routines\n");
@@ -123,7 +148,7 @@ err:
 	return(ret);
 	}
 
-static void cb(p, n)
+static void MS_CALLBACK cb(p, n)
 int p;
 int n;
 	{
