@@ -1,5 +1,5 @@
-/* lib/evp/evp_enc.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/evp/evp_enc.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,10 +46,10 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "envelope.h"
 
-char *EVP_version="EVP part of SSLeay 0.5.2 29-Apr-1996";
+char *EVP_version="EVP part of SSLeay 0.6.0 21-Jun-1996";
 
 void EVP_CipherInit(ctx,data,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
