@@ -1,5 +1,5 @@
-/* lib/idea/i_ecb.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/idea/i_ecb.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,7 +48,15 @@
 #include "idea.h"
 #include "idea_lcl.h"
 
-char *IDEA_version="IDEA part of SSLeay 0.5.2 29-Apr-1996";
+char *IDEA_version="IDEA part of SSLeay 0.6.0 21-Jun-1996";
+
+char *idea_options()
+	{
+	if (sizeof(short) != sizeof(IDEA_INT))
+		return("idea(int)");
+	else
+		return("idea(short)");
+	}
 
 void idea_ecb_encrypt(in, out, ks)
 unsigned char *in;
