@@ -1,5 +1,5 @@
-/* lib/lhash/lhash.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/lhash/lhash.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -45,7 +45,7 @@
  * [including the GNU Public Licence.]
  */
 
-char *lh_version="lhash part of SSLeay 0.5.2 29-Apr-1996";
+char *lh_version="lhash part of SSLeay 0.6.0 21-Jun-1996";
 
 /* Code for dynamic hash table routines
  * Author - Eric Young v 2.0
@@ -85,6 +85,7 @@ char *lh_version="lhash part of SSLeay 0.5.2 29-Apr-1996";
 #include <stdlib.h>
 #include "lhash.h"
 
+#undef MIN_NODES 
 #define MIN_NODES	16
 #define UP_LOAD		(2*LH_LOAD_MULT) /* load times 256  (default 2) */
 #define DOWN_LOAD	(LH_LOAD_MULT)   /* load times 256  (default 1) */
@@ -449,8 +450,8 @@ unsigned long lh_strhash(c)
 char *c;
 	{
 	unsigned long ret=0;
-	long n=0x100;
-	long v=0;
+	long n;
+	unsigned long v;
 	int r;
 
 	if ((c == NULL) || (*c == '\0'))
@@ -466,7 +467,7 @@ char *c;
 		{
 		v=n|(*c);
 		n+=0x100;
-		r= ((v>>2)^v)&0x0f;
+		r= (int)((v>>2)^v)&0x0f;
 		ret=(ret<<r)|(ret>>(32-r));
 		ret&=0xFFFFFFFFL;
 		ret^=v*v;
