@@ -1,5 +1,5 @@
-/* lib/md/md2_dgst.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/md/md2_dgst.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -50,7 +50,7 @@
 #include <string.h>
 #include "md2.h"
 
-char *MD2_version="MD2 part of SSLeay 0.5.2 29-Apr-1996";
+char *MD2_version="MD2 part of SSLeay 0.6.0 21-Jun-1996";
 
 /* Implemented from RFC1319 The MD2 Message-Digest Algorithm
  */
@@ -58,9 +58,9 @@ char *MD2_version="MD2 part of SSLeay 0.5.2 29-Apr-1996";
 #define UCHAR	unsigned char
 
 #ifndef NOPROTO
-static void do_block(MD2_CTX *c, unsigned char *d);
+static void md2_block(MD2_CTX *c, unsigned char *d);
 #else
-static void do_block();
+static void md2_block();
 #endif
 
 /* The magic S table - I have converted it to hex since it is
@@ -100,6 +100,14 @@ static MD2_INT S[256]={
 	0xDB, 0x99, 0x8D, 0x33, 0x9F, 0x11, 0x83, 0x14,
 	};
 
+char *MD2_options()
+	{
+	if (sizeof(MD2_INT) == 1)
+		return("md2(char)");
+	else
+		return("md2(int)");
+	}
+
 void MD2_Init(c)
 MD2_CTX *c;
 	{
@@ -124,7 +132,7 @@ unsigned long len;
 		if ((c->num+len) >= MD2_BLOCK)
 			{
 			memcpy(&(p[c->num]),data,MD2_BLOCK-c->num);
-			do_block(c,c->data);
+			md2_block(c,c->data);
 			data+=(MD2_BLOCK - c->num);
 			len-=(MD2_BLOCK - c->num);
 			c->num=0;
@@ -132,9 +140,9 @@ unsigned long len;
 			}
 		else
 			{
-			memcpy(&(p[c->num]),data,len);
-			data+=len;
-			c->num+=len;
+			memcpy(&(p[c->num]),data,(int)len);
+			/* data+=len; */
+			c->num+=(int)len;
 			return;
 			}
 		}
@@ -142,15 +150,15 @@ unsigned long len;
 	 * chars and save the leftovers to c->data. */
 	while (len >= MD2_BLOCK)
 		{
-		do_block(c,data);
+		md2_block(c,data);
 		data+=MD2_BLOCK;
 		len-=MD2_BLOCK;
 		}
-	memcpy(p,data,len);
-	c->num=len;
+	memcpy(p,data,(int)len);
+	c->num=(int)len;
 	}
 
-static void do_block(c, d)
+static void md2_block(c, d)
 MD2_CTX *c;
 unsigned char *d;
 	{
@@ -203,14 +211,14 @@ MD2_CTX *c;
 	for (i=c->num; i<MD2_BLOCK; i++)
 		cp[i]=(UCHAR)v;
 
-	do_block(c,cp);
+	md2_block(c,cp);
 
 	for (i=0; i<MD2_BLOCK; i++)
-		cp[i]=p2[i];
-	do_block(c,cp);
+		cp[i]=(UCHAR)p2[i];
+	md2_block(c,cp);
 
 	for (i=0; i<16; i++)
-		md[i]=p1[i]&0xff;
+		md[i]=(UCHAR)(p1[i]&0xff);
 	memset((char *)&c,0,sizeof(c));
 	}
 
