@@ -1,5 +1,5 @@
-/* lib/md/md5_dgst.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/md/md5_dgst.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,7 +48,7 @@
 #include <stdio.h>
 #include "md5_locl.h"
 
-char *MD5_version="MD5 part of SSLeay 0.5.2 29-Apr-1996";
+char *MD5_version="MD5 part of SSLeay 0.6.0 21-Jun-1996";
 
 /* Implemented from RFC1321 The MD5 Message-Digest Algorithm
  */
@@ -59,9 +59,9 @@ char *MD5_version="MD5 part of SSLeay 0.5.2 29-Apr-1996";
 #define INIT_DATA_D (unsigned long)0x10325476L
 
 #ifndef NOPROTO
-static void do_block(MD5_CTX *c, unsigned long *p);
+static void md5_block(MD5_CTX *c, unsigned long *p);
 #else
-static void do_block();
+static void md5_block();
 #endif
 
 void MD5_Init(c)
@@ -82,7 +82,7 @@ register unsigned char *data;
 unsigned long len;
 	{
 	register ULONG *p;
-	int ew,ec,sw,sc;
+	int sw,sc;
 	ULONG l;
 
 	if (len == 0) return;
@@ -113,7 +113,7 @@ unsigned long len;
 				}
 			len-=(MD5_CBLOCK-c->num);
 
-			do_block(c,p);
+			md5_block(c,p);
 			c->num=0;
 			/* drop through and do the rest */
 			}
@@ -121,7 +121,7 @@ unsigned long len;
 			{
 			int ew,ec;
 
-			c->num+=len;
+			c->num+=(int)len;
 			if ((sc+len) < 4) /* ugly, add char's to a word */
 				{
 				l= p[sw];
@@ -151,30 +151,53 @@ unsigned long len;
 	p=c->data;
 	while (len >= MD5_CBLOCK)
 		{
-		for (sw=(MD5_BLOCK/4); sw; sw--)
+#if defined(L_ENDIAN) || defined(B_ENDIAN)
+		memcpy(p,data,MD5_CBLOCK);
+		data+=MD5_CBLOCK;
+#ifdef B_ENDIAN
+		for (sw=(MD5_LBLOCK/4); sw; sw--)
+			{
+			Endian_Reverse32(p[0]);
+			Endian_Reverse32(p[1]);
+			Endian_Reverse32(p[2]);
+			Endian_Reverse32(p[3]);
+			p+=4;
+			}
+#endif
+#else
+		for (sw=(MD5_LBLOCK/4); sw; sw--)
 			{
 			c2l(data,l); *(p++)=l;
 			c2l(data,l); *(p++)=l;
 			c2l(data,l); *(p++)=l;
-			c2l(data,l); *(p++)=l;
-			}
+			c2l(data,l); *(p++)=l; 
+			} 
+#endif
 		p=c->data;
-		do_block(c,p);
+		md5_block(c,p);
 		len-=MD5_CBLOCK;
 		}
-	ew=(len>>2);
-	ec=(len&0x03);
-
-	for (sw=0; sw < ew; sw++)
-		{ c2l(data,l); p[sw]=l; }
-	c2l_p(data,l,ec);
-	p[sw]=l;
-	c->num=len;
+	sc=(int)len;
+	c->num=sc;
+	if (sc)
+		{
+		sw=sc>>2;	/* words to copy */
+#ifdef L_ENDIAN
+		p[sw]=0;
+		memcpy(p,data,sc);
+#else
+		sc&=0x03;
+		for ( ; sw; sw--)
+			{ c2l(data,l); *(p++)=l; }
+		c2l_p(data,l,sc);
+		*p=l;
+#endif
+		}
 	}
 
-static void do_block(c, X)
+static void md5_block(c, X)
 MD5_CTX *c;
-register unsigned long *X;
+register ULONG *X;
 	{
 	register ULONG A,B,C,D;
 
@@ -294,21 +317,21 @@ MD5_CTX *c;
 		{
 		for (; i<MD5_LBLOCK; i++)
 			p[i]=0;
-		do_block(c,p);
+		md5_block(c,p);
 		i=0;
 		}
 	for (; i<(MD5_LBLOCK-2); i++)
 		p[i]=0;
 	p[MD5_LBLOCK-2]=c->Nl;
 	p[MD5_LBLOCK-1]=c->Nh;
-	do_block(c,p);
+	md5_block(c,p);
 	cp=md;
 	l=c->A; l2c(l,cp);
 	l=c->B; l2c(l,cp);
 	l=c->C; l2c(l,cp);
 	l=c->D; l2c(l,cp);
 
-	/* clear stuff, do_block may be leaving some stuff on the stack
+	/* clear stuff, md5_block may be leaving some stuff on the stack
 	 * but I'm not worried :-) */
 	c->num=0;
 /*	memset((char *)&c,0,sizeof(c));*/
