@@ -1,5 +1,5 @@
 /* lib/pem/pem.h */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,17 +48,14 @@
 #ifndef HEADER_PEM_H
 #define HEADER_PEM_H
 
-#ifndef HEADER_RSA_H
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 #include "rsa.h"
-#endif
-
-#ifndef HEADER_X509_H
+#include "dsa.h"
 #include "x509.h"
-#endif
-
-#ifndef HEADER_ENVELOPE_H
 #include "envelope.h"
-#endif
 
 #define PEM_OBJ_UNDEF		0
 #define PEM_OBJ_X509		1
@@ -90,7 +87,9 @@
 #define PEM_STRING_X509_REQ_OLD	"NEW CERTIFICATE REQUEST"
 #define PEM_STRING_X509_REQ	"CERTIFICATE REQUEST"
 #define PEM_STRING_X509_CRL	"X509 CRL"
+#define PEM_STRING_EVP_PKEY	"PRIVATE KEY"
 #define PEM_STRING_RSA		"RSA PRIVATE KEY"
+#define PEM_STRING_DSA		"DSA PRIVATE KEY"
 #define PEM_STRING_PKCS7	"PKCS7"
 #define PEM_STRING_DHPARAMS	"DH PARAMETERS"
 #define PEM_STRING_SSL_SESSION	"SSL SESSION PARAMETERS"
@@ -100,12 +99,12 @@
 
 #ifndef HEADER_ENVELOPE_H
 
-#define EVP_ENCODE_CTX_SIZE 88
-#define EVP_MD_SIZE 24
-#define EVP_MD_CTX_SIZE 152
-#define EVP_CIPHER_SIZE 28
-#define EVP_CIPHER_CTX_SIZE 1068
-#define EVP_MAX_MD_SIZE 20
+#define EVP_ENCODE_CTX_SIZE  88
+#define EVP_MD_SIZE  36
+#define EVP_MD_CTX_SIZE  152
+#define EVP_CIPHER_SIZE  28
+#define EVP_CIPHER_CTX_SIZE  1068
+#define EVP_MAX_MD_SIZE  20
 
 typedef struct evp_encode_ctx_st
 	{
@@ -182,7 +181,7 @@ typedef struct pem_ctx_st
 
 	int md_enc;		/* is the md encrypted or not? */
 	int md_len;		/* length of md_data */
-	char *md_data;		/* message digest, could be rsa encrypted */
+	char *md_data;		/* message digest, could be pkey encrypted */
 
 	EVP_CIPHER *dec;	/* date encryption cipher */
 	int key_len;		/* key length */
@@ -197,29 +196,55 @@ typedef struct pem_ctx_st
 
 #ifndef NOPROTO
 int	PEM_get_EVP_CIPHER_INFO(char *header, EVP_CIPHER_INFO *cipher);
-
 int	PEM_do_header (EVP_CIPHER_INFO *cipher, unsigned char *data,long *len,
 		int (*callback)());
+
+#ifdef HEADER_BUFFER_H
+int	PEM_read_bio(BIO *bp, char **name, char **header,
+		unsigned char **data,long *len);
+int	PEM_write_bio(BIO *bp,char *name,char *hdr,unsigned char *data,
+		long len);
+char *	PEM_ASN1_read_bio(char *(*d2i)(),char *name,BIO *bp,char **x,
+		int (*cb)());
+int	PEM_ASN1_write_bio(int (*i2d)(),char *name,BIO *bp,char *x,
+		EVP_CIPHER *enc,unsigned char *kstr,int klen,int (*callback)());
+#else
+int	PEM_read_bio(char *bp, char **name, char **header,
+		unsigned char **data,long *len);
+int	PEM_write_bio(char *bp,char *name,char *hdr,unsigned char *data,
+		long len);
+char *	PEM_ASN1_read_bio(char *(*d2i)(),char *name,char *bp,char **x,
+		int (*cb)());
+int	PEM_ASN1_write_bio(int (*i2d)(),char *name,char *bp,char *x,
+		EVP_CIPHER *enc,unsigned char *kstr,int klen,int (*callback)());
+#endif
+
+#ifndef WIN16
 int	PEM_read(FILE *fp, char **name, char **header,
 		unsigned char **data,long *len);
-
 int	PEM_write(FILE *fp,char *name,char *hdr,unsigned char *data,long len);
-
+char *	PEM_ASN1_read(char *(*d2i)(),char *name,FILE *fp,char **x,
+		int (*cb)());
+int	PEM_ASN1_write(int (*i2d)(),char *name,FILE *fp,char *x,
+		EVP_CIPHER *enc,unsigned char *kstr,int klen,int (*callback)());
 STACK *	PEM_X509_INFO_read(FILE *fp, STACK *sk, int (*cb)());
-char *	PEM_ASN1_read(char *(*d2i)(),char *name,FILE *fp,char **x,int (*cb)());
-int	PEM_ASN1_write(int (*i2d)(),char *name,FILE *fp,char *x,EVP_CIPHER *enc,
-		unsigned char *kstr,int klen,int (*callback)());
+#endif
+
+#ifdef HEADER_BUFFER_H
+STACK *	PEM_X509_INFO_read_bio(BIO *fp, STACK *sk, int (*cb)());
+#else
+STACK *	PEM_X509_INFO_read_bio(char *fp, STACK *sk, int (*cb)());
+#endif
 
 int	PEM_SealInit(PEM_ENCODE_SEAL_CTX *ctx, EVP_CIPHER *type,
 		EVP_MD *md_type, unsigned char **ek, int *ekl,
-		unsigned char *iv, RSA **pubk, int npubk);
+		unsigned char *iv, EVP_PKEY **pubk, int npubk);
 void	PEM_SealUpdate(PEM_ENCODE_SEAL_CTX *ctx, unsigned char *out, int *outl,
 		unsigned char *in, int inl);
 int	PEM_SealFinal(PEM_ENCODE_SEAL_CTX *ctx, unsigned char *sig,int *sigl,
-		unsigned char *out, int *outl, RSA *priv);
-
+		unsigned char *out, int *outl, EVP_PKEY *priv);
 int	PEM_SignFinal(EVP_MD_CTX *ctx, unsigned char *sigret,
-		unsigned int *siglen, RSA *rsa);
+		unsigned int *siglen, EVP_PKEY *pkey);
 
 void	ERR_load_PEM_strings(void);
 
@@ -227,11 +252,18 @@ void	ERR_load_PEM_strings(void);
 
 int	PEM_get_EVP_CIPHER_INFO();
 int	PEM_do_header();
+int	PEM_read_bio();
+int	PEM_write_bio();
+#ifndef WIN16
 int	PEM_read();
 int	PEM_write();
 STACK *	PEM_X509_INFO_read();
 char *	PEM_ASN1_read();
 int	PEM_ASN1_write();
+#endif
+STACK *	PEM_X509_INFO_read_bio();
+char *	PEM_ASN1_read_bio();
+int	PEM_ASN1_write_bio();
 int	PEM_SealInit();
 void	PEM_SealUpdate();
 int	PEM_SealFinal();
@@ -243,27 +275,29 @@ void	ERR_load_PEM_strings();
 
 
 #define PEM_write_SSL_SESSION(fp,x) \
-		PEM_ASN1_write(i2d_SSL_SESSION,PEM_STRING_SSL_SESSION,fp, \
-			(char *)x, NULL,NULL,0,NULL)
-
+		PEM_ASN1_write((int (*)())i2d_SSL_SESSION, \
+			PEM_STRING_SSL_SESSION,fp, (char *)x, NULL,NULL,0,NULL)
 #define PEM_write_X509(fp,x) \
-		PEM_ASN1_write(i2d_X509,PEM_STRING_X509,fp,(char *)x, \
-			NULL,NULL,0,NULL)
+		PEM_ASN1_write((int (*)())i2d_X509,PEM_STRING_X509,fp, \
+			(char *)x, NULL,NULL,0,NULL)
 #define PEM_write_X509_REQ(fp,x) PEM_ASN1_write( \
-		i2d_X509_REQ,PEM_STRING_X509_REQ,fp,(char *)x, \
+		(int (*)())i2d_X509_REQ,PEM_STRING_X509_REQ,fp,(char *)x, \
 			NULL,NULL,0,NULL)
 #define PEM_write_X509_CRL(fp,x) \
-		PEM_ASN1_write(i2d_X509_CRL,PEM_STRING_X509_CRL,fp,(char *)x, \
-			NULL,NULL,0,NULL)
+		PEM_ASN1_write((int (*)())i2d_X509_CRL,PEM_STRING_X509_CRL, \
+			fp,(char *)x, NULL,NULL,0,NULL)
 #define	PEM_write_RSAPrivateKey(fp,x,enc,kstr,klen,cb) \
-		PEM_ASN1_write(i2d_RSAPrivateKey,PEM_STRING_RSA,fp,\
+		PEM_ASN1_write((int (*)())i2d_RSAPrivateKey,PEM_STRING_RSA,fp,\
+			(char *)x,enc,kstr,klen,cb)
+#define	PEM_write_DSAPrivateKey(fp,x,enc,kstr,klen,cb) \
+		PEM_ASN1_write((int (*)())i2d_DSAPrivateKey,PEM_STRING_DSA,fp,\
 			(char *)x,enc,kstr,klen,cb)
 #define PEM_write_PKCS7(fp,x) \
-		PEM_ASN1_write(i2d_PKCS7,PEM_STRING_PKCS7,fp,(char *)x, \
-			NULL,NULL,0,NULL)
+		PEM_ASN1_write((int (*)())i2d_PKCS7,PEM_STRING_PKCS7,fp, \
+			(char *)x, NULL,NULL,0,NULL)
 #define PEM_write_DHparams(fp,x) \
-		PEM_ASN1_write(i2d_DHparams,PEM_STRING_DHPARAMS,fp,(char *)x, \
-			NULL,NULL,0,NULL)
+		PEM_ASN1_write((int (*)())i2d_DHparams,PEM_STRING_DHPARAMS,fp,\
+			(char *)x,NULL,NULL,0,NULL)
 
 #define	PEM_read_SSL_SESSION(fp,x,cb) (SSL_SESSION *)PEM_ASN1_read( \
 	(char *(*)())d2i_SSL_SESSION,PEM_STRING_SSL_SESSION,fp,(char **)x,cb)
@@ -275,11 +309,60 @@ void	ERR_load_PEM_strings();
 	(char *(*)())d2i_X509_CRL,PEM_STRING_X509_CRL,fp,(char **)x,cb)
 #define	PEM_read_RSAPrivateKey(fp,x,cb) (RSA *)PEM_ASN1_read( \
 	(char *(*)())d2i_RSAPrivateKey,PEM_STRING_RSA,fp,(char **)x,cb)
+#define	PEM_read_DSAPrivateKey(fp,x,cb) (DSA *)PEM_ASN1_read( \
+	(char *(*)())d2i_DSAPrivateKey,PEM_STRING_DSA,fp,(char **)x,cb)
+#define	PEM_read_PrivateKey(fp,x,cb) (EVP_PKEY *)PEM_ASN1_read( \
+	(char *(*)())d2i_PrivateKey,PEM_STRING_EVP_PKEY,fp,(char **)x,cb)
 #define	PEM_read_PKCS7(fp,x,cb) (PKCS7 *)PEM_ASN1_read( \
 	(char *(*)())d2i_PKCS7,PEM_STRING_PKCS7,fp,(char **)x,cb)
 #define	PEM_read_DHparams(fp,x,cb) (DH *)PEM_ASN1_read( \
 	(char *(*)())d2i_DHparams,PEM_STRING_DHPARAMS,fp,(char **)x,cb)
 
+#define PEM_write_bio_SSL_SESSION(bp,x) \
+		PEM_ASN1_write_bio((int (*)())i2d_SSL_SESSION, \
+			PEM_STRING_SSL_SESSION,bp, (char *)x, NULL,NULL,0,NULL)
+#define PEM_write_bio_X509(bp,x) \
+		PEM_ASN1_write_bio((int (*)())i2d_X509,PEM_STRING_X509,bp, \
+			(char *)x, NULL,NULL,0,NULL)
+#define PEM_write_bio_X509_REQ(bp,x) PEM_ASN1_write_bio( \
+		(int (*)())i2d_X509_REQ,PEM_STRING_X509_REQ,bp,(char *)x, \
+			NULL,NULL,0,NULL)
+#define PEM_write_bio_X509_CRL(bp,x) \
+		PEM_ASN1_write_bio((int (*)())i2d_X509_CRL,PEM_STRING_X509_CRL,\
+			bp,(char *)x, NULL,NULL,0,NULL)
+#define	PEM_write_bio_RSAPrivateKey(bp,x,enc,kstr,klen,cb) \
+		PEM_ASN1_write_bio((int (*)())i2d_RSAPrivateKey,PEM_STRING_RSA,\
+			bp,(char *)x,enc,kstr,klen,cb)
+#define	PEM_write_bio_DSAPrivateKey(bp,x,enc,kstr,klen,cb) \
+		PEM_ASN1_write_bio((int (*)())i2d_DSAPrivateKey,PEM_STRING_DSA,\
+			bp,(char *)x,enc,kstr,klen,cb)
+#define PEM_write_bio_PKCS7(bp,x) \
+		PEM_ASN1_write_bio((int (*)())i2d_PKCS7,PEM_STRING_PKCS7,bp, \
+			(char *)x, NULL,NULL,0,NULL)
+#define PEM_write_bio_DHparams(bp,x) \
+		PEM_ASN1_write_bio((int (*)())i2d_DHparams,PEM_STRING_DHPARAMS,\
+			bp,(char *)x,NULL,NULL,0,NULL)
+
+#define	PEM_read_bio_SSL_SESSION(bp,x,cb) (SSL_SESSION *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_SSL_SESSION,PEM_STRING_SSL_SESSION,bp,(char **)x,cb)
+#define	PEM_read_bio_X509(bp,x,cb) (X509 *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_X509,PEM_STRING_X509,bp,(char **)x,cb)
+#define	PEM_read_bio_X509_REQ(bp,x,cb) (X509_REQ *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_X509_REQ,PEM_STRING_X509_REQ,bp,(char **)x,cb)
+#define	PEM_read_bio_X509_CRL(bp,x,cb) (X509_CRL *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_X509_CRL,PEM_STRING_X509_CRL,bp,(char **)x,cb)
+#define	PEM_read_bio_RSAPrivateKey(bp,x,cb) (RSA *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_RSAPrivateKey,PEM_STRING_RSA,bp,(char **)x,cb)
+#define	PEM_read_bio_DSAPrivateKey(bp,x,cb) (DSA *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_DSAPrivateKey,PEM_STRING_DSA,bp,(char **)x,cb)
+#define	PEM_read_bio_PrivateKey(bp,x,cb) (EVP_PKEY *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_PrivateKey,PEM_STRING_EVP_PKEY,bp,(char **)x,cb)
+
+#define	PEM_read_bio_PKCS7(bp,x,cb) (PKCS7 *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_PKCS7,PEM_STRING_PKCS7,bp,(char **)x,cb)
+#define	PEM_read_bio_DHparams(bp,x,cb) (DH *)PEM_ASN1_read_bio( \
+	(char *(*)())d2i_DHparams,PEM_STRING_DHPARAMS,bp,(char **)x,cb)
+
 /* BEGIN ERROR CODES */
 /* Error codes for the PEM functions. */
 
@@ -287,15 +370,20 @@ void	ERR_load_PEM_strings();
 #define PEM_F_DEF_CALLBACK				 100
 #define PEM_F_LOAD_IV					 101
 #define PEM_F_PEM_ASN1_READ				 102
-#define PEM_F_PEM_DO_HEADER				 103
-#define PEM_F_PEM_EVP_CIPHER_INFO			 104
-#define PEM_F_PEM_READ					 105
-#define PEM_F_PEM_SEALFINAL				 106
-#define PEM_F_PEM_SEALINIT				 107
-#define PEM_F_PEM_SIGNFINAL				 108
-#define PEM_F_PEM_WRITE					 109
-#define PEM_F_PEM_WRITE_RSA				 110
-#define PEM_F_PEM_X509_INFO_READ			 111
+#define PEM_F_PEM_ASN1_READ_BIO				 103
+#define PEM_F_PEM_ASN1_WRITE				 104
+#define PEM_F_PEM_ASN1_WRITE_BIO			 105
+#define PEM_F_PEM_DO_HEADER				 106
+#define PEM_F_PEM_GET_EVP_CIPHER_INFO			 107
+#define PEM_F_PEM_READ					 108
+#define PEM_F_PEM_READ_BIO				 109
+#define PEM_F_PEM_SEALFINAL				 110
+#define PEM_F_PEM_SEALINIT				 111
+#define PEM_F_PEM_SIGNFINAL				 112
+#define PEM_F_PEM_WRITE					 113
+#define PEM_F_PEM_WRITE_BIO				 114
+#define PEM_F_PEM_X509_INFO_READ			 115
+#define PEM_F_PEM_X509_INFO_READ_BIO			 116
 
 /* Reason codes. */
 #define PEM_R_BAD_BASE64_DECODE				 100
@@ -307,9 +395,15 @@ void	ERR_load_PEM_strings();
 #define PEM_R_NOT_ENCRYPTED				 106
 #define PEM_R_NOT_PROC_TYPE				 107
 #define PEM_R_NO_START_LINE				 108
-#define PEM_R_READ_KEY					 109
-#define PEM_R_SHORT_HEADER				 110
-#define PEM_R_UNSUPPORTED_CIPHER			 111
-#define PEM_R_UNSUPPORTED_ENCRYPTION			 112
+#define PEM_R_PROBLEMS_GETTING_PASSWORD			 109
+#define PEM_R_PUBLIC_KEY_NO_RSA				 110
+#define PEM_R_READ_KEY					 111
+#define PEM_R_SHORT_HEADER				 112
+#define PEM_R_UNSUPPORTED_CIPHER			 113
+#define PEM_R_UNSUPPORTED_ENCRYPTION			 114
 
+#ifdef  __cplusplus
+}
 #endif
+#endif
+
