@@ -1,5 +1,5 @@
-/* lib/pem/pem_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/pem/pem_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,20 +46,24 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
+#include "buffer.h"
 #include "objects.h"
 #include "rsa.h"
 #include "envelope.h"
 #include "rand.h"
 #include "x509.h"
 #include "pem.h"
-#include "buffer.h"
 #include "des.h"
 
-char *PEM_version="PEM part of SSLeay 0.5.2 29-Apr-1996";
+char *PEM_version="PEM part of SSLeay 0.6.0 21-Jun-1996";
 
 #define MIN_LENGTH	4
 
+/* PEMerr(PEM_F_PEM_WRITE_BIO,ERR_R_MALLOC_FAILURE);
+ * PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE);
+ */
+
 #ifndef NOPROTO
 static void PEM_proc_type(char *buf, int type);
 static void PEM_dek_info(char *buf, char *type, int len, char *str);
@@ -77,28 +81,27 @@ char *buf;
 int num;
 int w;
 	{
-	int i,j;
-
-#ifdef _Windows
+#ifdef WIN16
 	/* We should not ever call the default callback routine from
 	 * windows. */
 	PEMerr(PEM_F_DEF_CALLBACK,ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
 	return(-1);
 #else
+	int i,j;
 
 	for (;;)
 		{
 		i=EVP_read_pw_string(buf,num,"Enter PEM pass phrase:",w);
 		if (i != 0)
 			{
-			Fprintf(stderr,"problems getting password\n");
+			PEMerr(PEM_F_DEF_CALLBACK,PEM_R_PROBLEMS_GETTING_PASSWORD);
 			memset(buf,0,(unsigned int)num);
 			return(-1);
 			}
 		j=strlen(buf);
 		if (j < MIN_LENGTH)
 			{
-			Fprintf(stderr,"phrase is too short, needs to be at least %d chars\n",MIN_LENGTH);
+			fprintf(stderr,"phrase is too short, needs to be at least %d chars\n",MIN_LENGTH);
 			}
 		else
 			break;
@@ -150,11 +153,34 @@ char *str;
 	buf[j+i*2+1]='\0';
 	}
 
+#ifndef WIN16
 char *PEM_ASN1_read(d2i,name,fp, x, cb)
 char *(*d2i)();
 char *name;
 FILE *fp;
 char **x;
+int (*cb)();
+	{
+        BIO *b;
+        char *ret;
+
+        if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		PEMerr(PEM_F_PEM_ASN1_READ,ERR_R_BUF_LIB);
+                return(0);
+		}
+        BIO_set_fp(b,fp,BIO_NOCLOSE);
+        ret=PEM_ASN1_read_bio(d2i,name,b,x,cb);
+        BIO_free(b);
+        return(ret);
+	}
+#endif
+
+char *PEM_ASN1_read_bio(d2i,name,bp, x, cb)
+char *(*d2i)();
+char *name;
+BIO *bp;
+char **x;
 int (*cb)();
 	{
 	EVP_CIPHER_INFO cipher;
@@ -165,12 +191,16 @@ int (*cb)();
 
 	for (;;)
 		{
-		if (!PEM_read(fp,&nm,&header,&data,&len)) return(NULL);
+		if (!PEM_read_bio(bp,&nm,&header,&data,&len)) return(NULL);
 		if (	(strcmp(nm,name) == 0) ||
+			((strcmp(nm,PEM_STRING_RSA) == 0) &&
+			 (strcmp(name,PEM_STRING_EVP_PKEY) == 0)) ||
+			((strcmp(nm,PEM_STRING_DSA) == 0) &&
+			 (strcmp(name,PEM_STRING_EVP_PKEY) == 0)) ||
 			((strcmp(nm,PEM_STRING_X509_OLD) == 0) &&
-			(strcmp(name,PEM_STRING_X509) == 0)) ||
+			 (strcmp(name,PEM_STRING_X509) == 0)) ||
 			((strcmp(nm,PEM_STRING_X509_REQ_OLD) == 0) &&
-			(strcmp(name,PEM_STRING_X509_REQ) == 0))
+			 (strcmp(name,PEM_STRING_X509_REQ) == 0))
 			)
 			break;
 		free(nm);
@@ -180,9 +210,17 @@ int (*cb)();
 	if (!PEM_get_EVP_CIPHER_INFO(header,&cipher)) goto err;
 	if (!PEM_do_header(&cipher,data,&len,cb)) goto err;
 	p=data;
-	ret=d2i(x,&p,len);
+	if (strcmp(name,PEM_STRING_EVP_PKEY) == 0)
+		{
+		if (strcmp(nm,PEM_STRING_RSA) == 0)
+			ret=d2i(EVP_PKEY_RSA,x,&p,len);
+		else if (strcmp(nm,PEM_STRING_DSA) == 0)
+			ret=d2i(EVP_PKEY_DSA,x,&p,len);
+		}
+	else	
+		ret=d2i(x,&p,len);
 	if (ret == NULL)
-		PEMerr(PEM_F_PEM_ASN1_READ,ERR_R_ASN1_LIB);
+		PEMerr(PEM_F_PEM_ASN1_READ_BIO,ERR_R_ASN1_LIB);
 err:
 	free(nm);
 	free(header);
@@ -190,6 +228,7 @@ err:
 	return(ret);
 	}
 
+#ifndef WIN16
 int PEM_ASN1_write(i2d,name,fp, x, enc, kstr, klen, callback)
 int (*i2d)();
 char *name;
@@ -198,10 +237,35 @@ char *x;
 EVP_CIPHER *enc;
 unsigned char *kstr;
 int klen;
+int (*callback)();
+        {
+        BIO *b;
+        int ret;
+
+        if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		PEMerr(PEM_F_PEM_ASN1_WRITE,ERR_R_BUF_LIB);
+                return(0);
+		}
+        BIO_set_fp(b,fp,BIO_NOCLOSE);
+        ret=PEM_ASN1_write_bio(i2d,name,b,x,enc,kstr,klen,callback);
+        BIO_free(b);
+        return(ret);
+        }
+#endif
+
+int PEM_ASN1_write_bio(i2d,name,bp, x, enc, kstr, klen, callback)
+int (*i2d)();
+char *name;
+BIO *bp;
+char *x;
+EVP_CIPHER *enc;
+unsigned char *kstr;
+int klen;
 int (*callback)();
 	{
 	EVP_CIPHER_CTX ctx;
-	int dsize=0,i,j,k,ret=0;
+	int dsize=0,i,j,ret=0;
 	unsigned char *p,*data=NULL;
 	char *objstr=NULL;
 #define PEM_BUFSIZE	1024
@@ -214,7 +278,7 @@ int (*callback)();
 		objstr=OBJ_nid2sn(enc->type);
 		if (objstr == NULL)
 			{
-			PEMerr(PEM_F_PEM_WRITE_RSA,PEM_R_UNSUPPORTED_CIPHER);
+			PEMerr(PEM_F_PEM_ASN1_WRITE_BIO,PEM_R_UNSUPPORTED_CIPHER);
 			goto err;
 			}
 		}
@@ -224,7 +288,7 @@ int (*callback)();
 	data=(unsigned char *)malloc((unsigned int)dsize+20);
 	if (data == NULL)
 		{
-		PEMerr(PEM_F_PEM_WRITE_RSA,ERR_R_MALLOC_FAILURE);
+		PEMerr(PEM_F_PEM_ASN1_WRITE_BIO,ERR_R_MALLOC_FAILURE);
 		goto err;
 		}
 	p=data;
@@ -240,7 +304,7 @@ int (*callback)();
 				klen=(*callback)(buf,PEM_BUFSIZE,1);
 			if (klen <= 0)
 				{
-				PEMerr(PEM_F_PEM_WRITE_RSA,PEM_R_READ_KEY);
+				PEMerr(PEM_F_PEM_ASN1_WRITE_BIO,PEM_R_READ_KEY);
 				goto err;
 				}
 			kstr=(unsigned char *)buf;
@@ -249,14 +313,14 @@ int (*callback)();
 		RAND_bytes(iv,8);	/* Generate a salt */
 		/* The 'iv' is used as the iv and as a salt.  It is
 		 * NOT taken from the BytesToKey function */
-		EVP_BytesToKey(enc,EVP_md5,iv,kstr,klen,1,key,NULL);
+		EVP_BytesToKey(enc,EVP_md5(),iv,kstr,klen,1,key,NULL);
 
 		if (kstr == (unsigned char *)buf) memset(buf,0,PEM_BUFSIZE);
 
 		buf[0]='\0';
 		PEM_proc_type(buf,PEM_TYPE_ENCRYPTED);
 		PEM_dek_info(buf,objstr,8,(char *)iv);
-		k=strlen(buf);
+		/* k=strlen(buf); */
 	
 		EVP_EncryptInit(&ctx,enc,key,iv);
 		EVP_EncryptUpdate(&ctx,data,&j,data,i);
@@ -269,7 +333,7 @@ int (*callback)();
 		ret=1;
 		buf[0]='\0';
 		}
-	i=PEM_write(fp,name,buf,data,i);
+	i=PEM_write_bio(bp,name,buf,data,i);
 	if (i <= 0) ret=0;
 err:
 	memset(key,0,sizeof(key));
@@ -305,10 +369,10 @@ int (*callback)();
 		PEMerr(PEM_F_PEM_DO_HEADER,PEM_R_BAD_PASSWORD_READ);
 		return(0);
 		}
-	EVP_BytesToKey(cipher->cipher,EVP_md5,&(cipher->iv[0]),
+	EVP_BytesToKey(cipher->cipher,EVP_md5(),&(cipher->iv[0]),
 		(unsigned char *)buf,klen,1,key,NULL);
 
-	j=len;
+	j=(int)len;
 	EVP_DecryptInit(&ctx,cipher->cipher,key,&(cipher->iv[0]));
 	EVP_DecryptUpdate(&ctx,data,&i,data,j);
 	o=EVP_DecryptFinal(&ctx,&(data[i]),&j);
@@ -335,19 +399,19 @@ EVP_CIPHER_INFO *cipher;
 	if ((header == NULL) || (*header == '\0') || (*header == '\n'))
 		return(1);
 	if (strncmp(header,"Proc-Type: ",11) != 0)
-		{ PEMerr(PEM_F_PEM_EVP_CIPHER_INFO,PEM_R_NOT_PROC_TYPE); return(0); }
+		{ PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_NOT_PROC_TYPE); return(0); }
 	header+=11;
 	if (*header != '4') return(0); header++;
 	if (*header != ',') return(0); header++;
 	if (strncmp(header,"ENCRYPTED",9) != 0)
-		{ PEMerr(PEM_F_PEM_EVP_CIPHER_INFO,PEM_R_NOT_ENCRYPTED); return(0); }
+		{ PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_NOT_ENCRYPTED); return(0); }
 	for (; (*header != '\n') && (*header != '\0'); header++)
 		;
 	if (*header == '\0')
-		{ PEMerr(PEM_F_PEM_EVP_CIPHER_INFO,PEM_R_SHORT_HEADER); return(0); }
+		{ PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_SHORT_HEADER); return(0); }
 	header++;
 	if (strncmp(header,"DEK-Info: ",10) != 0)
-		{ PEMerr(PEM_F_PEM_EVP_CIPHER_INFO,PEM_R_NOT_DEK_INFO); return(0); }
+		{ PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_NOT_DEK_INFO); return(0); }
 	header+=10;
 
 	p=header;
@@ -367,18 +431,18 @@ EVP_CIPHER_INFO *cipher;
 	switch (o)
 		{
 	case NID_des_cbc:
-		cipher->cipher=enc=EVP_des_cbc;
+		cipher->cipher=enc=EVP_des_cbc();
 		break;
 	case NID_des_ede3_cbc:
-		cipher->cipher=EVP_des_ede3_cbc;
+		cipher->cipher=EVP_des_ede3_cbc();
 		break;
-#ifndef NOIDEA
+#ifndef NO_IDEA
 	case NID_idea_cbc:
-		cipher->cipher=EVP_idea_cbc;
+		cipher->cipher=EVP_idea_cbc();
 		break;
 #endif
 	default:
-		PEMerr(PEM_F_PEM_EVP_CIPHER_INFO,PEM_R_UNSUPPORTED_ENCRYPTION);
+		PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_UNSUPPORTED_ENCRYPTION);
 		return(0);
 		}
 	if (!load_iv((unsigned char **)&header,&(cipher->iv[0]),8)) return(0);
@@ -417,80 +481,140 @@ int num;
 	return(1);
 	}
 
+#ifndef WIN16
 int PEM_write(fp, name, header, data,len)
 FILE *fp;
 char *name;
 char *header;
 unsigned char *data;
+long len;
+        {
+        BIO *b;
+        int ret;
+
+        if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		PEMerr(PEM_F_PEM_WRITE,ERR_R_BUF_LIB);
+                return(0);
+		}
+        BIO_set_fp(b,fp,BIO_NOCLOSE);
+        ret=PEM_write_bio(b, name, header, data,len);
+        BIO_free(b);
+        return(ret);
+        }
+#endif
+
+int PEM_write_bio(bp, name, header, data,len)
+BIO *bp;
+char *name;
+char *header;
+unsigned char *data;
 long len;
 	{
-	int n,i,outl;
+	int nlen,n,i,j,outl;
 	unsigned char *buf;
 	EVP_ENCODE_CTX ctx;
+	int reason=ERR_R_BUF_LIB;
 	
 	EVP_EncodeInit(&ctx);
+	nlen=strlen(name);
 
-	Fprintf(fp,"-----BEGIN %s-----\n",name);
-	Fwrite(header,strlen(header),1,fp);
-	if (header[0] != '\0')
-		Fprintf(fp,"\n");
+	if (	(BIO_write(bp,"-----BEGIN ",11) != 11) ||
+		(BIO_write(bp,name,nlen) != nlen) ||
+		(BIO_write(bp,"-----\n",6) != 6))
+		goto err;
+		
+	i=strlen(header);
+	if (i > 0)
+		{
+		if (	(BIO_write(bp,header,i) != i) ||
+			(BIO_write(bp,"\n",1) != 1))
+			goto err;
+		}
 
 	buf=(unsigned char *)malloc(PEM_BUFSIZE*8);
 	if (buf == NULL)
 		{
-		PEMerr(PEM_F_PEM_WRITE,ERR_R_MALLOC_FAILURE);
-		return(0);
+		reason=ERR_R_MALLOC_FAILURE;
+		goto err;
 		}
 
-	i=0;
+	i=j=0;
 	while (len > 0)
 		{
-		n=((len>(PEM_BUFSIZE*5))?(PEM_BUFSIZE*5):len);
-		EVP_EncodeUpdate(&ctx,buf,&outl,&(data[i]),n);
-		if (outl)
-			Fwrite(buf,(unsigned int)outl,1,fp);
+		n=(int)((len>(PEM_BUFSIZE*5))?(PEM_BUFSIZE*5):len);
+		EVP_EncodeUpdate(&ctx,buf,&outl,&(data[j]),n);
+		if ((outl) && (BIO_write(bp,(char *)buf,outl) != outl))
+			goto err;
 		i+=outl;
 		len-=n;
+		j+=n;
 		}
 	EVP_EncodeFinal(&ctx,buf,&outl);
-	if (outl > 0)
-		Fwrite(buf,(unsigned int)outl,1,fp);
+	if ((outl > 0) && (BIO_write(bp,(char *)buf,outl) != outl)) goto err;
 	free(buf);
-	Fprintf(fp,"-----END %s-----\n",name);
+	if (	(BIO_write(bp,"-----END ",9) != 9) ||
+		(BIO_write(bp,name,nlen) != nlen) ||
+		(BIO_write(bp,"-----\n",6) != 6))
+		goto err;
 	return(i+outl);
+err:
+	PEMerr(PEM_F_PEM_WRITE_BIO,reason);
+	return(0);
 	}
 
-int PEM_read(fp, name, header, data, len)
+#ifndef WIN16
+int PEM_read(fp, name, header, data,len)
 FILE *fp;
 char **name;
 char **header;
 unsigned char **data;
+long *len;
+        {
+        BIO *b;
+        int ret;
+
+        if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		PEMerr(PEM_F_PEM_READ,ERR_R_BUF_LIB);
+                return(0);
+		}
+        BIO_set_fp(b,fp,BIO_NOCLOSE);
+        ret=PEM_read_bio(b, name, header, data,len);
+        BIO_free(b);
+        return(ret);
+        }
+#endif
+
+int PEM_read_bio(bp, name, header, data, len)
+BIO *bp;
+char **name;
+char **header;
+unsigned char **data;
 long *len;
 	{
 	EVP_ENCODE_CTX ctx;
 	int end=0,i,k,bl=0,hl=0,nohead=0;
 	char buf[256];
-	BUFFER *nameB;
-	BUFFER *headerB;
-	BUFFER *dataB,*tmpB;
+	BUF_MEM *nameB;
+	BUF_MEM *headerB;
+	BUF_MEM *dataB,*tmpB;
 	
-	nameB=BUF_new();
-	headerB=BUF_new();
-	dataB=BUF_new();
+	nameB=BUF_MEM_new();
+	headerB=BUF_MEM_new();
+	dataB=BUF_MEM_new();
 	if ((nameB == NULL) || (headerB == NULL) || (dataB == NULL))
 		{
-		PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE);
+		PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE);
 		return(0);
 		}
 
 	for (;;)
 		{
-		buf[0]='\0';
-		Fgets(buf,256,fp);
-
-		if (buf[0] == '\0')
+		i=BIO_gets(bp,buf,256);
+		if (i <= 0)
 			{
-			PEMerr(PEM_F_PEM_READ,PEM_R_NO_START_LINE);
+			PEMerr(PEM_F_PEM_READ_BIO,PEM_R_NO_START_LINE);
 			goto err;
 			}
 		if (strncmp(buf,"-----BEGIN ",11) == 0)
@@ -498,9 +622,9 @@ long *len;
 			i=strlen(&(buf[11]));
 			if (strncmp(&(buf[11+i-6]),"-----\n",6) != 0)
 				continue;
-			if (!BUF_grow(nameB,i+9))
+			if (!BUF_MEM_grow(nameB,i+9))
 				{
-				PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE);
+				PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE);
 				goto err;
 				}
 			strncpy(nameB->data,&(buf[11]),(unsigned int)i-6);
@@ -509,18 +633,16 @@ long *len;
 			}
 		}
 	hl=0;
-	if (!BUF_grow(headerB,256))
-		{ PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE); goto err; }
+	if (!BUF_MEM_grow(headerB,256))
+		{ PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE); goto err; }
 	headerB->data[0]='\0';
 	for (;;)
 		{
-		buf[0]='\0';
-		Fgets(buf,256,fp);
-		if (buf[0] == '\0') break;
+		i=BIO_gets(bp,buf,256);
+		if (i <= 0) break;
 		if (buf[0] == '\n') break;
-		i=strlen(buf);
-		if (!BUF_grow(headerB,hl+i+9))
-			{ PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE); goto err; }
+		if (!BUF_MEM_grow(headerB,hl+i+9))
+			{ PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE); goto err; }
 		if (strncmp(buf,"-----END ",9) == 0)
 			{
 			nohead=1;
@@ -532,24 +654,22 @@ long *len;
 		}
 
 	bl=0;
-	if (!BUF_grow(dataB,1024))
-		{ PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE); goto err; }
+	if (!BUF_MEM_grow(dataB,1024))
+		{ PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE); goto err; }
 	dataB->data[0]='\0';
 	if (!nohead)
 		{
 		for (;;)
 			{
-			buf[0]='\0';
-			Fgets(buf,256,fp);
-			if (buf[0] == '\0') break;
-			i=strlen(buf);
+			i=BIO_gets(bp,buf,256);
+			if (i <= 0) break;
 			if (i != 65) end=1;
 			if (strncmp(buf,"-----END ",9) == 0)
 				break;
 			if (i > 65) break;
-			if (!BUF_grow(dataB,i+bl+9))
+			if (!BUF_MEM_grow(dataB,i+bl+9))
 				{
-				PEMerr(PEM_F_PEM_READ,ERR_R_MALLOC_FAILURE);
+				PEMerr(PEM_F_PEM_READ_BIO,ERR_R_MALLOC_FAILURE);
 				goto err;
 				}
 			strncpy(&(dataB->data[bl]),buf,(unsigned int)i);
@@ -558,7 +678,7 @@ long *len;
 			if (end)
 				{
 				buf[0]='\0';
-				Fgets(buf,256,fp);
+				BIO_gets(bp,buf,256);
 				break;
 				}
 			}
@@ -575,7 +695,7 @@ long *len;
 		(strncmp(nameB->data,&(buf[9]),(unsigned int)i) != 0) ||
 		(strncmp(&(buf[9+i]),"-----\n",6) != 0))
 		{
-		PEMerr(PEM_F_PEM_READ,PEM_R_BAD_END_LINE);
+		PEMerr(PEM_F_PEM_READ_BIO,PEM_R_BAD_END_LINE);
 		goto err;
 		}
 	
@@ -583,9 +703,17 @@ long *len;
 	i=EVP_DecodeUpdate(&ctx,
 		(unsigned char *)dataB->data,&bl,
 		(unsigned char *)dataB->data,bl);
-	if (i < 0) { PEMerr(PEM_F_PEM_READ,PEM_R_BAD_BASE64_DECODE); goto err; }
+	if (i < 0)
+		{
+		PEMerr(PEM_F_PEM_READ_BIO,PEM_R_BAD_BASE64_DECODE);
+		goto err;
+		}
 	i=EVP_DecodeFinal(&ctx,(unsigned char *)&(dataB->data[bl]),&k);
-	if (i < 0) { PEMerr(PEM_F_PEM_READ,PEM_R_BAD_BASE64_DECODE); goto err; }
+	if (i < 0)
+		{
+		PEMerr(PEM_F_PEM_READ_BIO,PEM_R_BAD_BASE64_DECODE);
+		goto err;
+		}
 	bl+=k;
 
 	if (bl == 0) goto err;
@@ -598,9 +726,9 @@ long *len;
 	free(dataB);
 	return(1);
 err:
-	BUF_free(nameB);
-	BUF_free(headerB);
-	BUF_free(dataB);
+	BUF_MEM_free(nameB);
+	BUF_MEM_free(headerB);
+	BUF_MEM_free(dataB);
 	return(0);
 	}
 
