@@ -1,5 +1,5 @@
-/* lib/rand/md5_rand.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/rand/md5_rand.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,7 +46,7 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include <sys/types.h>
 #include <time.h>
 #include "md5.h"
@@ -67,7 +67,7 @@ static unsigned char state[STATE_SIZE];
 static unsigned char md[MD5_DIGEST_LENGTH];
 static int count=0;
 
-char *RAND_version="RAND part of SSLeay 0.5.2 29-Apr-1996";
+char *RAND_version="RAND part of SSLeay 0.6.0 21-Jun-1996";
 
 void RAND_cleanup()
 	{
@@ -133,6 +133,9 @@ int num;
 	MD5_CTX m;
 	static int init=1;
 	unsigned long l;
+#ifdef DEVRANDOM
+	FILE *fh;
+#endif
 
 #ifdef PREDICT
 	{
@@ -150,7 +153,7 @@ int num;
 		/* put in some default random data, we need more than
 		 * just this */
 		RAND_seed((unsigned char *)&m,sizeof(m));
-#ifndef WIN32
+#ifndef MSDOS
 		l=getpid();
 		RAND_seed((unsigned char *)&l,sizeof(l));
 		l=getuid();
@@ -158,6 +161,29 @@ int num;
 #endif
 		l=time(NULL);
 		RAND_seed((unsigned char *)&l,sizeof(l));
+
+#ifdef DEVRANDOM
+		/* 
+		 * Use a random entropy pool device.
+		 * Linux 1.3.x and FreeBSD-Current has 
+		 * this. Use /dev/urandom if you can
+		 * as /dev/random will block if it runs out
+		 * of random entries.
+		 */
+		if ((fh = fopen(DEVRANDOM, "r")) != NULL)
+			{
+			unsigned char buf[32];
+
+			fread((unsigned char *)buf,1,32,fh);
+			/* we don't care how many bytes we read,
+			 * we will just copy the 'stack' if there is
+			 * nothing else :-) */
+			fclose(fh);
+			RAND_seed(b,32);
+			memset(buf,0,32);
+			}
+#endif
+
 #ifdef PURIFY
 		memset(state,0,STATE_SIZE);
 		memset(md,0,MD5_DIGEST_LENGTH);
