@@ -1,5 +1,5 @@
-/* lib/rand/randfile.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/rand/randfile.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,14 +46,15 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include <sys/stat.h>
 #include <sys/types.h>
 #include "rand.h"
 
 #define BUFSIZE	1024
 #define RAND_DATA 1024
-#define RFILE	".rand"
+
+/* #define RFILE ".rand" - defined in ../../e_os.h */
 
 int RAND_load_file(file,bytes)
 char *file;
@@ -140,9 +141,9 @@ int size;
 	else
 		{
 		s=getenv("HOME");
-		if (s == NULL) return(NULL);
+		if (s == NULL) return(RFILE);
 		if (((int)(strlen(s)+strlen(RFILE)+2)) > size)
-			return(NULL);
+			return(RFILE);
 		strcpy(buf,s);
 		strcat(buf,"/");
 		strcat(buf,RFILE);
