@@ -1,5 +1,5 @@
-/* lib/rsa/rsa_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/rsa/rsa_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,11 +46,11 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "bn.h"
 #include "rsa.h"
 
-char *RSA_version="RSA part of SSLeay 0.5.2 29-Apr-1996";
+char *RSA_version="RSA part of SSLeay 0.6.0 21-Jun-1996";
 
 RSA *RSA_new()
 	{
@@ -62,6 +62,7 @@ RSA *RSA_new()
 		RSAerr(RSA_F_RSA_NEW,ERR_R_MALLOC_FAILURE);
 		return(NULL);
 		}
+	ret->pad=0;
 	ret->version=0;
 	ret->n=NULL;
 	ret->e=NULL;
@@ -79,9 +80,15 @@ void RSA_free(r)
 RSA *r;
 	{
 	if (r == NULL) return;
-	CS_BEGIN;
-	if (--r->references > 0) return;
-	CS_END;
+
+	CRYPTO_w_lock(CRYPTO_LOCK_RSA);
+	if (--r->references > 0)
+		{
+		CRYPTO_w_unlock(CRYPTO_LOCK_RSA);
+		return;
+		}
+	CRYPTO_w_unlock(CRYPTO_LOCK_RSA);
+
 	if (r->n != NULL) BN_clear_free(r->n);
 	if (r->e != NULL) BN_clear_free(r->e);
 	if (r->d != NULL) BN_clear_free(r->d);
