@@ -1,5 +1,5 @@
-/* lib/sha/sha1dgst.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/sha/sha1dgst.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -50,7 +50,7 @@
 #include "sha.h"
 #include "sha_locl.h"
 
-char *SHA1_version="SHA1 part of SSLeay 0.5.2 29-Apr-1996";
+char *SHA1_version="SHA1 part of SSLeay 0.6.0 21-Jun-1996";
 
 /* Implemented from SHA-1 document - The Secure Hash Algorithm
  */
@@ -67,9 +67,9 @@ char *SHA1_version="SHA1 part of SSLeay 0.5.2 29-Apr-1996";
 #define K_60_79 0xca62c1d6L
 
 #ifndef NOPROTO
-static void do_block(SHA_CTX *c, register unsigned long *p);
+static void sha1_block(SHA_CTX *c, register unsigned long *p);
 #else
-static void do_block();
+static void sha1_block();
 #endif
 
 void SHA1_Init(c)
@@ -120,7 +120,7 @@ unsigned long len;
 				}
 			len-=(SHA_CBLOCK-c->num);
 
-			do_block(c,p);
+			sha1_block(c,p);
 			c->num=0;
 			/* drop through and do the rest */
 			}
@@ -128,7 +128,7 @@ unsigned long len;
 			{
 			int ew,ec;
 
-			c->num+=len;
+			c->num+=(int)len;
 			if ((sc+len) < 4) /* ugly, add char's to a word */
 				{
 				l= p[sw];
@@ -158,6 +158,20 @@ unsigned long len;
 	p=c->data;
 	while (len >= SHA_CBLOCK)
 		{
+#if defined(B_ENDIAN) || defined(L_ENDIAN)
+		memcpy(p,data,SHA_CBLOCK);
+		data+=SHA_CBLOCK;
+#ifdef L_ENDIAN
+		for (sw=(SHA_LBLOCK/4); sw; sw--)
+			{
+			Endian_Reverse32(p[0]);
+			Endian_Reverse32(p[1]);
+			Endian_Reverse32(p[2]);
+			Endian_Reverse32(p[3]);
+			p+=4;
+			}
+#endif
+#else
 		for (sw=(SHA_BLOCK/4); sw; sw--)
 			{
 			c2nl(data,l); *(p++)=l;
@@ -165,21 +179,23 @@ unsigned long len;
 			c2nl(data,l); *(p++)=l;
 			c2nl(data,l); *(p++)=l;
 			}
+#endif
 		p=c->data;
-		do_block(c,p);
+		sha1_block(c,p);
 		len-=SHA_CBLOCK;
 		}
-	ew=(len>>2);
-	ec=(len&0x03);
+	ec=(int)len;
+	c->num=ec;
+	ew=(ec>>2);
+	ec&=0x03;
 
 	for (sw=0; sw < ew; sw++)
 		{ c2nl(data,l); p[sw]=l; }
 	c2nl_p(data,l,ec);
 	p[sw]=l;
-	c->num=len;
 	}
 
-static void do_block(c, X)
+static void sha1_block(c, X)
 SHA_CTX *c;
 register unsigned long *X;
 	{
@@ -296,6 +312,9 @@ SHA_CTX *c;
 	p=c->data;
 	j=c->num;
 	i=j>>2;
+#ifdef PURIFY
+	if ((j&0x03) == 0) p[i]=0;
+#endif
 	l=p[i];
 	p_c2nl(cp,l,j&0x03);
 	p[i]=l;
@@ -305,14 +324,14 @@ SHA_CTX *c;
 		{
 		for (; i<SHA_LBLOCK; i++)
 			p[i]=0;
-		do_block(c,p);
+		sha1_block(c,p);
 		i=0;
 		}
 	for (; i<(SHA_LBLOCK-2); i++)
 		p[i]=0;
 	p[SHA_LBLOCK-2]=c->Nh;
 	p[SHA_LBLOCK-1]=c->Nl;
-	do_block(c,p);
+	sha1_block(c,p);
 	cp=md;
 	l=c->h0; nl2c(l,cp);
 	l=c->h1; nl2c(l,cp);
@@ -320,7 +339,7 @@ SHA_CTX *c;
 	l=c->h3; nl2c(l,cp);
 	l=c->h4; nl2c(l,cp);
 
-	/* clear stuff, do_block may be leaving some stuff on the stack
+	/* clear stuff, sha1_block may be leaving some stuff on the stack
 	 * but I'm not worried :-) */
 	c->num=0;
 /*	memset((char *)&c,0,sizeof(c));*/
