@@ -1,5 +1,5 @@
-/* lib/sha/sha_locl.h */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/sha/sha_locl.h */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -45,8 +45,8 @@
  * [including the GNU Public Licence.]
  */
 
-#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 /* one or the other needs to be defined */
 #ifndef SHA_1 /* FIPE 180-1 */
@@ -56,15 +56,8 @@
 #define ULONG	unsigned long
 #define UCHAR	unsigned char
 #define UINT	unsigned int
-#if defined(__STDC__) || defined(VMS) || defined(M_XENIX) || defined(MSDOS)
-#include <string.h>
-#define bcopy(b1,b2,len) memcpy(b2, b1, (size_t)(len))
-#define bzero(b,len) memset(b, 0, (size_t)(len))
-#define bcmp(b1,b2,len) memcmp(b1, b2, (size_t)(len))
-#define index(s1,char) strchr(s1,char)
-#endif
 
-#if defined(NOCONST)
+#ifdef NOCONST
 #define const
 #endif
 
@@ -111,20 +104,40 @@
 
 /* I have taken some of this code from my MD5 implementation */
 
-#ifndef WIN32
-#define ROTATE(a,n)     (((a)<<(n))|(((a)&0xffffffff)>>(32-(n))))
+#if defined(WIN32)
+#define ROTATE(a,n)     _lrotl(a,n)
 #else
-#define ROTATE(a,n)     _rotl(a,n)
+#define ROTATE(a,n)     (((a)<<(n))|(((a)&0xffffffff)>>(32-(n))))
+#endif
+
+/* A nice byte order reversal from Wei Dai <weidai@eskimo.com> */
+#if defined(WIN32)
+/* 5 instructions with rotate instruction, else 9 */
+#define Endian_Reverse32(a) \
+	{ \
+	unsigned long l=(a); \
+	((ROTATE(l,8)&0xFF00FF00)|(ROTATE(l,24)&0x00FF00FF)); \
+	}
+#else
+/* 6 instructions with rotate instruction, else 8 */
+#define Endian_Reverse32(a) \
+	{ \
+	unsigned long l=(a); \
+	l=(((l&0xFF00FF00)>>8L)|((l&0x00FF00FF)<<8L)); \
+	(a)=ROTATE(l,16L); \
+	}
 #endif
 
 /* As  pointed out by Wei Dai <weidai@eskimo.com>, F() below can be
  * simplified to the code in F_00_19.  Wei attributes these optimisations
  * to Peter Gutmann's SHS code, and he attributes it to Rich Schroeppel.
  * #define F(x,y,z) (((x) & (y))  |  ((~(x)) & (z)))
+ * I've just become aware of another tweak to be made, again from Wei Dai,
+ * in F_40_59, (x&a)|(y&a) -> (x|y)&a
  */
 #define	F_00_19(b,c,d)	((((c) ^ (d)) & (b)) ^ (d)) 
 #define	F_20_39(b,c,d)	((b) ^ (c) ^ (d))
-#define F_40_59(b,c,d)	(((b) & (c)) | ((b) & (d)) | ((c) & (d)))
+#define F_40_59(b,c,d)	(((b) & (c)) | (((b)|(c)) & (d))) 
 #define	F_60_79(b,c,d)	F_20_39(b,c,d)
 
 #ifdef SHA_0
