@@ -1,5 +1,5 @@
-/* lib/stack/stack.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/stack/stack.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -52,12 +52,13 @@
  * 1.0 eay - First version 29/07/92
  */
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "stack.h"
 
+#undef MIN_NODES
 #define MIN_NODES	4
 
-char *STACK_version="STACK part of SSLeay 0.5.2 29-Apr-1996";
+char *STACK_version="STACK part of SSLeay 0.6.0 21-Jun-1996";
 
 #ifndef NOPROTO
 #define	FP_ICC	(int (*)(const void *,const void *))
@@ -65,6 +66,7 @@ char *STACK_version="STACK part of SSLeay 0.5.2 29-Apr-1996";
 #define FP_ICC
 #endif
 
+#include <errno.h>
 STACK *sk_new(c)
 int (*c)();
 	{
@@ -122,7 +124,7 @@ char *sk_delete_ptr(st,p)
 STACK *st;
 char *p;
 	{
-	unsigned int i;
+	int i;
 
 	for (i=0; i<st->num; i++)
 		if (st->data[i] == p)
@@ -136,10 +138,10 @@ int loc;
 	{
 	char *ret;
 
-	if ((st->num == 0) || (loc < 0) || ((unsigned int)loc >= st->num)) return(NULL);
+	if ((st->num == 0) || (loc < 0) || (loc >= st->num)) return(NULL);
 
 	ret=st->data[loc];
-	if ((unsigned int)loc != st->num-1)
+	if (loc != st->num-1)
 		memcpy( &(st->data[loc]),
 			&(st->data[loc+1]),
 			sizeof(char *)*(st->num-loc-1));
@@ -152,7 +154,7 @@ STACK *st;
 char *data;
 	{
 	char **r;
-	unsigned int i;
+	int i;
 	int (*comp_func)();
 
 	if (st->comp == NULL)
@@ -213,7 +215,7 @@ void sk_pop_free(st,func)
 STACK *st;
 void (*func)();
 	{
-	unsigned int i;
+	int i;
 
 	if (st == NULL) return;
 	for (i=0; i<st->num; i++)
