@@ -1,5 +1,5 @@
-/* lib/txt_db/txt_db.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/txt_db/txt_db.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,15 +47,17 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "txt_db.h"
+#include <string.h>
 #include "buffer.h"
+#include "txt_db.h"
 
+#undef BUFSIZE
 #define BUFSIZE	512
 
-char *TXT_DB_version="TXT_DB part of SSLeay 0.5.2 29-Apr-1996";
+char *TXT_DB_version="TXT_DB part of SSLeay 0.6.0 21-Jun-1996";
 
-TXT_DB *TXT_DB_load(in,num)
-FILE *in;
+TXT_DB *TXT_DB_read(in,num)
+BIO *in;
 int num;
 	{
 	TXT_DB *ret=NULL;
@@ -66,10 +68,10 @@ int num;
 	int size=BUFSIZE;
 	int offset=0;
 	char *p,**pp,*f;
-	BUFFER *buf=NULL;
+	BUF_MEM *buf=NULL;
 
-	if ((buf=BUF_new()) == NULL) goto err;
-	if (!BUF_grow(buf,size)) goto err;
+	if ((buf=BUF_MEM_new()) == NULL) goto err;
+	if (!BUF_MEM_grow(buf,size)) goto err;
 
 	if ((ret=(TXT_DB *)malloc(sizeof(TXT_DB))) == NULL)
 		goto err;
@@ -96,10 +98,10 @@ int num;
 		if (offset != 0)
 			{
 			size+=BUFSIZE;
-			if (!BUF_grow(buf,size)) goto err;
+			if (!BUF_MEM_grow(buf,size)) goto err;
 			}
 		buf->data[offset]='\0';
-		fgets(&(buf->data[offset]),size-offset,in);
+		BIO_gets(in,&(buf->data[offset]),size-offset);
 		ln++;
 		if (buf->data[offset] == '\0') break;
 		if ((offset == 0) && (buf->data[0] == '#')) continue;
@@ -143,24 +145,30 @@ int num;
 		*(p++)='\0';
 		if ((n != num) || (*f != '\0'))
 			{
+#ifndef WIN16	/* temporaty fix :-( */
 			fprintf(stderr,"wrong number of fields on line %ld\n",ln);
+#endif
 			er=2;
 			goto err;
 			}
 		pp[n]=p;
 		if (!sk_push(ret->data,(char *)pp))
 			{
+#ifndef WIN16	/* temporaty fix :-( */
 			fprintf(stderr,"failure in sk_push\n");
+#endif
 			er=2;
 			goto err;
 			}
 		}
 	er=0;
 err:
-	BUF_free(buf);
+	BUF_MEM_free(buf);
 	if (er)
 		{
+#ifndef WIN16
 		if (er == 1) fprintf(stderr,"malloc failure\n");
+#endif
 		if (ret->data != NULL) sk_free(ret->data);
 		if (ret->index != NULL) free(ret->index);
 		if (ret->qual != NULL) free((char *)ret->qual);
@@ -237,15 +245,15 @@ int (*cmp)();
 	}
 
 long TXT_DB_write(out,db)
-FILE *out;
+BIO *out;
 TXT_DB *db;
 	{
 	long i,j,n,nn,l,tot=0;
 	char *p,**pp,*f;
-	BUFFER *buf=NULL;
-	int ret=-1;
+	BUF_MEM *buf=NULL;
+	long ret=-1;
 
-	if ((buf=BUF_new()) == NULL)
+	if ((buf=BUF_MEM_new()) == NULL)
 		goto err;
 	n=sk_num(db->data);
 	nn=db->num_fields;
@@ -259,7 +267,7 @@ TXT_DB *db;
 			if (pp[j] != NULL)
 				l+=strlen(pp[j]);
 			}
-		if (!BUF_grow(buf,l*2+nn)) goto err;
+		if (!BUF_MEM_grow(buf,(int)(l*2+nn))) goto err;
 
 		p=buf->data;
 		for (j=0; j<nn; j++)
@@ -276,13 +284,13 @@ TXT_DB *db;
 			}
 		p[-1]='\n';
 		j=p-buf->data;
-		if (fwrite(buf->data,j,1,out) != 1)
+		if (BIO_write(out,buf->data,(int)j) != j)
 			goto err;
 		tot+=j;
 		}
 	ret=tot;
 err:
-	if (buf != NULL) BUF_free(buf);
+	if (buf != NULL) BUF_MEM_free(buf);
 	return(ret);
 	}
 
