@@ -1,5 +1,5 @@
-/* lib/x509/x509.h */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/x509/x509.h */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -48,12 +48,19 @@
 #ifndef HEADER_X509_H
 #define HEADER_X509_H
 
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 #include <stack.h>
 #include <asn1.h>
 
-#ifndef HEADER_RSA_H
+/*
 #include "rsa.h"
-#endif
+#include "dsa.h"
+#include "dh.h"
+*/
+#include "envelope.h"
 
 #define X509_FILETYPE_PEM                       1
 #define X509_FILETYPE_ASN1                      2
@@ -171,49 +178,23 @@ typedef struct x509_attributes_st
 		} value;
 	} X509_ATTRIBUTE;
 
-#define X509_PKEY_NONE		0
-#define X509_PKEY_ENCRYPTED	1
-#define X509_PKEY_RSA		2
-#define X509_PKEY_DH		3
-#define X509_PKEY_DSA		4
-
 typedef struct private_key_st
 	{
-	ASN1_INTEGER *version;
-	X509_ALGOR *algor;
-	ASN1_OCTET_STRING *data;
-	STACK /* X509_ATTRIBUTE */ *attributes; /* [ 0 ] */
-
-	/* What follows is 'derived' from the above */
-#ifdef HEADER_ENVELOPE_H
-	EVP_CIPHER *cipher;
-#else
-	char *cipher;
-#endif
-	int key_len;
-	char *key;
-	char *iv; /* 8 bytes */
+	int version;
+	/* The PKCS#8 data types */
+	X509_ALGOR *enc_algor;
+	ASN1_OCTET_STRING *enc_pkey;	/* encrypted pub key */
 
 	/* When decrypted, the following will not be NULL */
-	int type;
-	union	{
-		char *ptr;
-#ifdef HEADER_DH_H
-		RSA	*rsa;
-#else
-		char	*rsa;
-#endif
-#ifdef HEADER_DH_H
-		DH	*dh;
-#else
-		char	*dh;
-#endif
-#ifdef HEADER_DSA_H
-		DSA	*dsa;
-#else
-		char	*dsa;
-#endif
-		} pkey;
+	EVP_PKEY *dec_pkey;
+
+	/* used to encrypt and decrypt */
+	int key_length;
+	char *key_data;
+	int key_free;	/* true if we should auto free key_data */
+
+	/* expanded version of 'enc_algor' */
+	EVP_CIPHER_INFO cipher;
 
 	int references;
 	} X509_PKEY;
@@ -223,17 +204,33 @@ typedef struct X509_info_st
 	{
 	X509 *x509;
 	X509_CRL *crl;
-	RSA *rsa;
-	/* these are for holding RSA header and data so it can be
-	 * decrypted later, if at all */
-	EVP_CIPHER_INFO rsa_cipher;
-	unsigned char *rsa_data;
-	int rsa_len;
+	X509_PKEY *x_pkey;
+
+	EVP_CIPHER_INFO enc_cipher;
+	int enc_len;
+	char *enc_data;
 
 	int references;
 	} X509_INFO;
 #endif
-	
+
+/* The next 2 structures and their 8 routines were sent to me by
+ * Pat Richard <patr@x509.com> and are used to manipulate
+ * Netscapes spki strucutres - usefull if you are writing a CA web page
+ */
+typedef struct Netscape_spkac_st
+	{
+	X509_PUBKEY *pubkey;
+	ASN1_IA5STRING *challenge;	/* challenge sent in atlas >= PR2 */
+	} NETSCAPE_SPKAC;
+
+typedef struct Netscape_spki_st
+	{
+	NETSCAPE_SPKAC *spkac;	/* signed public key and challenge */
+	X509_ALGOR *sig_algor;
+	ASN1_BIT_STRING *signature;
+	} NETSCAPE_SPKI;
+
 #ifndef HEADER_BN_H
 #define BIGNUM 		char
 #endif
@@ -300,32 +297,42 @@ typedef struct X509_AUTH_st
 #define	VERIFY_ERR_OUT_OF_MEM				22
 #define	VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT		23
 
-#define X509_REQ_verify(a,r) ASN1_verify(i2d_X509_REQ_INFO,a->sig_alg,\
-	a->signature,(char *)a->req_info,r)
-
 #define X509_verify(a,r) ASN1_verify(i2d_X509_CINF,a->sig_alg,\
 	a->signature,(char *)a->cert_info,r)
 
+#define X509_REQ_verify(a,r) ASN1_verify(i2d_X509_REQ_INFO,a->sig_alg,\
+	a->signature,(char *)a->req_info,r)
+
 #define X509_CRL_verify(a,r) ASN1_verify(i2d_X509_CRL_INFO,a->sig_alg,\
 	a->signature,(char *)a->crl,r)
 
-#define X509_sign(x,rsa,md) \
+#define NETSCAPE_SPKI_verify(a,r) ASN1_sign(i2d_NETSCAPE_SPKAC,a->sig_algor,\
+		a->signature, (char *)a->spkac,r)
+
+#define X509_sign(x,pkey,md) \
 	ASN1_sign(i2d_X509_CINF, x->cert_info->signature, x->sig_alg, \
-		x->signature, (char *)x->cert_info,rsa,md)
+		x->signature, (char *)x->cert_info,pkey,md)
 
-#define X509_REQ_sign(x,rsa,md) \
+#define X509_REQ_sign(x,pkey,md) \
 	ASN1_sign(i2d_X509_REQ_INFO,x->sig_alg, NULL, \
-		x->signature, (char *)x->req_info,rsa,md)
+		x->signature, (char *)x->req_info,pkey,md)
 
-#define X509_CRL_sign(x,rsa,md) \
+#define X509_CRL_sign(x,pkey,md) \
 	ASN1_sign(i2d_X509_CRL_INFO,x->crl->sig_alg,x->sig_alg, \
-		x->signature, (char *)x->crl,rsa,md)
+		x->signature, (char *)x->crl,pkey,md)
+
+#define NETSCAPE_SPKI_sign(x,pkey,md) \
+	ASN1_sign(i2d_NETSCAPE_SPKAC, x->sig_algor,NULL, \
+		x->signature, (char *)x->spkac,pkey,md)
 
 #define X509_dup(x509) (X509 *)ASN1_dup(i2d_X509, \
 		(char *(*)())d2i_X509,(char *)x509)
 #define d2i_X509_fp(fp,x509) (X509 *)ASN1_d2i_fp((char *(*)())X509_new, \
 		(char *(*)())d2i_X509, (fp),(unsigned char **)(x509))
 #define i2d_X509_fp(fp,x509) ASN1_i2d_fp(i2d_X509,fp,(unsigned char *)x509)
+#define d2i_X509_bio(bp,x509) (X509 *)ASN1_d2i_bio((char *(*)())X509_new, \
+		(char *(*)())d2i_X509, (bp),(unsigned char **)(x509))
+#define i2d_X509_bio(bp,x509) ASN1_i2d_bio(i2d_X509,bp,(unsigned char *)x509)
 
 #define X509_CRL_dup(crl) (X509_CRL *)ASN1_dup(i2d_X509_CRL, \
 		(char *(*)())d2i_X509_CRL,(char *)crl)
@@ -334,6 +341,24 @@ typedef struct X509_AUTH_st
 		(unsigned char **)(crl))
 #define i2d_X509_CRL_fp(fp,crl) ASN1_i2d_fp(i2d_X509_CRL,fp,\
 		(unsigned char *)crl)
+#define d2i_X509_CRL_bio(bp,crl) (X509_CRL *)ASN1_d2i_bio((char *(*)()) \
+		X509_CRL_new,(char *(*)())d2i_X509_CRL, (bp),\
+		(unsigned char **)(crl))
+#define i2d_X509_CRL_bio(bp,crl) ASN1_i2d_bio(i2d_X509_CRL,bp,\
+		(unsigned char *)crl)
+
+#define PKCS7_dup(p7) (PKCS7 *)ASN1_dup(i2d_PKCS7, \
+		(char *(*)())d2i_PKCS7,(char *)p7)
+#define d2i_PKCS7_fp(fp,p7) (PKCS7 *)ASN1_d2i_fp((char *(*)()) \
+		PKCS7_new,(char *(*)())d2i_PKCS7, (fp),\
+		(unsigned char **)(p7))
+#define i2d_PKCS7_fp(fp,p7) ASN1_i2d_fp(i2d_PKCS7,fp,\
+		(unsigned char *)p7)
+#define d2i_PKCS7_bio(bp,p7) (PKCS7 *)ASN1_d2i_bio((char *(*)()) \
+		PKCS7_new,(char *(*)())d2i_PKCS7, (bp),\
+		(unsigned char **)(p7))
+#define i2d_PKCS7_bio(bp,p7) ASN1_i2d_bio(i2d_PKCS7,bp,\
+		(unsigned char *)p7)
 
 #define X509_REQ_dup(req) (X509_REQ *)ASN1_dup(i2d_X509_REQ, \
 		(char *(*)())d2i_X509_REQ,(char *)req)
@@ -342,6 +367,11 @@ typedef struct X509_AUTH_st
 		(unsigned char **)(req))
 #define i2d_X509_REQ_fp(fp,req) ASN1_i2d_fp(i2d_X509_REQ,fp,\
 		(unsigned char *)req)
+#define d2i_X509_REQ_bio(bp,req) (X509_REQ *)ASN1_d2i_bio((char *(*)())\
+		X509_REQ_new, (char *(*)())d2i_X509_REQ, (bp),\
+		(unsigned char **)(req))
+#define i2d_X509_REQ_bio(bp,req) ASN1_i2d_bio(i2d_X509_REQ,bp,\
+		(unsigned char *)req)
 
 #define RSAPublicKey_dup(rsa) (RSA *)ASN1_dup(i2d_RSAPublikKey, \
 		(char *(*)())d2i_RSAPublicKey,(char *)rsa)
@@ -352,6 +382,11 @@ typedef struct X509_AUTH_st
 		(unsigned char **)(rsa))
 #define i2d_RSAPrivateKey_fp(fp,rsa) ASN1_i2d_fp(i2d_RSAPrivateKey,fp, \
 		(unsigned char *)rsa)
+#define d2i_RSAPrivateKey_bio(bp,rsa) (RSA *)ASN1_d2i_bio((char *(*)())\
+		RSA_new,(char *(*)())d2i_RSAPrivateKey, (bp), \
+		(unsigned char **)(rsa))
+#define i2d_RSAPrivateKey_bio(bp,rsa) ASN1_i2d_bio(i2d_RSAPrivateKey,bp, \
+		(unsigned char *)rsa)
 
 #define X509_NAME_dup(xn) (X509_NAME *)ASN1_dup(i2d_X509_NAME, \
 		(char *(*)())d2i_X509_NAME,(char *)xn)
@@ -366,25 +401,25 @@ int  		X509_add_cert(CERTIFICATE_CTX *ctx,X509 *);
 
 char *		X509_cert_verify_error_string(int n);
 int 		X509_add_cert_file(CERTIFICATE_CTX *c,char *file, int type);
-char *		X509_gmtime (char *s, long adj);
+int		X509_cmp_current_time(char *s);
+char *		X509_gmtime_adj(char *s, long adj);
 int		X509_add_cert_dir (CERTIFICATE_CTX *c,char *dir, int type);
 int		X509_load_verify_locations (CERTIFICATE_CTX *ctx,
 			char *file_env, char *dir_env);
 int		X509_set_default_verify_paths(CERTIFICATE_CTX *cts);
-X509 *		X509_new_D2i_X509(int len, unsigned char *p);
 char *		X509_get_default_cert_area(void );
 char *		X509_get_default_cert_dir(void );
 char *		X509_get_default_cert_file(void );
 char *		X509_get_default_cert_dir_env(void );
 char *		X509_get_default_cert_file_env(void );
 char *		X509_get_default_private_dir(void );
-X509_REQ *	X509_X509_TO_req(X509 *x, RSA *rsa);
+X509_REQ *	X509_to_X509_REQ(X509 *x, EVP_PKEY *pkey);
+X509 *		X509_REQ_to_X509(X509_REQ *r, int days,EVP_PKEY *pkey);
 int		X509_cert_verify(CERTIFICATE_CTX *ctx,X509 *xs, int (*cb)()); 
-int		i2f_ASN1_OBJECT(FILE *fp,ASN1_OBJECT *object);
 
 void ERR_load_X509_strings(void );
 
-CERTIFICATE_CTX *CERTIFICATE_CTX_new();
+CERTIFICATE_CTX *CERTIFICATE_CTX_new(void );
 void CERTIFICATE_CTX_free(CERTIFICATE_CTX *c);
 
 X509_ALGOR *	X509_ALGOR_new(void );
@@ -432,7 +467,6 @@ void		X509_NAME_ENTRY_free(X509_NAME_ENTRY *a);
 int		i2d_X509_NAME_ENTRY(X509_NAME_ENTRY *a,unsigned char **pp);
 X509_NAME_ENTRY *d2i_X509_NAME_ENTRY(X509_NAME_ENTRY **a,unsigned char **pp,
 			long length);
-void X509_NAME_print(FILE *fp, X509_NAME *name, int obase);
 
 X509_NAME *	X509_NAME_new(void);
 void		X509_NAME_free(X509_NAME *a);
@@ -448,7 +482,6 @@ X509 *		X509_new(void);
 void		X509_free(X509 *a);
 int		i2d_X509(X509 *a,unsigned char **pp);
 X509 *		d2i_X509(X509 **a,unsigned char **pp,long length);
-int		X509_print(FILE *fp,X509 *x);
 
 X509_REVOKED *	X509_REVOKED_new(void);
 void		X509_REVOKED_free(X509_REVOKED *a);
@@ -466,6 +499,23 @@ void		X509_CRL_free(X509_CRL *a);
 int		i2d_X509_CRL(X509_CRL *a,unsigned char **pp);
 X509_CRL *	d2i_X509_CRL(X509_CRL **a,unsigned char **pp,long length);
 
+X509_PKEY *	X509_PKEY_new();
+void		X509_PKEY_free(X509_PKEY *a);
+int		i2d_X509_PKEY(X509_PKEY *a,unsigned char **pp);
+X509_PKEY *	d2i_X509_PKEY(X509_PKEY **a,unsigned char **pp,long length);
+
+NETSCAPE_SPKI *	NETSCAPE_SPKI_new(void );
+void		NETSCAPE_SPKI_free(NETSCAPE_SPKI *a);
+int		i2d_NETSCAPE_SPKI(NETSCAPE_SPKI *a,unsigned char **pp);
+NETSCAPE_SPKI *	d2i_NETSCAPE_SPKI(NETSCAPE_SPKI **a,unsigned char **pp,
+			long length);
+
+NETSCAPE_SPKAC *NETSCAPE_SPKAC_new(void );
+void		NETSCAPE_SPKAC_free(NETSCAPE_SPKAC *a);
+int		i2d_NETSCAPE_SPKAC(NETSCAPE_SPKAC *a,unsigned char **pp);
+NETSCAPE_SPKAC *d2i_NETSCAPE_SPKAC(NETSCAPE_SPKAC **a,unsigned char **pp,
+		long length);
+
 #ifdef HEADER_ENVELOPE_H
 X509_INFO *	X509_INFO_new(void);
 void		X509_INFO_free(X509_INFO *a);
@@ -474,16 +524,14 @@ void		X509_INFO_free(X509_INFO *a);
 char *		X509_NAME_oneline(X509_NAME *a);
 
 int ASN1_verify(int (*i2d)(), X509_ALGOR *algor1,
-	ASN1_BIT_STRING *signature,char *data, RSA *rsa);
+	ASN1_BIT_STRING *signature,char *data,EVP_PKEY *pkey);
 
 int ASN1_sign(int (*i2d)(), X509_ALGOR *algor1, X509_ALGOR *algor2,
 	ASN1_BIT_STRING *signature,
-	char *data, RSA *rsa, EVP_MD *type);
+	char *data,EVP_PKEY *pkey, EVP_MD *type);
 
-void X509_REQ_print(FILE *fp,X509_REQ *req);
-
-RSA *X509_REQ_extract_key(X509_REQ *req);
-RSA *X509_extract_key(X509 *x509);
+EVP_PKEY *	X509_REQ_extract_key(X509_REQ *req);
+EVP_PKEY *	X509_extract_key(X509 *x509);
 
 int		X509_issuer_and_serial_cmp(X509 *a, X509 *b);
 unsigned long	X509_issuer_and_serial_hash(X509 *a);
@@ -499,26 +547,41 @@ unsigned long	X509_subject_name_hash(X509 *x);
 int		X509_name_cmp (X509_NAME *a, X509_NAME *b);
 unsigned long	X509_name_hash(X509_NAME *x);
 
+#ifndef WIN16
+int		X509_print_fp(FILE *bp,X509 *x);
+int		X509_REQ_print_fp(FILE *bp,X509_REQ *req);
+#endif
+
+#ifdef HEADER_BUFFER_H
+int		X509_NAME_print(BIO *bp, X509_NAME *name, int obase);
+int		X509_print(BIO *bp,X509 *x);
+int		X509_REQ_print(BIO *bp,X509_REQ *req);
+#else
+int		X509_NAME_print(char *bp, X509_NAME *name, int obase);
+int		X509_print(char *bp,X509 *x);
+int		X509_REQ_print(char *bp,X509_REQ *req);
+#endif
+
 #else
 
 X509 *		X509_get_cert();
 int  		X509_add_cert();
 char *		X509_cert_verify_error_string();
 int 		X509_add_cert_file();
-char *		X509_gmtime ();
+int		X509_cmp_current_time();
+char *		X509_gmtime_adj();
 int		X509_add_cert_dir ();
 int		X509_load_verify_locations ();
 int		X509_set_default_verify_paths();
-X509 *		X509_new_D2i_X509();
 char *		X509_get_default_cert_area();
 char *		X509_get_default_cert_dir();
 char *		X509_get_default_cert_file();
 char *		X509_get_default_cert_dir_env();
 char *		X509_get_default_cert_file_env();
 char *		X509_get_default_private_dir();
-X509_REQ *	X509_X509_TO_req();
+X509_REQ *	X509_to_X509_REQ();
+X509 *		X509_REQ_to_X509();
 int		X509_cert_verify();
-int		i2f_ASN1_OBJECT();
 void		ERR_load_X509_strings();
 CERTIFICATE_CTX *CERTIFICATE_CTX_new();
 void		CERTIFICATE_CTX_free();
@@ -554,7 +617,6 @@ X509_NAME_ENTRY *X509_NAME_ENTRY_new();
 void		X509_NAME_ENTRY_free();
 int		i2d_X509_NAME_ENTRY();
 X509_NAME_ENTRY *d2i_X509_NAME_ENTRY();
-void X509_NAME_print();
 X509_NAME *	X509_NAME_new();
 void		X509_NAME_free();
 int		i2d_X509_NAME();
@@ -567,7 +629,6 @@ X509 *		X509_new();
 void		X509_free();
 int		i2d_X509();
 X509 *		d2i_X509();
-int		X509_print();
 X509_REVOKED *	X509_REVOKED_new();
 void		X509_REVOKED_free();
 int		i2d_X509_REVOKED();
@@ -580,6 +641,10 @@ X509_CRL *	X509_CRL_new();
 void		X509_CRL_free();
 int		i2d_X509_CRL();
 X509_CRL *	d2i_X509_CRL();
+X509_PKEY *	X509_PKEY_new();
+void		X509_PKEY_free();
+int		i2d_X509_PKEY();
+X509_PKEY *	d2i_X509_PKEY();
 #ifdef HEADER_ENVELOPE_H
 X509_INFO *	X509_INFO_new();
 void		X509_INFO_free();
@@ -587,9 +652,8 @@ void		X509_INFO_free();
 char *		X509_NAME_oneline();
 int		ASN1_sign();
 int		ASN1_verify();
-void		X509_REQ_print();
-RSA *		X509_REQ_extract_key();
-RSA *		X509_extract_key();
+EVP_PKEY *	X509_REQ_extract_key();
+EVP_PKEY *	X509_extract_key();
 int		X509_issuer_and_serial_cmp();
 unsigned long	X509_issuer_and_serial_hash();
 X509_NAME *	X509_get_issuer_name();
@@ -598,9 +662,26 @@ unsigned long	X509_issuer_name_hash();
 X509_NAME *	X509_get_subject_name();
 int		X509_subject_name_cmp();
 unsigned long	X509_subject_name_hash();
-int		X509_name_cmp ();
+int		X509_name_cmp();
 unsigned long	X509_name_hash();
 
+#ifndef WIN16
+int		X509_print_fp();
+int		X509_REQ_print_fp();
+#endif
+
+int		X509_NAME_print();
+int		X509_print();
+int		X509_REQ_print();
+NETSCAPE_SPKAC *NETSCAPE_SPKAC_new();
+void		NETSCAPE_SPKAC_free();
+int		i2d_NETSCAPE_SPKAC();
+NETSCAPE_SPKAC *d2i_NETSCAPE_SPKAC();
+NETSCAPE_SPKI *NETSCAPE_SPKI_new();
+void		NETSCAPE_SPKI_free();
+int		i2d_NETSCAPE_SPKI();
+NETSCAPE_SPKI *d2i_NETSCAPE_SPKI();
+
 #endif
 
 /* BEGIN ERROR CODES */
@@ -614,13 +695,25 @@ unsigned long	X509_name_hash();
 #define X509_F_X509_EXTRACT_KEY				 104
 #define X509_F_X509_GET_CERT				 105
 #define X509_F_X509_NAME_ONELINE			 106
-#define X509_F_X509_PRINT				 107
-#define X509_F_X509_REQ_TO_X509				 108
-#define X509_F_X509_X509_TO_REQ				 109
+#define X509_F_X509_NAME_PRINT				 107
+#define X509_F_X509_PRINT_FP				 108
+#define X509_F_X509_REQ_PRINT				 109
+#define X509_F_X509_REQ_PRINT_FP			 110
+#define X509_F_X509_REQ_TO_X509				 111
+#define X509_F_X509_SET_DEFAULT_VERIFY_PATHS		 112
+#define X509_F_X509_TO_X509_REQ				 113
 
 /* Reason codes. */
-#define X509_R_BAD_X509_FILETYPE			 100
-#define X509_R_CERT_ALREADY_IN_HASH_TABLE		 101
-#define X509_R_ERR_ASN1_LIB				 102
+#define X509_R_BAD_PUBLIC_KEY_TYPE			 100
+#define X509_R_BAD_X509_FILETYPE			 101
+#define X509_R_CERT_ALREADY_IN_HASH_TABLE		 102
+#define X509_R_ERR_ASN1_LIB				 103
+#define X509_R_LOADING_DEFAULTS				 104
+#define X509_R_LOADING_DIR_ENV_VAR			 105
+#define X509_R_LOADING_FILE_ENV_VAR			 106
 
+#ifdef  __cplusplus
+}
 #endif
+#endif
+
