@@ -1,5 +1,5 @@
-/* lib/x509/x509_vrf.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/x509/x509_vrf.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,10 +47,11 @@
 
 #include <stdio.h>
 #include <time.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "crypto.h"
+#include "cryptlib.h"
 #include "lhash.h"
 #include "buffer.h"
 #include "md5.h"
@@ -63,41 +64,50 @@
 #ifndef NOPROTO
 static int verify(CERTIFICATE_CTX *c, X509 *xs, int (*cb)(), int depth);
 static int null_callback(int e, X509 *a, X509 *b, int c, int d);
-static int time_cmp(char *str);
 #else
 static int verify();
 static int null_callback();
-static int time_cmp();
 #endif
 
 #ifdef DEBUG
 void RSA_print_bytes();
 #endif
 
-char *X509_version="X509 part of SSLeay 0.5.2 29-Apr-1996";
+char *X509_version="X509 part of SSLeay 0.6.0 21-Jun-1996";
 
 int X509_add_cert_file(ctx,file, type)
 CERTIFICATE_CTX *ctx;
 char *file;
 int type;
 	{
-	FILE *in=NULL;
+	int ret=0;
+	BIO *in=NULL;
 	int i,count=0;
 	X509 *x=NULL;
 
-	in=fopen(file,"r");
+#ifndef WIN16
+	in=BIO_new(BIO_s_file());
+#else
+	in=BIO_new(BIO_s_file_internal_w16());
+#endif
 	if (in == NULL)
 		{
 		SYSerr(ERR_F_FOPEN,errno);
 		X509err(X509_F_X509_ADD_CERT_FILE,ERR_R_SYS_LIB);
-		return(0);
+		goto err;
+		}
+	if (BIO_read_filename(in,file) <= 0)
+		{
+		SYSerr(ERR_F_FOPEN,errno);
+		X509err(X509_F_X509_ADD_CERT_FILE,ERR_R_SYS_LIB);
+		goto err;
 		}
 
 	if (type == X509_FILETYPE_PEM)
 		{
 		for (;;)
 			{
-			x=PEM_read_X509(in,NULL,NULL);
+			x=PEM_read_bio_X509(in,NULL,NULL);
 			if (x == NULL)
 				{
 
@@ -116,15 +126,14 @@ int type;
 				}
 			i=X509_add_cert(ctx,x);
 			if (!i) goto err;
+			x=NULL;
 			count++;
 			}
-		fclose(in);
-		return(count);
+		ret=count;
 		}
 	else if (type == X509_FILETYPE_ASN1)
 		{
-		x=d2i_X509_fp(in,NULL);
-		fclose(in);
+		x=d2i_X509_bio(in,NULL);
 		if (x == NULL)
 			{
 			X509err(X509_F_X509_ADD_CERT_FILE,ERR_R_ASN1_LIB);
@@ -132,7 +141,8 @@ int type;
 			}
 		i=X509_add_cert(ctx,x);
 		if (!i) goto err;
-		return(i);
+		x=NULL;
+		ret=i;
 		}
 	else
 		{
@@ -141,8 +151,8 @@ int type;
 		}
 err:
 	if (x != NULL) X509_free(x);
-	if (in != NULL) fclose(in);
-	return(0);
+	if (in != NULL) BIO_free(in);
+	return(ret);
 	}
 
 int X509_add_cert_dir(ctx,dir, type)
@@ -236,34 +246,42 @@ X509 *xs;
 int (*cb)();
 int depth;
 	{
-	X509 *xi;
-	BUFFER *buf=NULL;
-	RSA *rsa=NULL;
+	X509 *xi=NULL;
+	BUF_MEM *buf=NULL;
+	EVP_PKEY *pkey=NULL;
 	int ret,i;
 
-	xi=X509_get_cert(ctx,
-		X509_get_issuer_name(xs),NULL);
+	xi=X509_get_cert(ctx,X509_get_issuer_name(xs),NULL);
 	if (xi == NULL)
 		{
-		ret=(*cb)(0,xs,xi,depth,
-			VERIFY_ERR_UNABLE_TO_GET_ISSUER);
+		if ((X509_name_cmp(xs->cert_info->subject,xs->cert_info->issuer)
+			== 0) && (depth == 0))
+			{
+			ret=(*cb)(0,xs,xi,depth,
+				VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT);
+			}
+		else
+			{
+			ret=(*cb)(0,xs,xi,depth,
+				VERIFY_ERR_UNABLE_TO_GET_ISSUER);
+			}
 		goto end;
 		}
 
-	if ((rsa=X509_extract_key(xi)) == NULL)
+	if ((pkey=X509_extract_key(xi)) == NULL)
 		{
 		ret=(*cb)(0,xs,xi,depth,
 			VERIFY_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY);
 		goto end;
 		}
 
-	if (!X509_verify(xs,rsa)) /* EAY */
+	if (!X509_verify(xs,pkey)) /* EAY */
 		{
 		ret=(*cb)(0,xs,xi,depth,VERIFY_ERR_SIGNATURE_FAILURE);
 		goto end;
 		}
 
-	i=time_cmp(xs->cert_info->validity->notBefore);
+	i=X509_cmp_current_time(xs->cert_info->validity->notBefore);
 	if (i == 0)
 		{
 		ret=(*cb)(0,xs,xi,depth,VERIFY_ERR_ERROR_IN_NOT_BEFORE_FIELD);
@@ -275,7 +293,7 @@ int depth;
 		goto end;
 		}
 
-	i=time_cmp(xs->cert_info->validity->notAfter);
+	i=X509_cmp_current_time(xs->cert_info->validity->notAfter);
 	if (i == 0)
 		ret=((*cb)(0,xs,xi,depth,
 			VERIFY_ERR_ERROR_IN_NOT_AFTER_FIELD));
@@ -308,12 +326,12 @@ int depth;
 			}
 		}
 end:
-	if (buf != NULL) BUF_free(buf);
-	if (rsa != NULL) RSA_free(rsa);
+	if (buf != NULL) BUF_MEM_free(buf);
+	if (pkey != NULL)EVP_PKEY_free(pkey);
 	return(ret);
 	}
 
-static int time_cmp(str)
+int X509_cmp_current_time(str)
 char *str;
 	{
 	time_t offset;
@@ -344,7 +362,7 @@ char *str;
 		if (*str == '-')
 			offset-=offset;
 		}
-	X509_gmtime(buff2,offset);
+	X509_gmtime_adj(buff2,offset);
 
 	i=(buff1[0]-'0')*10+(buff1[1]-'0');
 	if (i < 70) i+=100;
@@ -360,49 +378,17 @@ char *str;
 		return(i);
 	}
 
-char *X509_gmtime(s, adj)
-char *s;
+ASN1_UTCTIME *X509_gmtime_adj(s, adj)
+ASN1_UTCTIME *s;
 long adj;
 	{
 	time_t t;
-	struct tm *ts;
-#ifdef THREADS
-	struct tm data;
-#endif
 
-	if (s == NULL)
-		s=ASN1_UTCTIME_new();
-	if (s == NULL) return(NULL);
 	time(&t);
 	t+=adj;
-#ifdef THREADS
-	ts=gmtime_r(&t,&data);
-#else
-	ts=gmtime(&t);
-#endif
-	sprintf(s,"%02d%02d%02d%02d%02d%02dZ",ts->tm_year%100,
-		ts->tm_mon+1,ts->tm_mday,ts->tm_hour,ts->tm_min,ts->tm_sec);
-	return(s);
+	return(ASN1_UTCTIME_set(s,t));
 	}
 
-#ifdef DEBUG
-void RSA_print_bytes(f,n,b)
-FILE *f;
-int n;
-char *b;
-	{
-	int i;
-	static char *h="0123456789abcdef";
-
-	for (i=0; i<n; i++)
-		{
-		Fputc(h[(b[i]>>4)&0x0f],f);
-		Fputc(h[(b[i]   )&0x0f],f);
-		Fputc(' ',f);
-		}
-	}
-#endif
-
 char *X509_cert_verify_error_string(n)
 int n;
 	{
@@ -442,17 +428,19 @@ int n;
 		}
 	}
 
-RSA *X509_extract_key(x)
+EVP_PKEY *X509_extract_key(x)
 X509 *x;
 	{
-	RSA *rsa=NULL;
+	EVP_PKEY *ret=NULL;
 	long j;
+	int type;
 	unsigned char *p;
 
+	type=OBJ_obj2nid(x->cert_info->key->algor->algorithm);
 	p=x->cert_info->key->public_key->data;
 	j=x->cert_info->key->public_key->length;
-	if ((rsa=d2i_RSAPublicKey(NULL,&p,(long)j)) == NULL)
+	if ((ret=d2i_PublicKey(type,NULL,&p,(long)j)) == NULL)
 		X509err(X509_F_X509_EXTRACT_KEY,X509_R_ERR_ASN1_LIB);
-	return(rsa);
+	return(ret);
 	}
 
