@@ -1,5 +1,5 @@
 /* ssl/ssl_cert.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,6 +46,7 @@
  */
 
 #include <stdio.h>
+#include "objects.h"
 #include "ssl_locl.h"
 #include "ssl_rsa.h"
 
@@ -72,12 +73,17 @@ CERT *ssl_cert_new()
 void ssl_cert_free(c)
 CERT *c;
 	{
-	CS_BEGIN;
-	if (--c->references > 0) return;
-	CS_END;
+	CRYPTO_w_lock(CRYPTO_LOCK_ssl_CERT);
+	if (--c->references > 0)
+		{
+		CRYPTO_w_unlock(CRYPTO_LOCK_ssl_CERT);
+		return;
+		}
+	CRYPTO_w_unlock(CRYPTO_LOCK_ssl_CERT);
+
 	if (c->x509 != NULL) X509_free(c->x509);
-	if (c->privatekey != NULL) RSA_free(c->privatekey);
-	if (c->publickey != NULL) RSA_free(c->publickey);
+	if (c->privatekey != NULL) EVP_PKEY_free(c->privatekey);
+	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
 	free(c);
 	}
 
@@ -88,27 +94,33 @@ int type;
 int len;
 unsigned char *data;
 	{
-	CERT *c;
+	EVP_PKEY *pkey=NULL;
+	CERT *c=NULL;
 	int i;
-	X509 *x509; /* will don't want to include the headers */
+	X509 *x509=NULL; /* will don't want to include the headers */
+	int ret=0;
 	
 	x509=d2i_X509(NULL,&data,(long)len);
 	if (x509 == NULL)
 		{
 		SSLerr(SSL_F_SSL_SET_CERTIFICATE,ERR_R_X509_LIB);
-		return(0);
+		goto err;
 		}
 
 	i=X509_cert_verify(s->ctx->cert,x509,s->verify_callback);
-	if ((s->verify_mode & SSL_VERIFY_PEER) && (!i))
+	if ((s->verify_mode != SSL_VERIFY_NONE) && (!i))
 		{
 		SSLerr(SSL_F_SSL_SET_CERTIFICATE,ERR_R_X509_LIB);
-		return(0);
+		goto err;
 		}
 
 	/* cert for ssl */
 	c=ssl_cert_new();
-	if (c == NULL) return(-1);
+	if (c == NULL)
+		{
+		ret= -1;
+		goto err;
+		}
 
 	/* cert for session */
 	if (s->session->cert) ssl_cert_free(s->session->cert);
@@ -116,11 +128,27 @@ unsigned char *data;
 
 	c->cert_type=type;
 	c->x509=x509;
-	if (c->publickey != NULL) RSA_free(c->publickey);
-	c->publickey=(RSA *)X509_extract_key(x509);
-	if (c->publickey == NULL) return(0);
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	return(1);
+	pkey=X509_extract_key(x509);
+	x509=NULL;
+	if (pkey == NULL)
+		{
+		SSLerr(SSL_F_SSL_SET_CERTIFICATE,SSL_R_UNABLE_TO_EXTRACT_PUBLIC_KEY);
+		goto err;
+		}
+	if (pkey->type != EVP_PKEY_RSA)
+		{
+		SSLerr(SSL_F_SSL_SET_CERTIFICATE,SSL_R_PUBLIC_KEY_NO_RSA);
+		EVP_PKEY_free(pkey);
+		goto err;
+		}
+	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
+	c->publickey=pkey;
+	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE))
+		goto err;
+	ret=1;
+err:
+	if (x509 != NULL) X509_free(x509);
+	return(ret);
 	}
 
 int ssl_set_cert_type(c, type)
