@@ -1,5 +1,5 @@
 /* ssl/ssl_clnt.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -54,21 +54,21 @@
 #include "envelope.h"
 
 #ifndef NOPROTO
-static int get_server_finish(SSL *s);
+static int get_server_finished(SSL *s);
 static int get_server_verify(SSL *s);
 static int get_server_hello(SSL *s);
 static int client_hello(SSL *s); 
 static int client_master_key(SSL *s);
-static int client_finish(SSL *s);
+static int client_finished(SSL *s);
 static int client_certificate(SSL *s);
 static int choose_cipher(SSL *s);
 #else
-static int get_server_finish();
+static int get_server_finished();
 static int get_server_verify();
 static int get_server_hello();
 static int client_hello(); 
 static int client_master_key();
-static int client_finish();
+static int client_finished();
 static int client_certificate();
 static int choose_cipher();
 #endif
@@ -79,122 +79,170 @@ int SSL_connect(s)
 SSL *s;
 	{
 	unsigned long l=time(NULL);
-	BUFFER *buf=NULL;
-	int ret;
+	BUF_MEM *buf=NULL;
+	int ret= -1;
+	void (*cb)()=NULL;
 
 	RAND_seed((unsigned char *)&l,sizeof(l));
 	ERR_clear_error();
+	errno=0;
+
+	if (s->info_callback != NULL)
+		cb=s->info_callback;
+	else if (s->ctx->info_callback != NULL)
+		cb=s->ctx->info_callback;
 
 	/* init things to blank */
-	if (!SSL_ST_INIT(s->state)) SSL_clear(s);
+	if (!SSL_in_init(s)) SSL_clear(s);
 
-	errno=0;
 	for (;;)
 		{
+		int state;
+
 #ifdef SSL_DEBUG
 		SSL_TRACE(SSL_ERR,"state=%02X %d %s\n",
 			s->state,s->init_num,ssl_state_string(s));
 #endif
+
+		state=s->state;
+
 		switch (s->state)
 			{
-		case ST_BEFORE:
-			if ((buf=BUF_new()) == NULL) return(-1);
-			if (!BUF_grow(buf,(unsigned long)
+		case SSL_ST_BEFORE:
+		case SSL_ST_CONNECT:
+			if ((buf=BUF_MEM_new()) == NULL)
+				{
+				ret=-1;
+				goto end;
+				}
+			if (!BUF_MEM_grow(buf,
 				SSL_MAX_RECORD_LENGTH_3_BYTE_HEADER))
-				return(-1);
+				{
+				ret=-1;
+				goto end;
+				}
 			if (s->init_buf) free(s->init_buf);
 			s->init_buf=(unsigned char *)buf->data;
 			free(buf);
 			s->init_num=0;
-			s->state=ST_SEND_CLIENT_HELLO_A;
+			s->state=SSL_ST_SEND_CLIENT_HELLO_A;
 			s->ctx->sess_connect++;
 			BREAK;
 
-		case ST_SEND_CLIENT_HELLO_A:
-		case ST_SEND_CLIENT_HELLO_B:
+		case SSL_ST_SEND_CLIENT_HELLO_A:
+		case SSL_ST_SEND_CLIENT_HELLO_B:
 			ret=client_hello(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_GET_SERVER_HELLO_A;
+			s->state=SSL_ST_GET_SERVER_HELLO_A;
 			BREAK;
 		
-		case ST_GET_SERVER_HELLO_A:
-		case ST_GET_SERVER_HELLO_B:
+		case SSL_ST_GET_SERVER_HELLO_A:
+		case SSL_ST_GET_SERVER_HELLO_B:
 			ret=get_server_hello(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
 			if (!s->hit) /* new session */
 				{
-				s->state=ST_SEND_CLIENT_MASTER_KEY_A;
+				s->state=SSL_ST_SEND_CLIENT_MASTER_KEY_A;
 				BREAK; 
 				}
 			else
 				{
-				s->state=ST_START_ENCRYPTION;
+				s->state=SSL_ST_CLIENT_START_ENCRYPTION;
 				break;
 				}
 	
-		case ST_SEND_CLIENT_MASTER_KEY_A:
-		case ST_SEND_CLIENT_MASTER_KEY_B:
+		case SSL_ST_SEND_CLIENT_MASTER_KEY_A:
+		case SSL_ST_SEND_CLIENT_MASTER_KEY_B:
 			ret=client_master_key(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_START_ENCRYPTION;
-			BREAK;
+			s->state=SSL_ST_CLIENT_START_ENCRYPTION;
+			break;
 
-		case ST_START_ENCRYPTION:
+		case SSL_ST_CLIENT_START_ENCRYPTION:
 			/* Ok, we now have all the stuff needed to
 			 * start encrypting, so lets fire it up :-) */
-			if (!s->session->cipher->crypt_init(s,1)) return(-1);
+			if (!s->session->cipher->crypt_init(s,1))
+				{
+				ret=-1;
+				goto end;
+				}
 			s->clear_text=0;
-			s->state=ST_SEND_CLIENT_FINISHED_A;
-			BREAK;
+			s->state=SSL_ST_SEND_CLIENT_FINISHED_A;
+			break;
 
-		case ST_SEND_CLIENT_FINISHED_A:
-		case ST_SEND_CLIENT_FINISHED_B:
-			ret=client_finish(s);
-			if (ret <= 0) return(ret);
+		case SSL_ST_SEND_CLIENT_FINISHED_A:
+		case SSL_ST_SEND_CLIENT_FINISHED_B:
+			ret=client_finished(s);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_GET_SERVER_VERIFY_A;
-			BREAK;
+			s->state=SSL_ST_GET_SERVER_VERIFY_A;
+			break;
 
-		case ST_GET_SERVER_VERIFY_A:
-		case ST_GET_SERVER_VERIFY_B:
+		case SSL_ST_GET_SERVER_VERIFY_A:
+		case SSL_ST_GET_SERVER_VERIFY_B:
 			ret=get_server_verify(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_GET_SERVER_FINISHED_A;
-			BREAK;
-
-		case ST_GET_SERVER_FINISHED_A:
-		case ST_GET_SERVER_FINISHED_B:
-			ret=get_server_finish(s);
-			if (ret <= 0) return(ret);
+			s->state=SSL_ST_GET_SERVER_FINISHED_A;
 			break;
 
-		case ST_SEND_CLIENT_CERTIFICATE_A:
-		case ST_SEND_CLIENT_CERTIFICATE_B:
-		case ST_SEND_CLIENT_CERTIFICATE_C:
-		case ST_SEND_CLIENT_CERTIFICATE_D:
+		case SSL_ST_GET_SERVER_FINISHED_A:
+		case SSL_ST_GET_SERVER_FINISHED_B:
+			ret=get_server_finished(s);
+			if (ret <= 0) goto end;
+			break;
+
+		case SSL_ST_SEND_CLIENT_CERTIFICATE_A:
+		case SSL_ST_SEND_CLIENT_CERTIFICATE_B:
+		case SSL_ST_SEND_CLIENT_CERTIFICATE_C:
+		case SSL_ST_SEND_CLIENT_CERTIFICATE_D:
+		case SSL_ST_X509_GET_CLIENT_CERTIFICATE:
 			ret=client_certificate(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_GET_SERVER_FINISHED_A;
+			s->state=SSL_ST_GET_SERVER_FINISHED_A;
 			break;
 
-		case ST_OK:
+		case SSL_ST_OK:
 			free(s->init_buf);
 			s->init_buf=NULL;
 			s->init_num=0;
 			ERR_clear_error();
+
+			if (SSL_add_session(s->ctx,s->session) &&
+				(s->ctx->new_session_cb != NULL))
+				{
+				CRYPTO_w_lock(CRYPTO_LOCK_SSL_SESSION);
+				s->session->references++;
+				CRYPTO_w_unlock(CRYPTO_LOCK_SSL_SESSION);
+
+				if (!s->ctx->new_session_cb(s->session))
+					{
+					CRYPTO_w_lock(CRYPTO_LOCK_SSL_SESSION);
+					s->session->references--;
+					CRYPTO_w_unlock(CRYPTO_LOCK_SSL_SESSION);
+					}
+				}
 			s->ctx->sess_connect_good++;
-			return(1);
+			ret=1;
+			goto end;
+			break;
 		default:
 			SSLerr(SSL_F_SSL_CONNECT,SSL_R_UNKNOWN_STATE);
 			return(-1);
-			break;
+			/* break; */
 			}
+
+		if ((cb != NULL) && (s->state != state))
+			cb(s,SSL_CB_CONNECT_LOOP,1);
 		}
+end:
+	if (cb != NULL) 
+		cb(s,SSL_CB_CONNECT_EXIT,ret);
+	return(ret);
 	}
 
 static int get_server_hello(s)
@@ -208,7 +256,7 @@ SSL *s;
 
 	buf=s->init_buf;
 	p=buf;
-	if (s->state == ST_GET_SERVER_HELLO_A)
+	if (s->state == SSL_ST_GET_SERVER_HELLO_A)
 		{
 		i=SSL_read(s,(char *)&(buf[s->init_num]),11-s->init_num);
 		if (i < (11-s->init_num)) 
@@ -234,11 +282,11 @@ SSL *s;
 		n2s(p,i); s->state_cert_length=i;
 		n2s(p,i); s->state_csl=i;
 		n2s(p,i); s->state_conn_id_length=i;
-		s->state=ST_GET_SERVER_HELLO_B;
+		s->state=SSL_ST_GET_SERVER_HELLO_B;
 		s->init_num=0;
 		}
 
-	/* ST_GET_SERVER_HELLO_B */
+	/* SSL_ST_GET_SERVER_HELLO_B */
 	j=s->state_cert_length+s->state_csl+s->state_conn_id_length
 		- s->init_num;
 	i=SSL_read(s,(char *)&(buf[s->init_num]),j);
@@ -325,10 +373,16 @@ SSL *s;
 			}
 		}
 
-	if ((s->peer_status == SSL_PEER_IN_SSL) && (s->peer != NULL))
-		X509_free(s->peer);
-	s->peer=NULL;
-	s->peer_status=SSL_PEER_IN_CERT;
+	if ((s->session != NULL) && (s->session->peer != NULL))
+		X509_free(s->session->peer);
+	s->session->peer=s->session->cert->x509;
+
+	/* hmmm, can we have the problem of the other session with this
+	 * cert, free's it before we increment the reference count. */
+	CRYPTO_w_lock(CRYPTO_LOCK_X509);
+	s->session->peer->references++;
+	CRYPTO_w_lock(CRYPTO_LOCK_X509);
+
 	/* get conn_id */
 	if (s->conn_id_length != s->state_conn_id_length)
 		{
@@ -355,7 +409,7 @@ SSL *s;
 	int i,n,j;
 
 	buf=s->init_buf;
-	if (s->state == ST_SEND_CLIENT_HELLO_A)
+	if (s->state == SSL_ST_SEND_CLIENT_HELLO_A)
 		{
 		if (s->session == NULL)
 			if (!ssl_get_new_session(s,0))
@@ -370,7 +424,7 @@ SSL *s;
 		d=p+9;					/* data section */
 		*(p++)=SSL_MT_CLIENT_HELLO;		/* type */
 		s2n(SSL_CLIENT_VERSION,p);		/* version */
-		i=n=j=0;
+		n=j=0;
 		for (i=0; i<NUM_CIPHERS; i++)
 			{
 			if (ssl_ciphers[i].valid)
@@ -417,11 +471,11 @@ SSL *s;
 		memcpy(d,s->challenge,CHALLENGE_LENGTH);
 		d+=CHALLENGE_LENGTH;
 
-		s->state=ST_SEND_CLIENT_HELLO_B;
+		s->state=SSL_ST_SEND_CLIENT_HELLO_B;
 		s->init_num=d-buf;
 		s->init_off=0;
 		}
-	/* ST_SEND_CLIENT_HELLO_B */
+	/* SSL_ST_SEND_CLIENT_HELLO_B */
 	return(ssl_do_write(s));
 	}
 
@@ -433,7 +487,7 @@ SSL *s;
 	int clear,enc,karg,i;
 
 	buf=s->init_buf;
-	if (s->state == ST_SEND_CLIENT_MASTER_KEY_A)
+	if (s->state == SSL_ST_SEND_CLIENT_MASTER_KEY_A)
 		{
 		p=buf;
 		d=p+10;
@@ -523,27 +577,27 @@ SSL *s;
 		memcpy(d,s->session->key_arg,(unsigned int)karg);
 		d+=karg;
 
-		s->state=ST_SEND_CLIENT_MASTER_KEY_B;
+		s->state=SSL_ST_SEND_CLIENT_MASTER_KEY_B;
 		s->init_num=d-buf;
 		s->init_off=0;
 		}
 
-	/* ST_SEND_CLIENT_MASTER_KEY_B */
+	/* SSL_ST_SEND_CLIENT_MASTER_KEY_B */
 	return(ssl_do_write(s));
 	}
 
-static int client_finish(s)
+static int client_finished(s)
 SSL *s;
 	{
 	unsigned char *p;
 
-	if (s->state == ST_SEND_CLIENT_FINISHED_A)
+	if (s->state == SSL_ST_SEND_CLIENT_FINISHED_A)
 		{
 		p=s->init_buf;
 		*(p++)=SSL_MT_CLIENT_FINISHED;
 		memcpy(p,s->conn_id,(unsigned int)s->conn_id_length);
 
-		s->state=ST_SEND_CLIENT_FINISHED_B;
+		s->state=SSL_ST_SEND_CLIENT_FINISHED_B;
 		s->init_num=s->conn_id_length+1;
 		s->init_off=0;
 		}
@@ -563,7 +617,11 @@ SSL *s;
 
 	buf=s->init_buf;
 	cert_ch= &(buf[2]);
-	if (s->state == ST_SEND_CLIENT_CERTIFICATE_A)
+
+	/* We have a cert associated with out SSL, so attach it to
+	 * the session if it does not have one */
+
+	if (s->state == SSL_ST_SEND_CLIENT_CERTIFICATE_A)
 		{
 		i=SSL_read(s,(char *)&(buf[s->init_num]),
 			MAX_CERT_CHALLENGE_LENGTH+1-s->init_num);
@@ -583,30 +641,56 @@ SSL *s;
 		if ((s->cert == NULL) || (s->cert->x509 == NULL) ||
 			(s->cert->privatekey == NULL))
 			{
+			s->state=SSL_ST_X509_GET_CLIENT_CERTIFICATE;
+			}
+		else
+			s->state=SSL_ST_SEND_CLIENT_CERTIFICATE_C;
+		}
+
+	if (s->state == SSL_ST_X509_GET_CLIENT_CERTIFICATE)
+		{
+		/* If we get an error we need to
+		 * ssl->rwstate=SSL_X509_LOOKUP;
+		 * return(error);
+		 * We should then be retried when things are ok and we
+		 * can get a cert or not */
+		if (0) /* we get a good client cert */
+			{
+			/*s->cert=NULL; */ /* ! NULL */
+			s->cert->x509=NULL; /* new cert */
+			s->cert->privatekey=NULL; /* new pkey */
+			/* This is where we need to call the application
+			 * to give us a client certificate.
+			 * This need to support blocking */
+			s->state=SSL_ST_SEND_CLIENT_CERTIFICATE_C;
+			}
+		else
+			{
+			/* We have no client certificate to respond with
+			 * so send the correct error message back */
+			s->state=SSL_ST_SEND_CLIENT_CERTIFICATE_B;
 			p=buf;
 			*(p++)=SSL_MT_ERROR;
 			s2n(SSL_PE_NO_CERTIFICATE,p);
 			s->init_off=0;
 			s->init_num=3;
-			s->state=ST_SEND_CLIENT_CERTIFICATE_B;
+			/* Write is done at the end */
 			}
-		else
-			s->state=ST_SEND_CLIENT_CERTIFICATE_C;
 		}
 
-	if (s->state == ST_SEND_CLIENT_CERTIFICATE_B)
+	if (s->state == SSL_ST_SEND_CLIENT_CERTIFICATE_B)
 		{
 		return(ssl_do_write(s));
 		}
 
-	if (s->state == ST_SEND_CLIENT_CERTIFICATE_C)
+	if (s->state == SSL_ST_SEND_CLIENT_CERTIFICATE_C)
 		{
 		EVP_MD_CTX ctx;
 
 		/* ok, now we calculate the checksum
 		 * do it first so we can reuse buf :-) */
 		p=buf;
-		EVP_SignInit(&ctx,EVP_md5);
+		EVP_SignInit(&ctx,EVP_md5());
 		EVP_SignUpdate(&ctx,s->key_material,
 			(unsigned int)s->key_material_length);
 		EVP_SignUpdate(&ctx,cert_ch,(unsigned int)cert_ch_len);
@@ -631,11 +715,11 @@ SSL *s;
 		s2n(n,p);
 		d+=n;
 
-		s->state=ST_SEND_CLIENT_CERTIFICATE_D;
+		s->state=SSL_ST_SEND_CLIENT_CERTIFICATE_D;
 		s->init_num=d-buf;
 		s->init_off=0;
 		}
-	/* if (s->state == ST_SEND_CLIENT_CERTIFICATE_D) */
+	/* if (s->state == SSL_ST_SEND_CLIENT_CERTIFICATE_D) */
 	return(ssl_do_write(s));
 	}
 
@@ -646,13 +730,13 @@ SSL *s;
 	int i;
 
 	p=s->init_buf;
-	if (s->state == ST_GET_SERVER_VERIFY_A)
+	if (s->state == SSL_ST_GET_SERVER_VERIFY_A)
 		{
 		i=SSL_read(s,(char *)&(p[s->init_num]),1-s->init_num);
 		if (i < (1-s->init_num)) 
 			return(ssl_part_read(s,SSL_F_GET_SERVER_VERIFY,i));
 
-		s->state= ST_GET_SERVER_VERIFY_B;
+		s->state= SSL_ST_GET_SERVER_VERIFY_B;
 		s->init_num=0;
 		if (*p != SSL_MT_SERVER_VERIFY)
 			{
@@ -683,7 +767,7 @@ SSL *s;
 	return(1);
 	}
 
-static int get_server_finish(s)
+static int get_server_finished(s)
 SSL *s;
 	{
 	unsigned char *buf;
@@ -692,7 +776,7 @@ SSL *s;
 
 	buf=s->init_buf;
 	p=buf;
-	if (s->state == ST_GET_SERVER_FINISHED_A)
+	if (s->state == SSL_ST_GET_SERVER_FINISHED_A)
 		{
 		i=SSL_read(s,(char *)&(buf[s->init_num]),1-s->init_num);
 		if (i < (1-s->init_num))
@@ -700,7 +784,7 @@ SSL *s;
 		s->init_num=i;
 		if (*p == SSL_MT_REQUEST_CERTIFICATE)
 			{
-			s->state=ST_SEND_CLIENT_CERTIFICATE_A;
+			s->state=SSL_ST_SEND_CLIENT_CERTIFICATE_A;
 			return(1);
 			}
 		else if (*p != SSL_MT_SERVER_FINISHED)
@@ -714,7 +798,7 @@ SSL *s;
 				SSLerr(SSL_F_GET_SERVER_FINISHED,SSL_R_PEER_ERROR);
 			return(-1);
 			}
-		s->state=ST_OK;
+		s->state=SSL_ST_OK;
 		s->init_num=0;
 		}
 
@@ -743,7 +827,6 @@ SSL *s;
 			}
 		s->session->session_id_length=SSL_SESSION_ID_LENGTH;
 		memcpy(s->session->session_id,p,SSL_SESSION_ID_LENGTH);
-		SSL_add_session(s->ctx,s->session); /* should return error? */
 		}
 	else
 		{
@@ -766,19 +849,16 @@ SSL *s;
 
 	if (s->ctx->reverse)
 		{
-		for (;;)
+		for (j=0; j < s->session->num_ciphers; j++)
 			{
-			for (j=0; j < s->session->num_ciphers; j++)
+			if (strcmp(s->ctx->cipher,
+				s->session->ciphers[j]->name) == 0)
 				{
-				if (strcmp(s->ctx->cipher,
-					s->session->ciphers[j]->name) == 0)
-					{
-					s->session->cipher=s->session->ciphers[j];
-					return(1);
-					}
+				s->session->cipher=s->session->ciphers[j];
+				return(1);
 				}
-			return(0);
 			}
+		return(0);
 		}
 	i=0;
 	for (;;)
