@@ -1,5 +1,5 @@
 /* ssl/ssl_des.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -57,7 +57,6 @@ int client;
 	{
 	unsigned char md5d[MD5_DIGEST_LENGTH];
 	DES_CBC_STATE *ds;
-	SSL_SESSION *c;
 
 	if (s->crypt_state == NULL)
 		s->crypt_state=(char *)malloc(sizeof(DES_CBC_STATE));
@@ -65,7 +64,6 @@ int client;
 
 	ds=(DES_CBC_STATE *)s->crypt_state;
 
-	c=s->session;
 	if (s->key_material != NULL)
 		{
 		memset(s->key_material,0,s->key_material_length);
@@ -152,14 +150,12 @@ int client;
 	{
 	unsigned char md5d[MD5_DIGEST_LENGTH*3];
 	DES_EDE3_CBC_STATE *ds;
-	SSL_SESSION *c;
 
 	if (s->crypt_state == NULL)
 		s->crypt_state=(char *)malloc(sizeof(DES_EDE3_CBC_STATE));
 	if (s->crypt_state == NULL) goto err;
 	ds=(DES_EDE3_CBC_STATE *)s->crypt_state;
 
-	c=s->session;
 	if (s->key_material != NULL)
 		{
 		memset(s->key_material,0,s->key_material_length);
@@ -263,14 +259,12 @@ int client;
 	{
 	unsigned char md5d[MD5_DIGEST_LENGTH];
 	DES_CFB_STATE *ds;
-	SSL_SESSION *c;
 
 	if (s->crypt_state == NULL)
 		s->crypt_state=(char *)malloc(sizeof(DES_CFB_STATE));
 	if (s->crypt_state == NULL) goto err;
 	ds=(DES_CFB_STATE *)s->crypt_state;
 
-	c=s->session;
 	if (s->key_material != NULL)
 		{
 		memset(s->key_material,0,s->key_material_length);
@@ -310,7 +304,7 @@ int client;
 	memset(md5d,0,MD5_DIGEST_LENGTH);
 	return(1);
 err:
-	SSLerr(SSL_F_SSL_ENC_DES_EDE3_CBC_INIT,ERR_R_MALLOC_FAILURE);
+	SSLerr(SSL_F_SSL_ENC_DES_CFB_INIT,ERR_R_MALLOC_FAILURE);
 	return(0);
 	}
 
