@@ -1,5 +1,5 @@
 /* ssl/ssl_idea.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -52,7 +52,7 @@
  * Sorry Andreas :-).
  */
 
-#ifndef NOIDEA
+#ifndef NO_IDEA
 
 #include <stdio.h>
 #include "idea.h"
@@ -66,14 +66,12 @@ int client;
 	{
 	unsigned char *md5d1,*md5d2;
 	IDEA_CBC_STATE *is;
-	SSL_SESSION *c;
 
 	if (s->crypt_state == NULL)
 		s->crypt_state=(char *)malloc(sizeof(IDEA_CBC_STATE));
 	if (s->crypt_state == NULL) goto err;
 	is=(IDEA_CBC_STATE *)s->crypt_state;
 
-	c=s->session;
 	if (s->key_material != NULL)
 		{
 		memset(s->key_material,0,s->key_material_length);
