@@ -1,5 +1,5 @@
-/* lib/ssl/ssl_lib.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* ssl/ssl_lib.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -57,7 +57,7 @@
 #include "ssl_rsa.h"
 #include "ssl_null.h"
 
-char *SSL_version="SSLeay 0.5.2 29-Apr-1996";
+char *SSL_version="SSLeay 0.6.0 21-Jun-1996";
 
 /* THIS ARRAY MUST BE KEPT ORDERED BY c1, c2 and c3.
  * basically the second last 'value' which is a #define for these 3
@@ -65,103 +65,117 @@ char *SSL_version="SSLeay 0.5.2 29-Apr-1996";
 CIPHER ssl_ciphers[NUM_CIPHERS]={
 /* NULL_WITH_MD5 v3 */
 	{	1,SSL_TXT_NULL_WITH_MD5,
-	ssl_enc_null_init,ssl_enc_null,ssl_compute_md5_mac,
+	ssl_enc_null_init,(void (*)())free,
+	ssl_enc_null, ssl_compute_md5_mac,
 	8,MD5_MAC_SIZE,1,0,SSL_CK_NULL_WITH_MD5,0},
 
-#ifndef NORC4
+#ifndef NO_RC4
 /* RC4_128_WITH_MD5 */
 	{	1,SSL_TXT_RC4_128_WITH_MD5,
-	ssl_enc_rc4_init,ssl_enc_rc4,ssl_compute_md5_mac,
+	ssl_enc_rc4_init,(void (*)())free,
+	ssl_enc_rc4,ssl_compute_md5_mac,
 	16,MD5_MAC_SIZE,1,0,SSL_CK_RC4_128_WITH_MD5,0},
 
 /* RC4_128_EXPORT40_WITH_MD5 */
 	{	1,SSL_TXT_RC4_128_EXPORT40_WITH_MD5,
-	ssl_enc_rc4_init,ssl_enc_rc4,ssl_compute_md5_mac,
+	ssl_enc_rc4_init,(void (*)())free,
+	ssl_enc_rc4,ssl_compute_md5_mac,
 	16,MD5_MAC_SIZE,1,0, SSL_CK_RC4_128_EXPORT40_WITH_MD5,40},
 #endif
 
-#ifndef NORC2
+#ifndef NO_RC2
 /* RC2_128_CBC_WITH_MD5 */
 	{	1,SSL_TXT_RC2_128_CBC_WITH_MD5,
-	ssl_enc_rc2_cbc_init,ssl_enc_rc2_cbc,ssl_compute_md5_mac,
+	ssl_enc_rc2_cbc_init,(void (*)())free,
+	ssl_enc_rc2_cbc,ssl_compute_md5_mac,
 	16,MD5_MAC_SIZE,8,8,SSL_CK_RC2_128_CBC_WITH_MD5,0},
 
 /* RC2_128_CBC_EXPORT40_WITH_MD5 */
 	{	1,SSL_TXT_RC2_128_CBC_EXPORT40_WITH_MD5,
-	ssl_enc_rc2_cbc_init,ssl_enc_rc2_cbc,ssl_compute_md5_mac,
+	ssl_enc_rc2_cbc_init,(void (*)())
+	free,ssl_enc_rc2_cbc,ssl_compute_md5_mac,
 	16,MD5_MAC_SIZE,8,8,SSL_CK_RC2_128_CBC_EXPORT40_WITH_MD5,40},
 #endif
 
-#ifndef NOIDEA
+#ifndef NO_IDEA
 /* IDEA_128_CBC_WITH_MD5 */
 	{	1,SSL_TXT_IDEA_128_CBC_WITH_MD5,
-	ssl_enc_idea_cbc_init,ssl_enc_idea_cbc,ssl_compute_md5_mac,
+	ssl_enc_idea_cbc_init,(void (*)())free,
+	ssl_enc_idea_cbc,ssl_compute_md5_mac,
 	16,MD5_MAC_SIZE,8,8, SSL_CK_IDEA_128_CBC_WITH_MD5,0},
 #endif
 
 /* DES_64_CBC_WITH_MD5 */
 	{	1,SSL_TXT_DES_64_CBC_WITH_MD5,
-	ssl_enc_des_cbc_init,ssl_enc_des_cbc,ssl_compute_md5_mac,
+	ssl_enc_des_cbc_init,(void (*)())free,
+	ssl_enc_des_cbc,ssl_compute_md5_mac,
 	8,MD5_MAC_SIZE,8,8,
 	SSL_CK_DES_64_CBC_WITH_MD5,0},
 
 /* DES_64_CBC_WITH_SHA */
 	{	1,SSL_TXT_DES_64_CBC_WITH_SHA,
-	ssl_enc_des_cbc_init,ssl_enc_des_cbc,ssl_compute_sha_mac,
+	ssl_enc_des_cbc_init,(void (*)())free,
+	ssl_enc_des_cbc,ssl_compute_sha_mac,
 	8,SHA_MAC_SIZE,8,8,
 	SSL_CK_DES_64_CBC_WITH_SHA,0},
 
 /* DES_192_EDE3_CBC_WITH_MD5 */
 	{	1,SSL_TXT_DES_192_EDE3_CBC_WITH_MD5,
-	ssl_enc_des_ede3_cbc_init,ssl_enc_des_ede3_cbc,ssl_compute_md5_mac,
+	ssl_enc_des_ede3_cbc_init,(void (*)())free,
+	ssl_enc_des_ede3_cbc,ssl_compute_md5_mac,
 	24,MD5_MAC_SIZE,8,8,
 	SSL_CK_DES_192_EDE3_CBC_WITH_MD5,0},
 
 /* DES_192_EDE3_CBC_WITH_SHA */
 	{	1,SSL_TXT_DES_192_EDE3_CBC_WITH_SHA,
-	ssl_enc_des_ede3_cbc_init,ssl_enc_des_ede3_cbc,ssl_compute_sha_mac,
+	ssl_enc_des_ede3_cbc_init,(void (*)())free,
+	ssl_enc_des_ede3_cbc,ssl_compute_sha_mac,
 	24,SHA_MAC_SIZE,8,8,
 	SSL_CK_DES_192_EDE3_CBC_WITH_SHA,0},
 
 /* DES_64_CFB64_WITH_MD5_1 SSLeay */
 	{	1,SSL_TXT_DES_64_CFB64_WITH_MD5_1,
-	ssl_enc_des_cfb_init,ssl_enc_des_cfb,ssl_compute_md5_mac_1,
+	ssl_enc_des_cfb_init,(void (*)())free,
+	ssl_enc_des_cfb,ssl_compute_md5_mac_1,
 	8,1,1,8,
 	SSL_CK_DES_64_CFB64_WITH_MD5_1,0},
 
 /* NULL SSLeay (testing) */
 	{	0,SSL_TXT_NULL,
-	ssl_enc_null_init,ssl_enc_null,ssl_compute_null_mac,
+	ssl_enc_null_init,(void (*)())free,
+	ssl_enc_null,ssl_compute_null_mac,
 	0,0,1,0,SSL_CK_NULL,0},
 
 /* end of list :-) */
 	};
 
-static int num_pref_cipher=(4 HAVE_IDEA HAVE_RC4 HAVE_RC2);
 static char *pref_cipher[]={
-#ifndef NORC4
+#ifndef NO_RC4
 	SSL_TXT_RC4_128_WITH_MD5,		/* Stream cipher but fast */
 #endif
-#ifndef NOIDEA
+#ifndef NO_IDEA
 	SSL_TXT_IDEA_128_CBC_WITH_MD5,		/* best? */
 #endif
-#ifndef NORC2
+#ifndef NO_RC2
 	SSL_TXT_RC2_128_CBC_WITH_MD5,		/* how good is RC2? */
 #endif
 	SSL_TXT_DES_64_CBC_WITH_MD5,		/* Small key */
 	SSL_TXT_DES_64_CBC_WITH_SHA,		/* Small key, SHA */
 	SSL_TXT_DES_192_EDE3_CBC_WITH_MD5,	/* paranoid but slow */
 	SSL_TXT_DES_192_EDE3_CBC_WITH_SHA,	/* paranoid but slow, SHA */
-#ifndef NORC4
+#ifndef NO_RC4
 	SSL_TXT_RC4_128_EXPORT40_WITH_MD5,	/* Smaller key easy to break */
 #endif
-#ifndef NORC2
+#ifndef NO_RC2
 	SSL_TXT_RC2_128_CBC_EXPORT40_WITH_MD5,/* Slow and Small key :-( */
 #endif
 	NULL
 	};
 
-#ifndef _Windows
+#define NUM_OF(x) (sizeof(x) / sizeof(char *))
+static int num_pref_cipher=NUM_OF(pref_cipher);
+
+#if !defined(WIN16) && !defined(_DLL)
 FILE *SSL_LOG=stderr;
 FILE *SSL_ERR=stderr;
 #else
@@ -197,8 +211,8 @@ SSL *s;
 	{
 	s->version=SSL_SERVER_VERSION;
 	s->rwstate=SSL_NOTHING;
-	s->state=ST_BEFORE;
-	s->rstate=ST_READ_HEADER;
+	s->state=SSL_ST_BEFORE;
+	s->rstate=SSL_ST_READ_HEADER;
 	if (s->init_buf != NULL) free(s->init_buf);
 	s->init_buf=NULL;
 
@@ -269,8 +283,8 @@ SSL_CTX *ctx;
 	s=(SSL *)malloc(sizeof(SSL));
 	if (s == NULL) goto err;
 
-	s->rfd=-1;
-	s->wfd=-1;
+	s->rbio=NULL;
+	s->wbio=NULL;
 	s->init_buf=NULL;
 	s->rbuf=(unsigned char *)malloc(SSL_MAX_RECORD_LENGTH_2_BYTE_HEADER+2);
 	if (s->rbuf == NULL) goto err;
@@ -284,13 +298,15 @@ SSL_CTX *ctx;
 	s->key_material=NULL;
 	s->session=NULL;
 	s->cert=NULL;
-	s->peer=NULL;
-	s->peer_status=SSL_PEER_NOT_SET;
 	s->verify_mode=SSL_VERIFY_NONE;
 	s->verify_callback=NULL;
 	s->ctx=ctx;
+	s->debug=0;
+	s->info_callback=NULL;
 	CS_BEGIN;
+	CRYPTO_w_lock(CRYPTO_LOCK_SSL_CTX);
 	ctx->references++;
+	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
 	CS_END;
 	s->state_ccl=NULL;
 
@@ -304,6 +320,11 @@ err:
 void SSL_free(s)
 SSL *s;
 	{
+	if (s->rbio != NULL)
+		BIO_free(s->rbio);
+	if ((s->wbio != NULL) && (s->wbio != s->rbio))
+		BIO_free(s->wbio);
+
 	/* add extra stuff */
 	free(s->rbuf);
 	free(s->wbuf);
@@ -313,8 +334,10 @@ SSL *s;
 		free(s->cipher_list);
 		}
 	if (s->init_buf != NULL) free(s->init_buf);
-	if (s->crypt_state != NULL) free(s->crypt_state);
-	/* again, just let them linger around */
+	if ((s->session != NULL) && (s->session->cipher != NULL) &&
+		(s->session->cipher->crypt_cleanup != NULL) &&
+		(s->crypt_state != NULL))
+		s->session->cipher->crypt_cleanup(s->crypt_state);
 	if (s->session != NULL) SSL_SESSION_free(s->session);
 	if (s->cert != NULL) ssl_cert_free(s->cert);
 	if (s->challenge != NULL) free(s->challenge);
@@ -322,8 +345,6 @@ SSL *s;
 	if (s->key_material != NULL) free(s->key_material);
 	if (s->state_ccl != NULL) free(s->state_ccl);
 	/* free up if allocated */
-	if ((s->peer_status == SSL_PEER_IN_SSL) && (s->peer != NULL))
-		X509_free(s->peer);
 
 	if (s->ctx) SSL_CTX_free(s->ctx);
 	free((char *)s);
@@ -347,9 +368,9 @@ char *b;
 
 	for (i=0; i<n; i++)
 		{
-		Fputc(h[(b[i]>>4)&0x0f],f);
-		Fputc(h[(b[i]   )&0x0f],f);
-		Fputc(' ',f);
+		fputc(h[(b[i]>>4)&0x0f],f);
+		fputc(h[(b[i]   )&0x0f],f);
+		fputc(' ',f);
 		}
 #endif
 
@@ -366,19 +387,104 @@ SSL *s;
 #endif
 	}
 
-
-void SSL_set_fd(s, fd)
+void SSL_set_bio(s, rbio,wbio)
 SSL *s;
-int fd;
+BIO *rbio;
+BIO *wbio;
 	{
-	s->rfd=fd;
-	s->wfd=fd;
+	if ((s->rbio != NULL) && (s->rbio != rbio))
+		BIO_free(s->rbio);
+	if ((s->wbio != NULL) && (s->wbio != wbio) && (s->rbio != s->rbio))
+		BIO_free(s->wbio);
+	s->rbio=rbio;
+	s->wbio=wbio;
 	}
 
+BIO *SSL_get_rbio(s)
+SSL *s;
+	{ return(s->rbio); }
+
+BIO *SSL_get_wbio(s)
+SSL *s;
+	{ return(s->wbio); }
+
 int SSL_get_fd(s)
 SSL *s;
 	{
-	return(s->rfd);
+	int ret=-1;
+	BIO *b;
+
+	b=SSL_get_rbio(s);
+	if (b != NULL)
+		BIO_get_fd(b,&ret);
+	return(ret);
+	}
+
+int SSL_set_fd(s, fd)
+SSL *s;
+int fd;
+	{
+	int ret=0;
+	BIO *bio=NULL;
+
+	bio=BIO_new(BIO_s_socket());
+
+	if (bio == NULL)
+		{
+		SSLerr(SSL_F_SSL_SET_FD,ERR_R_BUF_LIB);
+		goto err;
+		}
+	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
+	if (s->rbio != NULL)
+		BIO_free(s->rbio);
+	if ((s->wbio != NULL) && (s->wbio != s->rbio))
+		BIO_free(s->wbio);
+	s->rbio=bio;
+	s->wbio=bio;
+	ret=1;
+err:
+	return(ret);
+	}
+
+int SSL_set_wfd(s, fd)
+SSL *s;
+int fd;
+	{
+	int ret=0;
+	BIO *bio=NULL;
+
+	bio=BIO_new(BIO_s_socket());
+
+	if (bio == NULL)
+		{ SSLerr(SSL_F_SSL_SET_WFD,ERR_R_BUF_LIB); goto err; }
+	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
+	if (s->wbio != NULL) BIO_free(s->wbio);
+	s->wbio=bio;
+	ret=1;
+err:
+	return(ret);
+	}
+
+int SSL_set_rfd(s, fd)
+SSL *s;
+int fd;
+	{
+	int ret=0;
+	BIO *bio=NULL;
+
+	bio=BIO_new(BIO_s_socket());
+
+	if (bio == NULL)
+		{
+		SSLerr(SSL_F_SSL_SET_RFD,ERR_R_BUF_LIB);
+		goto err;
+		}
+	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
+	if (s->wbio != NULL) BIO_free(s->wbio);
+	s->rbio=bio;
+	ret=1;
+err:
+	return(ret);
 	}
 
 int SSL_set_cipher_list(s, str)
@@ -397,7 +503,7 @@ char *str;
 	char *p,**pp;
 
 	if (str == NULL) return(1);
-	if (p == NULL) return(0);
+	if (rp == NULL) return(0);
 	if (n == NULL) return(0);
 
 	p=str;
@@ -431,7 +537,7 @@ char *str;
 	*n=i;
 	return(1);
 err:
-	SSLerr(SSL_F_SSL_NEW,ERR_R_MALLOC_FAILURE);
+	SSLerr(SSL_F_SSL_MAKE_CIPHER_LIST,ERR_R_MALLOC_FAILURE);
 	return(0);
 	}
 
@@ -481,7 +587,6 @@ int len;
 	p=buf;
 	c= &(s->session->ciphers[0]);
 	len--;
-	i=0;
 	for (i=0; i<s->session->num_ciphers; i++)
 		{
 		for (cp=c[i]->name; *cp; )
@@ -528,30 +633,30 @@ SSL *s;
 	return(s->rbuf_left);
 	}
 
-int SSL_is_init_finished(s)
-SSL *s;
-	{
-	return(s->state == ST_OK);
-	}
-
 X509 *SSL_get_peer_certificate(s)
 SSL *s;
 	{
-	X509 *r=NULL;
+	X509 *r;
 	
-	if (s->peer_status == SSL_PEER_NOT_SET)
+	CRYPTO_r_lock(CRYPTO_LOCK_SSL_SESSION);
+	CRYPTO_w_lock(CRYPTO_LOCK_SSL_CTX);
+
+	if ((s == NULL) || (s->session == NULL))
 		r=NULL;
-	else if (s->peer_status == SSL_PEER_IN_SSL)
-		r=s->peer;
-	else if ((s->peer_status == SSL_PEER_IN_CERT) &&
-		(s->session != NULL) &&
-		(s->session->cert != NULL))
-		r=s->session->cert->x509;
+	else
+		r=s->session->peer;
+
+	CRYPTO_r_unlock(CRYPTO_LOCK_SSL_SESSION);
+
+	if (r == NULL)
+		{
+		CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
+		return(0);
+		}
 
-	CS_BEGIN;
-	if (r == NULL) return(0);
 	r->references++;
-	CS_END;
+	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
+
 	return(r);
 	}
 
@@ -602,3 +707,21 @@ SSL *s;
 		km+=MD5_DIGEST_LENGTH;
 		}
 	}
+
+/* Now in theory, since the calling process own 't' it should be safe to
+ * modify.  We need to be able to read f without being hassled */
+void SSL_copy_session_id(t,f)
+SSL *t,*f;
+	{
+	CRYPTO_r_lock(CRYPTO_LOCK_SSL);
+
+	SSL_set_session(t,SSL_get_session(f));
+
+	CRYPTO_r_lock(CRYPTO_LOCK_X509);
+	if (t->cert != NULL) ssl_cert_free(t->cert);
+	if (f->cert != NULL) f->cert->references++;
+
+	t->cert=f->cert;
+	CRYPTO_r_unlock(CRYPTO_LOCK_X509);
+	CRYPTO_r_unlock(CRYPTO_LOCK_SSL);
+	}
