@@ -1,5 +1,5 @@
 /* ssl/ssl_locl.h */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -50,19 +50,14 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
-#ifdef WIN32
-#include <io.h>
-#else
-#include <unistd.h>
-#endif
 
+#include "../e_os.h"
+
+#include "crypto.h"
+#include "buffer.h"
 #include "rsa.h"
 #include "x509.h"
 
-#if defined(NOCONST)
-#define const
-#endif
-
 #define c2l(c,l)	(l = ((unsigned long)(*((c)++)))     , \
 			 l|=(((unsigned long)(*((c)++)))<< 8), \
 			 l|=(((unsigned long)(*((c)++)))<<16), \
@@ -119,24 +114,8 @@
 #define s2n(s,c)	(*((c)++)=(unsigned char)(((s)>> 8)&0xff), \
 			 *((c)++)=(unsigned char)(((s)    )&0xff))
 
-/* The next 2 macros are for use when I get around to implementing threads.
- * They wrap things that cannot be multi-threaded */
-#define	CS_BEGIN	
-#define	CS_END
-
 /* LOCAL STUFF */
 
-#ifdef _Windows
-#include <winsock.h>
-#define Write(a,b,c)	send((a),(b),(c),0)
-#define Read(a,b,c)	recv((a),(b),(c),0)
-#define Fputc(a,b)	fputc((a),(b))
-#else
-#define Read(a,b,c)	read((a),(b),(c))
-#define Write(a,b,c)	write((a),(b),(c))
-#define Fputc(a,b)	fputc((a),(b))
-#endif
-
 #define SSL_DECRYPT	0
 #define SSL_ENCRYPT	1
 
@@ -158,22 +137,17 @@
 #define MAX_CERT_CHALLENGE_LENGTH	32
 #define MIN_CERT_CHALLENGE_LENGTH	16
 
-/* used in SSL->peer_status */
-#define SSL_PEER_NOT_SET	0
-#define SSL_PEER_IN_SSL		1
-#define SSL_PEER_IN_CERT	2
-
-#ifndef NOIDEA
+#ifndef NO_IDEA
 #define HAVE_IDEA	+1
 #else
 #define HAVE_IDEA
 #endif
-#ifndef NORC4
+#ifndef NO_RC4
 #define HAVE_RC4	+2
 #else
 #define HAVE_RC4
 #endif
-#ifndef NORC4
+#ifndef NO_RC4
 #define HAVE_RC2	+2
 #else
 #define HAVE_RC2
@@ -186,6 +160,7 @@ typedef struct cipher_st
 	int valid;
 	char *name;
 	int (*crypt_init)();
+	void (*crypt_cleanup)();
 	void (*crypt)();
 	void (*hash)();
 	unsigned int key_size;
@@ -223,8 +198,13 @@ typedef struct ssl_session_st
 	unsigned int session_id_length;
 	unsigned char *session_id;
 
+	/* The cert is the certificate used to establish this connection */
 	struct cert_st *cert;
 
+	/* This is the cert for the other end.  On servers, it will be
+	 * the same as cert->x509 */
+	X509 *peer;
+
 	unsigned int references;
 	unsigned long timeout;
 	unsigned long time;
@@ -241,12 +221,12 @@ typedef struct cert_st
 	int (*public_encrypt)();
 	int (*private_decrypt)();
 	X509 *x509;
-	RSA *publickey; /* when extracted */
-	RSA *privatekey;
+	EVP_PKEY *publickey; /* when extracted */
+	EVP_PKEY *privatekey;
 	int references;
 	} CERT;
 
-extern CIPHER ssl_ciphers[];
+extern CIPHER ssl_ciphers[NUM_CIPHERS];
 
 /*#define MAC_DEBUG	*/
 
@@ -265,34 +245,21 @@ extern CIPHER ssl_ciphers[];
 /* this is for debug and tracing stuff */
 #include "ssl_trc.h"
 
-#ifndef NOPROTO
-#define P_CC_CC	const void *,const void *
-#else
-#define P_CC_CC
-#endif
-
 /* these are now externally visible ... in ssl.h 
 extern FILE *SSL_LOG;
 extern FILE *SSL_ERR;
 */
 
-extern int errno;
+#include <errno.h>
 
 #ifdef PKT_DEBUG
 void ssl_debug();
 #endif
 
-#ifdef _WINDLL
-#define Fprintf WIN_fprintf
-#else
-#define Fprintf fprintf
-#endif
-
 #ifndef NOPROTO
 CERT *ssl_cert_new(void);
 void ssl_cert_free(CERT *c);
 void ssl_return_error(SSL *s);
-RSA *ssl_rsa_extract_public_key(X509 *x);
 int ssl_set_certificate(SSL *s, int type, int len, unsigned char *data);
 int ssl_set_cert_type(CERT *c, int type);
 int ssl_get_new_session(SSL *s, int session);
@@ -310,7 +277,6 @@ int ssl_make_cipher_list(char ***rp,int *n,char *str);
 CERT *ssl_cert_new();
 void ssl_cert_free();
 void ssl_return_error();
-RSA *ssl_rsa_extract_public_key();
 int ssl_set_certificate();
 int ssl_set_cert_type();
 int ssl_get_new_session();
