@@ -1,5 +1,5 @@
 /* ssl/ssl_pkt.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -47,9 +47,17 @@
 
 #include <stdio.h>
 #include <errno.h>
+#define USE_SOCKETS
 #include "ssl_locl.h"
 #include "ssl_trc.h"
 
+/* SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_NO_CIPHER);
+ * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PERR_ERROR_NO_CERTIFICATE);
+ * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_CERTIFICATE);
+ * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE);
+ * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_UNKNOWN_REMOTE_ERROR_TYPE);
+ */
+
 #ifndef NOPROTO
 static int read_n(SSL *s,unsigned int n,unsigned int max,unsigned int extend);
 static int do_ssl_write(SSL *s, const char *buf, unsigned int len);
@@ -88,18 +96,18 @@ unsigned int len;
 		s->ract_data_length-=n;
 		s->ract_data+=n;
 		if (s->ract_data_length == 0)
-			s->rstate=ST_READ_HEADER;
+			s->rstate=SSL_ST_READ_HEADER;
 		return(n);
 		}
 
-	if (s->rstate == ST_READ_HEADER)
+	if (s->rstate == SSL_ST_READ_HEADER)
 		{
 		n=read_n(s,2,SSL_MAX_RECORD_LENGTH_2_BYTE_HEADER+2,0);
 
 		if (n <= 0) return(n); /* error or non-blocking */
 		/* part read stuff */
 
-		s->rstate=ST_READ_BODY;
+		s->rstate=SSL_ST_READ_BODY;
 		pp=s->packet;
 		/* Do header */
 		s->padding=0;
@@ -120,7 +128,7 @@ unsigned int len;
 			}
 		}
 
-	if (s->rstate == ST_READ_BODY)
+	if (s->rstate == SSL_ST_READ_BODY)
 		{
 		n=s->length+2+s->three_byte_header;
 		if (n > (int)s->packet_length)
@@ -128,11 +136,10 @@ unsigned int len;
 			n-=s->packet_length;
 			i=read_n(s,(unsigned int)n,(unsigned int)n,1);
 			if (i <= 0) return(i); /* ERROR */
-			pp= &(s->packet[2]);
 			}
 
-		s->rstate=ST_READ_HEADER;
 		pp= &(s->packet[2]);
+		s->rstate=SSL_ST_READ_HEADER;
 		if (s->three_byte_header)
 			s->padding= *(pp++);
 		else	s->padding=0;
@@ -241,8 +248,16 @@ unsigned int extend;
 	while (new < (int)n)
 		{
 		errno=0;
-		i=Read(s->rfd,&(s->rbuf[off+new]),max-new);
+		if (s->rbio != NULL)
+			i=BIO_read(s->rbio,(char *)&(s->rbuf[off+new]),max-new);
+		else
+			{
+			SSLerr(SSL_F_READ_N,SSL_R_NO_READ_METHOD_SET);
+			i=-1;
+			}
+#ifdef PKT_DEBUG
 		if (s->debug & 0x01) sleep(1);
+#endif
 		if (i <= 0)
 			{
 			s->rbuf_left+=new;
@@ -276,7 +291,7 @@ const char *buf;
 unsigned int len;
 	{
 	unsigned int n,tot;
-	long i;
+	int i;
 
 	s->rwstate=SSL_NOTHING;
 
@@ -286,7 +301,7 @@ unsigned int len;
 	n=(len-tot);
 	for (;;)
 		{
-		i=(long)do_ssl_write(s,&(buf[tot]),n);
+		i=do_ssl_write(s,&(buf[tot]),n);
 		if (i <= 0)
 			{
 			s->wnum=tot;
@@ -312,15 +327,25 @@ unsigned int len;
 	 * write */
 	if ((s->wpend_tot != (int)len) || (s->wpend_buf != buf))
 		{
-		SSLerr(SSL_F_SSL_WRITE,SSL_R_BAD_WRITE_RETRY);
+		SSLerr(SSL_F_WRITE_PENDING,SSL_R_BAD_WRITE_RETRY);
 		return(SSL_RWERR_BAD_WRITE_RETRY);
 		}
 	for (;;)
 		{
 		errno=0;
-		i=Write(s->wfd,&(s->write_ptr[s->wpend_off]),
-			(unsigned int)s->wpend_len);
+		if (s->wbio != NULL)
+			{
+			i=BIO_write(s->wbio,(char *)&(s->write_ptr[s->wpend_off]),
+				(unsigned int)s->wpend_len);
+			}
+		else
+			{
+			SSLerr(SSL_F_WRITE_PENDING,SSL_R_NO_WRITE_METHOD_SET);
+			i=-1;
+			}
+#ifdef PKT_DEBUG
 		if (s->debug & 0x01) sleep(1);
+#endif
 		if (i == s->wpend_len)
 			{
 			s->wpend_len=0;
@@ -467,7 +492,7 @@ int i;
 			int i;
 
 			i=(s->init_buf[1]<<8)|s->init_buf[2];
-			SSLerr(f,ssl_mt_error(i));
+			SSLerr((int)f,ssl_mt_error(i));
 			}
 		}
 
@@ -508,30 +533,28 @@ SSL *s;
 	return(0);
 	}
 
-/* SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_NO_CIPHER);
- * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PERR_ERROR_NO_CERTIFICATE);
- * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_CERTIFICATE);
- * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE);
- * SSLerr(SSL_F_GET_SERVER_HELLO,SSL_R_UNKNOWN_REMOTE_ERROR_TYPE);
- */
 static int ssl_mt_error(n)
 int n;
 	{
+	int ret;
+
 	switch (n)
 		{
 	case SSL_PE_NO_CIPHER:
-		return(SSL_R_PEER_ERROR_NO_CIPHER);
+		ret=SSL_R_PEER_ERROR_NO_CIPHER;
 		break;
 	case SSL_PE_NO_CERTIFICATE:
-		return(SSL_R_PERR_ERROR_NO_CERTIFICATE);
+		ret=SSL_R_PERR_ERROR_NO_CERTIFICATE;
 		break;
 	case SSL_PE_BAD_CERTIFICATE:
-		return(SSL_R_PEER_ERROR_CERTIFICATE);
+		ret=SSL_R_PEER_ERROR_CERTIFICATE;
 		break;
 	case SSL_PE_UNSUPPORTED_CERTIFICATE_TYPE:
-		return(SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE);
+		ret=SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE;
 		break;
 	default:
-		return(SSL_R_UNKNOWN_REMOTE_ERROR_TYPE);
+		ret=SSL_R_UNKNOWN_REMOTE_ERROR_TYPE;
+		break;
 		}
+	return(ret);
 	}
