@@ -1,5 +1,5 @@
 /* ssl/ssl_rc4.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -45,7 +45,7 @@
  * [including the GNU Public Licence.]
  */
 
-#ifndef NORC4
+#ifndef NO_RC4
 #include <stdio.h>
 #include "rc4.h"
 #include "md5.h"
@@ -58,14 +58,12 @@ int client;
 	{
 	unsigned char *md5d1,*md5d2;
 	RC4_STATE *rc4s;
-	SSL_SESSION *c;
 
 	if (s->crypt_state == NULL)
 		s->crypt_state=(char *)malloc(sizeof(RC4_STATE));
 	if (s->crypt_state == NULL) goto err;
 	rc4s=(RC4_STATE *)s->crypt_state;
 
-	c=s->session;
 	if (s->key_material != NULL)
 		{
 		memset(s->key_material,0,s->key_material_length);
