@@ -1,5 +1,5 @@
 /* ssl/ssl_rsa.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,6 +46,8 @@
  */
 
 #include <stdio.h>
+#include "buffer.h"
+#include "objects.h"
 #include "rsa.h"
 #include "x509.h"
 #include "pem.h"
@@ -63,10 +65,16 @@ unsigned char *to;
 
 	if ((c == NULL) || (c->publickey == NULL))
 		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PUBLIC_ENCRYPT,SSL_R_NO_PUBLICKEY);
+		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,SSL_R_NO_PUBLICKEY);
 		return(-1);
 		}
-	rsa=c->publickey;
+	if (c->publickey->type != EVP_PKEY_RSA)
+		{
+		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,SSL_R_PUBLIC_KEY_IS_NOT_RSA);
+		return(-1);
+		}
+
+	rsa=c->publickey->pkey.rsa;
 	/* we have the public key */
 #ifdef RSA_DEBUG
 	SSL_TRACE(SSL_ERR,"PUBLIC RSA INPUT %d %d\n",len,to);
@@ -78,7 +86,7 @@ unsigned char *to;
 	i=RSA_public_encrypt(len,from,to,rsa);
 	if (i < 0)
 		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PUBLIC_ENCRYPT,ERR_R_RSA_LIB);
+		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,ERR_R_RSA_LIB);
 		}
 #ifdef RSA_DEBUG
 	SSL_TRACE(SSL_ERR,"PUBLIC RSA OUTPUT %d %08x\n",i,to);
@@ -88,43 +96,6 @@ unsigned char *to;
 	return(i);
 	}
 
-#ifdef undef
-int ssl_rsa_private_encrypt(c, len, from, to)
-CERT *c;
-int len;
-unsigned char *from;
-unsigned char *to;
-	{
-	RSA *rsa;
-	int i;
-
-	if ((c == NULL) || (c->privatekey == NULL))
-		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PRIVATE_ENCRYPT,SSL_R_NO_PRIVATEKEY);
-		return(-1);
-		}
-	rsa=c->privatekey;
-	/* we have the private key */
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PRIVATE RSA INPUT %d %d\n",len,to);
-	for (j=0; j<len; j++) SSL_TRACE(SSL_ERR,"%02X ",from[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	i=RSA_private_encrypt(len,from,to,rsa);
-	if (i < 0)
-		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PRIVATE_ENCRYPT,ERR_R_RSA_LIB);
-		}
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PRIVATE RSA OUTPUT %d %d\n",i,to);
-	for (j=0; j<i; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	return(i);
-	}
-
-#endif
-
 int ssl_rsa_private_decrypt(c, len, from, to)
 CERT *c;
 int len;
@@ -136,10 +107,15 @@ unsigned char *to;
 
 	if ((c == NULL) || (c->privatekey == NULL))
 		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PRIVATE_DECRYPT,SSL_R_NO_PRIVATEKEY);
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_NO_PRIVATEKEY);
 		return(-1);
 		}
-	rsa=c->privatekey;
+	if (c->privatekey->type != EVP_PKEY_RSA)
+		{
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_PUBLIC_KEY_IS_NOT_RSA);
+		return(-1);
+		}
+	rsa=c->privatekey->pkey.rsa;
 
 	/* we have the public key */
 #ifdef RSA_DEBUG
@@ -149,7 +125,7 @@ unsigned char *to;
 #endif
 	i=RSA_private_decrypt(len,from,to,rsa);
 	if (i < 0)
-		SSLerr(SSL_F_SSL_CERT_RSA_PRIVATE_DECRYPT,ERR_R_RSA_LIB);
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,ERR_R_RSA_LIB);
 #ifdef RSA_DEBUG
 	SSL_TRACE(SSL_ERR,"PRIVATE RSA OUTPUT %d\n",i);
 	for (j=0; j<i; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
@@ -163,6 +139,7 @@ SSL *ssl;
 X509 *x;
 	{
 	CERT *c;
+	EVP_PKEY *pkey;
 
 	if (ssl->cert == NULL)
 		{
@@ -173,14 +150,27 @@ X509 *x;
 	c=ssl->cert;
 
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->x509 != NULL) X509_free(x);
-	c->x509=x;
+	if (c->x509 != NULL) X509_free(c->x509);
 	CS_BEGIN;
+	c->x509=x;
 	x->references++;
 	CS_END;
-	if (c->publickey != NULL) RSA_free(c->publickey);
-	c->publickey=X509_extract_key(x);
-	if (c->publickey == NULL) return(0);
+	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
+	pkey=X509_extract_key(x);
+	if (pkey == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_CERTIFICATE,SSL_R_X509_LIB);
+		return(0);
+		}
+#ifdef undef
+	if (pkey->type != EVP_PKEY_RSA)
+		{
+		EVP_PKEY_free(pkey);
+		SSLerr(SSL_F_SSL_USE_CERTIFICATE,SSL_R_WRONG_PUBLIC_KEY_TYPE);
+		return(0);
+		}
+#endif
+	c->publickey=pkey;
 	return(1);
 	}
 
@@ -190,41 +180,55 @@ char *file;
 int type;
 	{
 	int j;
-	FILE *in;
-	X509 *x;
+	BIO *in;
+	int ret=0;
+	X509 *x=NULL;
 
-	in=fopen(file,"r");
+#ifdef WIN16
+	in=BIO_new(BIO_s_file_internal_w16());
+#else
+	in=BIO_new(BIO_s_file());
+#endif
 	if (in == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_CERTIFICATE_FILE,ERR_R_BUF_LIB);
+		goto end;
+		}
+
+	if (BIO_read_filename(in,file) <= 0)
 		{
 		SYSerr(ERR_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_USE_CERTIFICATE_FILE,ERR_R_SYS_LIB);
-		return(0);
+		goto end;
 		}
 	if (type == SSL_FILETYPE_ASN1)
 		{
 		j=ERR_R_ASN1_LIB;
-		x=d2i_X509_fp(in,NULL);
+		x=d2i_X509_bio(in,NULL);
 		}
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		x=PEM_read_X509(in,NULL,NULL);
+		x=PEM_read_bio_X509(in,NULL,NULL);
 		}
 	else
 		{
 		SSLerr(SSL_F_SSL_USE_CERTIFICATE_FILE,SSL_R_BAD_SSL_FILETYPE);
-		fclose(in);
-		return(0);
+		goto end;
 		}
-	fclose(in);
 
 	if (x == NULL)
 		{
 		SSLerr(SSL_F_SSL_USE_CERTIFICATE_FILE,j);
-		return(0);
+		goto end;
 		}
 
-	return(SSL_use_certificate(ssl,x));
+	ret=SSL_use_certificate(ssl,x);
+	ret=1;
+end:
+	if (x != NULL) X509_free(x);
+	if (in != NULL) BIO_free(in);
+	return(ret);
 	}
 
 int SSL_use_certificate_ASN1(ssl, len, d)
@@ -233,6 +237,7 @@ int len;
 unsigned char *d;
 	{
 	X509 *x;
+	int ret;
 
 	x=d2i_X509(NULL,&d,(long)len);
 	if (x == NULL)
@@ -241,7 +246,9 @@ unsigned char *d;
 		return(0);
 		}
 
-	return(SSL_use_certificate(ssl,x));
+	ret=SSL_use_certificate(ssl,x);
+	X509_free(x);
+	return(ret);
 	}
 
 int SSL_use_RSAPrivateKey(ssl, rsa)
@@ -263,8 +270,15 @@ RSA *rsa;
 	c=ssl->cert;
 
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->privatekey) RSA_free(c->privatekey);
-	c->privatekey=rsa;
+	if (c->privatekey) EVP_PKEY_free(c->privatekey);
+	if ((c->privatekey=EVP_PKEY_new()) == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY,ERR_R_EVP_LIB);
+		return(0);
+		}
+	c->privatekey->type=EVP_PKEY_RSA;
+	c->privatekey->pkey.rsa=rsa;
+
 	CS_BEGIN;
 	rsa->references++;
 	CS_END;
@@ -276,42 +290,52 @@ SSL *ssl;
 char *file;
 int type;
 	{
-	int j;
-	FILE *in;
-	RSA *rsa;
+	int j,ret=0;
+	BIO *in;
+	RSA *rsa=NULL;
 
-	in=fopen(file,"r");
+#ifdef WIN16
+	in=BIO_new(BIO_s_file_internal_w16());
+#else
+	in=BIO_new(BIO_s_file());
+#endif
 	if (in == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,ERR_R_BUF_LIB);
+		goto end;
+		}
+
+	if (BIO_read_filename(in,file) <= 0)
 		{
 		SYSerr(ERR_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,ERR_R_SYS_LIB);
-		return(0);
+		goto end;
 		}
 	if	(type == SSL_FILETYPE_ASN1)
 		{
 		j=ERR_R_ASN1_LIB;
-		rsa=d2i_RSAPrivateKey_fp(in,NULL);
+		rsa=d2i_RSAPrivateKey_bio(in,NULL);
 		}
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		rsa=PEM_read_RSAPrivateKey(in,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
 		}
 	else
 		{
-		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,
-			SSL_R_BAD_SSL_FILETYPE);
-		fclose(in);
-		return(0);
+		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,SSL_R_BAD_SSL_FILETYPE);
+		goto end;
 		}
-	fclose(in);
 	if (rsa == NULL)
 		{
 		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,j);
-		return(0);
+		goto end;
 		}
-
-	return(SSL_use_RSAPrivateKey(ssl,rsa));
+	ret=SSL_use_RSAPrivateKey(ssl,rsa);
+	RSA_free(rsa);
+end:
+	if (in != NULL) BIO_free(in);
+	return(ret);
 	}
 
 int SSL_use_RSAPrivateKey_ASN1(ssl,d,len)
@@ -319,6 +343,7 @@ SSL *ssl;
 unsigned char *d;
 long len;
 	{
+	int ret;
 	unsigned char *p;
 	RSA *rsa;
 
@@ -329,42 +354,106 @@ long len;
 		return(0);
 		}
 
-	return(SSL_use_RSAPrivateKey(ssl,rsa));
+	ret=SSL_use_RSAPrivateKey(ssl,rsa);
+	RSA_free(rsa);
+	return(ret);
 	}
 
-#ifdef undef
-int ssl_rsa_public_decrypt(c, len, from, to)
-CERT *c;
-int len;
-unsigned char *from;
-unsigned char *to;
+int SSL_use_PrivateKey(ssl, pkey)
+SSL *ssl;
+EVP_PKEY *pkey;
 	{
-	RSA *rsa;
-	int i;
+	CERT *c;
 
-	if ((c == NULL) || (c->publickey == NULL))
+	if (ssl->cert == NULL)
 		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PUBLIC_DECRYPT,SSL_R_NO_PUBLICKEY);
-		return(-1);
+		c=ssl_cert_new();
+		if (c == NULL)
+			{
+			SSLerr(SSL_F_SSL_USE_PRIVATEKEY,ERR_R_MALLOC_FAILURE);
+			return(0);
+			}
+		ssl->cert=c;
 		}
-	rsa=c->publickey;
-	/* we have the public key */
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PUBLIC RSA INPUT %d\n",len);
-	for (j=0; j<len; j++) SSL_TRACE(SSL_ERR,"%02X ",from[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	i=RSA_public_decrypt(len,from,to,rsa);
-	if (i < 0)
-		{
-		SSLerr(SSL_F_SSL_CERT_RSA_PUBLIC_DECRYPT,ERR_R_RSA_LIB);
-		}
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PUBLIC RSA OUTPUT %d\n",i);
-	for (j=0; j<i; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	return(i);
+	c=ssl->cert;
+
+	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
+	if (c->privatekey) EVP_PKEY_free(c->privatekey);
+	c->privatekey=pkey;
+	CS_BEGIN;
+	pkey->references++;
+	CS_END;
+	return(1);
 	}
+
+int SSL_use_PrivateKey_file(ssl, file, type)
+SSL *ssl;
+char *file;
+int type;
+	{
+	int j,ret=0;
+	BIO *in;
+	EVP_PKEY *pkey=NULL;
+
+#ifdef WIN16
+	in=BIO_new(BIO_s_file_internal_w16());
+#else
+	in=BIO_new(BIO_s_file());
 #endif
+	if (in == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_FILE,ERR_R_BUF_LIB);
+		goto end;
+		}
+
+	if (BIO_read_filename(in,file) <= 0)
+		{
+		SYSerr(ERR_F_FOPEN,errno);
+		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_FILE,ERR_R_SYS_LIB);
+		goto end;
+		}
+	if (type == SSL_FILETYPE_PEM)
+		{
+		j=ERR_R_PEM_LIB;
+		pkey=PEM_read_bio_PrivateKey(in,NULL,NULL);
+		}
+	else
+		{
+		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_FILE,SSL_R_BAD_SSL_FILETYPE);
+		goto end;
+		}
+	if (pkey == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_FILE,j);
+		goto end;
+		}
+	ret=SSL_use_PrivateKey(ssl,pkey);
+	EVP_PKEY_free(pkey);
+end:
+	if (in != NULL) BIO_free(in);
+	return(ret);
+	}
+
+int SSL_use_PrivateKey_ASN1(type,ssl,d,len)
+int type;
+SSL *ssl;
+unsigned char *d;
+long len;
+	{
+	int ret;
+	unsigned char *p;
+	EVP_PKEY *pkey;
+
+	p=d;
+	if ((pkey=d2i_PrivateKey(type,&pkey,&p,(long)len)) == NULL)
+		{
+		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_ASN1,ERR_R_ASN1_LIB);
+		return(0);
+		}
+
+	ret=SSL_use_PrivateKey(ssl,pkey);
+	EVP_PKEY_free(pkey);
+	return(ret);
+	}
+
 
