@@ -1,5 +1,5 @@
-/* ssl/ssl_session.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* ssl/ssl_sess.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -69,7 +69,14 @@ SSL_SESSION *SSL_SESSION_new()
 	ss->key_arg=NULL;
 	ss->master_key_length=0;
 	ss->master_key=NULL;
+	ss->session_id_length=0;
 	ss->session_id=NULL;
+
+	ss->cert=NULL;
+	ss->peer=NULL;
+	ss->references=1;
+	ss->timeout=60*5; /* minute timeout by default */
+	ss->time=time(NULL);
 	return(ss);
 	}
 
@@ -93,7 +100,7 @@ int session;
 		ss->session_id=(unsigned char *)malloc(SSL_SESSION_ID_LENGTH);
 		if (ss->session_id == NULL)
 			{
-			SSLerr(SSL_F_GET_NEW_SSL_SESSION,ERR_R_MALLOC_FAILURE);
+			SSLerr(SSL_F_SSL_GET_NEW_SESSION,ERR_R_MALLOC_FAILURE);
 			SSL_SESSION_free(ss);
 			return(0);
 			}
@@ -114,10 +121,6 @@ int session;
 		ss->session_id_length=0;
 		}
 
-	ss->cert=NULL;
-	ss->references=1;
-	ss->timeout=100;
-	ss->time=time(NULL);
 	s->session=ss;
 	return(1);
 	}
@@ -136,7 +139,16 @@ unsigned char *session;
 		== NULL)
 		{
 		s->ctx->sess_miss++;
-		return(0);
+		ret=NULL;
+		if ((s->ctx->get_session_cb != NULL) &&
+			((ret=s->ctx->get_session_cb(session,len)) != NULL))
+			{
+			s->ctx->sess_cb_hit++;
+			/* The following should not return 1, otherwise,
+			 * things are very strange */
+			SSL_add_session(s->ctx,ret);
+			}
+		if (ret == NULL) return(0);
 		}
 #ifdef SSL_DEBUG
 	SSL_TRACE(SSL_ERR,"GOT ONE\n");
@@ -145,7 +157,7 @@ unsigned char *session;
 	/* If a thread got the session, then 'swaped', and another got
 	 * it and then due to a time-out decided to 'free' it we could
 	 * be in trouble.  So I'll increment it now, then double decrement
-	 * later. */
+	 * later - am I speaking rubbish?. */
 	CS_BEGIN;
 	ret->references++;
 	CS_END;
@@ -172,7 +184,7 @@ unsigned char *session;
 	return(1);
 	}
 
-void SSL_add_session(ctx,c)
+int SSL_add_session(ctx,c)
 SSL_CTX *ctx;
 SSL_SESSION *c;
 	{
@@ -183,11 +195,17 @@ SSL_SESSION *c;
 	c->references++;
 	CS_END;
 	s=(SSL_SESSION *)lh_insert(ctx->sessions,(char *)c);
-	/* If the same session if is being 'readded', free the old
+	/* If the same session if is being 're-added', free the old
 	 * one when the last person stops using it.
 	 * This will also work if it is alread in the cache.
 	 * The references will go up and then down :-) */
-	if (s != NULL) SSL_SESSION_free(s);
+	if (s != NULL)
+		{
+		SSL_SESSION_free(s);
+		return(0);
+		}
+	else
+		return(1);
 	}
 
 void SSL_remove_session(ctx,c)
@@ -232,8 +250,8 @@ SSL_SESSION *ss;
 		memset(ss->session_id,0,ss->session_id_length);
 		free(ss->session_id);
 		}
-	if (ss->cert != NULL)
-		ssl_cert_free(ss->cert);
+	if (ss->cert != NULL) ssl_cert_free(ss->cert);
+	if (ss->peer != NULL) X509_free(ss->peer);
 	free(ss);
 	}
 
@@ -292,7 +310,7 @@ long t;
 
 typedef struct timeout_param_st
 	{
-	long time;
+	unsigned long time;
 	LHASH *cache;
 	} TIMEOUT_PARAM;
 
@@ -311,7 +329,7 @@ void SSL_flush_sessions(s,t)
 SSL_CTX *s;
 long t;
 	{
-	int i;
+	unsigned long i;
 	TIMEOUT_PARAM tp;
 
 	tp.cache=SSL_CTX_sessions(s);
@@ -322,3 +340,4 @@ long t;
 	lh_doall_arg(tp.cache,(void (*)())timeout,(char *)&tp);
 	tp.cache->down_load=i;
 	}
+
