@@ -1,5 +1,5 @@
 /* ssl/ssl_srvr.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -79,19 +79,24 @@ int SSL_accept(s)
 SSL *s;
 	{
 	unsigned long l=time(NULL);
-	BUFFER *buf=NULL;
-	int ret;
+	BUF_MEM *buf=NULL;
+	int ret= -1;
+	void (*cb)()=NULL;
 
 	RAND_seed((unsigned char *)&l,sizeof(l));
-
 	ERR_clear_error();
+	errno=0;
+
+	if (s->info_callback != NULL)
+		cb=s->info_callback;
+	else if (s->ctx->info_callback != NULL)
+		cb=s->ctx->info_callback;
 
 	/* init things to blank */
-	if (!SSL_ST_INIT(s->state)) SSL_clear(s);
+	if (!SSL_in_init(s)) SSL_clear(s);
 
-	/*if (s->session == NULL)
-	 * Error picked up by sameer <sameer@c2.org> 11/12/95 */
-	if (s->cert == NULL)
+	if (((s->session == NULL) || (s->session->cert == NULL)) &&
+		(s->cert == NULL))
 		{
 		SSLerr(SSL_F_SSL_ACCEPT,SSL_R_NO_CERTIFICATE_SET);
 		return(-1);
@@ -100,121 +105,162 @@ SSL *s;
 	errno=0;
 	for (;;)
 		{
+		int state;
+
 #ifdef SSL_DEBUG
 		SSL_TRACE(SSL_ERR,"state=%02X %d %s\n",
 			s->state,s->init_num,ssl_state_string(s));
 #endif
+		state=s->state;
+
 		switch (s->state)
 			{
-		case ST_BEFORE:
-			if ((buf=BUF_new()) == NULL) return(-1);
-			if (!BUF_grow(buf,(unsigned long)
+		case SSL_ST_BEFORE:
+		case SSL_ST_ACCEPT:
+			if ((buf=BUF_MEM_new()) == NULL)
+				{ ret=-1; goto end; }
+			if (!BUF_MEM_grow(buf,(int)
 				SSL_MAX_RECORD_LENGTH_3_BYTE_HEADER))
-				return(-1);
+				{ ret=-1; goto end; }
 			if (s->init_buf) free(s->init_buf);
 			s->init_buf=(unsigned char *)buf->data;
 			free(buf);
 			s->init_num=0;
-			s->state=ST_GET_CLIENT_HELLO_A;
+			s->state=SSL_ST_GET_CLIENT_HELLO_A;
 			s->ctx->sess_accept++;
 			BREAK;
 
-		case ST_GET_CLIENT_HELLO_A:
-		case ST_GET_CLIENT_HELLO_B:
+		case SSL_ST_GET_CLIENT_HELLO_A:
+		case SSL_ST_GET_CLIENT_HELLO_B:
 			ret=get_client_hello(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_SEND_SERVER_HELLO_A;
+			s->state=SSL_ST_SEND_SERVER_HELLO_A;
 			BREAK;
 
-		case ST_SEND_SERVER_HELLO_A:
-		case ST_SEND_SERVER_HELLO_B:
+		case SSL_ST_SEND_SERVER_HELLO_A:
+		case SSL_ST_SEND_SERVER_HELLO_B:
 			ret=server_hello(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
 			if (!s->hit)
 				{
-				s->state=ST_GET_CLIENT_MASTER_KEY_A;
+				s->state=SSL_ST_GET_CLIENT_MASTER_KEY_A;
 				BREAK;
 				}
 			else
 				{
-				s->state=ST_START_ENCRYPTION;
-				break;
+				s->state=SSL_ST_SERVER_START_ENCRYPTION;
+				BREAK;
 				}
-		case ST_GET_CLIENT_MASTER_KEY_A:
-		case ST_GET_CLIENT_MASTER_KEY_B:
+		case SSL_ST_GET_CLIENT_MASTER_KEY_A:
+		case SSL_ST_GET_CLIENT_MASTER_KEY_B:
 			ret=get_client_master_key(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_START_ENCRYPTION;
+			s->state=SSL_ST_SERVER_START_ENCRYPTION;
 			BREAK;
 
-		case ST_START_ENCRYPTION:
+		case SSL_ST_SERVER_START_ENCRYPTION:
 			/* Ok we how have sent all the stuff needed to
 			 * start encrypting, the next packet back will
 			 * be encrypted. */
-			if (!s->session->cipher->crypt_init(s,0)) return(-1);
-			s->clear_text=0;
-			s->state=ST_SEND_SERVER_VERIFY_A;
-			BREAK;
-
-		case ST_SEND_SERVER_VERIFY_A:
-		case ST_SEND_SERVER_VERIFY_B:
-			ret=server_verify(s);
-			if (ret <= 0) return(ret);
-			s->init_num=0;
-			s->state=ST_GET_CLIENT_FINISHED_A;
-			BREAK;
-
-		case ST_GET_CLIENT_FINISHED_A:
-		case ST_GET_CLIENT_FINISHED_B:
-			ret=get_client_finished(s);
-			if (ret <= 0) return(ret);
-			s->init_num=0;
-			s->state=ST_SEND_REQUEST_CERTIFICATE_A;
-			BREAK;
-
-		case ST_SEND_REQUEST_CERTIFICATE_A:
-		case ST_SEND_REQUEST_CERTIFICATE_B:
-		case ST_SEND_REQUEST_CERTIFICATE_C:
-		case ST_SEND_REQUEST_CERTIFICATE_D:
-			if (!(s->verify_mode & SSL_VERIFY_PEER))
+			if (!s->session->cipher->crypt_init(s,0))
 				{
-				s->state=ST_SEND_SERVER_FINISHED_A;
+				ret=-1;
+				goto end;
+				}
+			s->clear_text=0;
+			s->state=SSL_ST_SEND_SERVER_VERIFY_A;
+			BREAK;
+
+		case SSL_ST_SEND_SERVER_VERIFY_A:
+		case SSL_ST_SEND_SERVER_VERIFY_B:
+			ret=server_verify(s);
+			if (ret <= 0) goto end;
+			s->init_num=0;
+			s->state=SSL_ST_GET_CLIENT_FINISHED_A;
+			BREAK;
+
+		case SSL_ST_GET_CLIENT_FINISHED_A:
+		case SSL_ST_GET_CLIENT_FINISHED_B:
+			ret=get_client_finished(s);
+			if (ret <= 0)
+				goto end;
+			s->init_num=0;
+			s->state=SSL_ST_SEND_REQUEST_CERTIFICATE_A;
+			BREAK;
+
+		case SSL_ST_SEND_REQUEST_CERTIFICATE_A:
+		case SSL_ST_SEND_REQUEST_CERTIFICATE_B:
+		case SSL_ST_SEND_REQUEST_CERTIFICATE_C:
+		case SSL_ST_SEND_REQUEST_CERTIFICATE_D:
+			/* don't do a 'request certificate' if we
+			 * don't want to, or we already have one, or
+			 * we once want to ever get one once. */
+			if (!(s->verify_mode & SSL_VERIFY_PEER)
+				|| (s->session->peer != NULL)
+				|| (s->verify_mode & SSL_VERIFY_CLIENT_ONCE))
+				{
+				s->state=SSL_ST_SEND_SERVER_FINISHED_A;
 				break;
 				}
 			else
 				{
 				ret=request_certificate(s);
-				if (ret <= 0) return(ret);
+				if (ret <= 0) goto end;
 				s->init_num=0;
-				s->state=ST_SEND_SERVER_FINISHED_A;
+				s->state=SSL_ST_SEND_SERVER_FINISHED_A;
 				}
 			BREAK;
 
-		case ST_SEND_SERVER_FINISHED_A:
-		case ST_SEND_SERVER_FINISHED_B:
+		case SSL_ST_SEND_SERVER_FINISHED_A:
+		case SSL_ST_SEND_SERVER_FINISHED_B:
 			ret=server_finish(s);
-			if (ret <= 0) return(ret);
+			if (ret <= 0) goto end;
 			s->init_num=0;
-			s->state=ST_OK;
+			s->state=SSL_ST_OK;
 			break;
 
-		case ST_OK:
+		case SSL_ST_OK:
 			free(s->init_buf);
 			s->init_buf=NULL;
 			s->init_num=0;
 			ERR_clear_error();
+
+			if (SSL_add_session(s->ctx,s->session) &&
+				(s->ctx->new_session_cb != NULL))
+				{
+				CS_BEGIN;
+				s->session->references++;
+				CS_END;
+				if (!s->ctx->new_session_cb(s->session))
+					{
+					CS_BEGIN;
+					s->session->references--;
+					CS_END;
+					}
+				}
+
 			s->ctx->sess_accept_good++;
-			return(1);
+			ret=1;
+			goto end;
+			BREAK;
 
 		default:
 			SSLerr(SSL_F_SSL_ACCEPT,SSL_R_UNKNOWN_STATE);
-			return(-1);
-			break;
+			ret=-1;
+			goto end;
+			BREAK;
 			}
+		
+		if ((cb != NULL) && (s->state != state))
+			cb(s,SSL_CB_ACCEPT_LOOP,1);
 		}
+end:
+	if (cb != NULL)
+		cb(s,SSL_CB_ACCEPT_EXIT,ret);
+	return(ret);
 	}
 
 static int get_client_master_key(s)
@@ -225,7 +271,7 @@ SSL *s;
 	CIPHER *cp,c;
 
 	p=s->init_buf;
-	if (s->state == ST_GET_CLIENT_MASTER_KEY_A)
+	if (s->state == SSL_ST_GET_CLIENT_MASTER_KEY_A)
 		{
 		i=SSL_read(s,(char *)&(p[s->init_num]),10-s->init_num);
 
@@ -261,11 +307,11 @@ SSL *s;
 		n2s(p,i); s->state_clear=i;
 		n2s(p,i); s->state_enc=i;
 		n2s(p,i); s->session->key_arg_length=i;
-		s->state=ST_GET_CLIENT_MASTER_KEY_B;
+		s->state=SSL_ST_GET_CLIENT_MASTER_KEY_B;
 		s->init_num=0;
 		}
 
-	/* ST_GET_CLIENT_MASTER_KEY_B */
+	/* SSL_ST_GET_CLIENT_MASTER_KEY_B */
 	p=s->init_buf;
 	keya=s->session->key_arg_length;
 	n=s->state_clear+s->state_enc+keya - s->init_num;
@@ -322,7 +368,7 @@ SSL *s;
 	CIPHER **cipher;
 
 	p=s->init_buf;
-	if (s->state == ST_GET_CLIENT_HELLO_A)
+	if (s->state == SSL_ST_GET_CLIENT_HELLO_A)
 		{
 		i=SSL_read(s,(char *)&(p[s->init_num]),9-s->init_num);
 		if (i < (9-s->init_num)) 
@@ -350,11 +396,11 @@ SSL *s;
 			SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_INVALID_CHALLENGE_LENGTH);
 			return(-1);
 			}
-		s->state=ST_GET_CLIENT_HELLO_B;
+		s->state=SSL_ST_GET_CLIENT_HELLO_B;
 		s->init_num=0;
 		}
 
-	/* ST_GET_CLIENT_HELLO_B */
+	/* SSL_ST_GET_CLIENT_HELLO_B */
 	p=s->init_buf;
 	n=s->state_cipher_spec_length+s->challenge_length+
 		s->state_session_id_length-s->init_num;
@@ -395,6 +441,13 @@ SSL *s;
 			}
 		else
 			{
+			if (s->cert == NULL)
+				{
+				ssl_return_error(s);
+				SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_NO_CERTIFICATE_SET);
+				return(-1);
+				}
+
 			if (!ssl_get_new_session(s,1))
 				{
 				ssl_return_error(s);
@@ -508,7 +561,7 @@ SSL *s;
 	CIPHER **cipher;
 
 	p=s->init_buf;
-	if (s->state == ST_SEND_SERVER_HELLO_A)
+	if (s->state == SSL_ST_SEND_SERVER_HELLO_A)
 		{
 		d=p+11;
 		*(p++)=SSL_MT_SERVER_HELLO;		/* type */
@@ -572,7 +625,7 @@ SSL *s;
 		s->conn_id_length=CONECTION_ID_LENGTH;
 		if (s->ctx->reverse)
 			{
-			if (s->ctx->conn_id_length != (int)s->conn_id_length)
+			if (s->ctx->conn_id_length != s->conn_id_length)
 				{
 				SSLerr(SSL_F_SERVER_HELLO,SSL_R_REVERSE_SSL_SESSION_ID_LENGTH_IS_WRONG);
 				return(-1);
@@ -585,11 +638,11 @@ SSL *s;
 		memcpy(d,s->conn_id,CONECTION_ID_LENGTH);
 		d+=CONECTION_ID_LENGTH;
 
-		s->state=ST_SEND_SERVER_HELLO_B;
+		s->state=SSL_ST_SEND_SERVER_HELLO_B;
 		s->init_num=d-s->init_buf;
 		s->init_off=0;
 		}
-	/* ST_SEND_SERVER_HELLO_B */
+	/* SSL_ST_SEND_SERVER_HELLO_B */
 	return(ssl_do_write(s));
 	}
 
@@ -600,7 +653,7 @@ SSL *s;
 	int i;
 
 	p=s->init_buf;
-	if (s->state == ST_GET_CLIENT_FINISHED_A)
+	if (s->state == SSL_ST_GET_CLIENT_FINISHED_A)
 		{
 		i=SSL_read(s,(char *)&(p[s->init_num]),1-s->init_num);
 		if (i < 1-s->init_num)
@@ -618,10 +671,10 @@ SSL *s;
 			return(-1);
 			}
 		s->init_num=0;
-		s->state=ST_GET_CLIENT_FINISHED_B;
+		s->state=SSL_ST_GET_CLIENT_FINISHED_B;
 		}
 
-	/* ST_GET_CLIENT_FINISHED_B */
+	/* SSL_ST_GET_CLIENT_FINISHED_B */
 	i=SSL_read(s,(char *)&(p[s->init_num]),
 		(unsigned int)s->conn_id_length-s->init_num);
 	if (i < (int)s->conn_id_length-s->init_num)
@@ -647,14 +700,14 @@ SSL *s;
 	{
 	unsigned char *p;
 
-	if (s->state == ST_SEND_SERVER_VERIFY_A)
+	if (s->state == SSL_ST_SEND_SERVER_VERIFY_A)
 		{
 		p=s->init_buf;
 		*(p++)=SSL_MT_SERVER_VERIFY;
 		memcpy(p,s->challenge,(unsigned int)s->challenge_length);
-		p+=s->challenge_length;
+		/* p+=s->challenge_length; */
 
-		s->state=ST_SEND_SERVER_VERIFY_B;
+		s->state=SSL_ST_SEND_SERVER_VERIFY_B;
 		s->init_num=s->challenge_length+1;
 		s->init_off=0;
 		}
@@ -666,22 +719,21 @@ SSL *s;
 	{
 	unsigned char *p;
 
-	if (s->state == ST_SEND_SERVER_FINISHED_A)
+	if (s->state == SSL_ST_SEND_SERVER_FINISHED_A)
 		{
 		p=s->init_buf;
 		*(p++)=SSL_MT_SERVER_FINISHED;
 
-		SSL_add_session(s->ctx,s->session);
 		memcpy(p,s->session->session_id,
 			(unsigned int)s->session->session_id_length);
-		p+=s->session->session_id_length;
+		/* p+=s->session->session_id_length; */
 
-		s->state=ST_SEND_SERVER_FINISHED_B;
+		s->state=SSL_ST_SEND_SERVER_FINISHED_B;
 		s->init_num=s->session->session_id_length+1;
 		s->init_off=0;
 		}
 
-	/* ST_SEND_SERVER_FINISHED_B */
+	/* SSL_ST_SEND_SERVER_FINISHED_B */
 	return(ssl_do_write(s));
 	}
 
@@ -704,7 +756,7 @@ SSL *s;
 			}
 		}
 	ccd=s->state_ccl;
-	if (s->state == ST_SEND_REQUEST_CERTIFICATE_A)
+	if (s->state == SSL_ST_SEND_REQUEST_CERTIFICATE_A)
 		{
 		p=s->init_buf;
 		*(p++)=SSL_MT_REQUEST_CERTIFICATE;
@@ -712,12 +764,12 @@ SSL *s;
 		RAND_bytes(ccd,MIN_CERT_CHALLENGE_LENGTH);
 		memcpy(p,ccd,MIN_CERT_CHALLENGE_LENGTH);
 
-		s->state=ST_SEND_REQUEST_CERTIFICATE_B;
+		s->state=SSL_ST_SEND_REQUEST_CERTIFICATE_B;
 		s->init_num=MIN_CERT_CHALLENGE_LENGTH+2;
 		s->init_off=0;
 		}
 
-	if (s->state == ST_SEND_REQUEST_CERTIFICATE_B)
+	if (s->state == SSL_ST_SEND_REQUEST_CERTIFICATE_B)
 		{
 		i=ssl_do_write(s);
 		if (i <= 0)
@@ -727,10 +779,10 @@ SSL *s;
 			}
 
 		s->init_num=0;
-		s->state=ST_SEND_REQUEST_CERTIFICATE_C;
+		s->state=SSL_ST_SEND_REQUEST_CERTIFICATE_C;
 		}
 
-	if (s->state == ST_SEND_REQUEST_CERTIFICATE_C)
+	if (s->state == SSL_ST_SEND_REQUEST_CERTIFICATE_C)
 		{
 		p=s->init_buf;
 		i=SSL_read(s,(char *)&(p[s->init_num]),6-s->init_num);
@@ -768,13 +820,12 @@ SSL *s;
 			}
 		n2s(p,i); s->state_clen=i;
 		n2s(p,i); s->state_rlen=i;
-		s->state=ST_SEND_REQUEST_CERTIFICATE_D;
+		s->state=SSL_ST_SEND_REQUEST_CERTIFICATE_D;
 		s->init_num=0;
 		}
 
-	/* ST_SEND_REQUEST_CERTIFICATE_D */
+	/* SSL_ST_SEND_REQUEST_CERTIFICATE_D */
 	p=s->init_buf;
-	if (s->state_ccl == NULL) free(s->state_ccl);
 	j=s->state_clen+s->state_rlen-s->init_num;
 	i=SSL_read(s,(char *)&(p[s->init_num]),(unsigned int)j);
 	if (i < j) 
@@ -794,47 +845,47 @@ SSL *s;
 	if (i)	/* we like the packet, now check the chksum */
 		{
 		EVP_MD_CTX ctx;
-		RSA *rsa=NULL;
+		EVP_PKEY *pkey=NULL;
 
-		EVP_VerifyInit(&ctx,EVP_md5);
+		EVP_VerifyInit(&ctx,EVP_md5());
 		EVP_VerifyUpdate(&ctx,s->key_material,
 			(unsigned int)s->key_material_length);
 		EVP_VerifyUpdate(&ctx,ccd,MIN_CERT_CHALLENGE_LENGTH);
 
-		i=i2d_X509(s->cert->x509,NULL);
-		buf2=(unsigned char *)malloc((unsigned int)i+10);
+		i=i2d_X509(s->session->cert->x509,NULL);
+		buf2=(unsigned char *)malloc((unsigned int)i);
 		if (buf2 == NULL)
 			{
 			SSLerr(SSL_F_REQUEST_CERTIFICATE,ERR_R_MALLOC_FAILURE);
 			goto end;
 			}
 		p2=buf2;
-		i=i2d_X509(s->cert->x509,&p2);
+		i=i2d_X509(s->session->cert->x509,&p2);
 		EVP_VerifyUpdate(&ctx,buf2,(unsigned int)i);
 		free(buf2);
 
-		rsa=X509_extract_key(x509);
-		if (rsa == NULL) goto end;
-		i=EVP_VerifyFinal(&ctx,p,s->state_rlen,rsa);
+		pkey=X509_extract_key(x509);
+		if (pkey == NULL) goto end;
+		i=EVP_VerifyFinal(&ctx,p,s->state_rlen,pkey);
 		memset(&ctx,0,sizeof(ctx));
-		RSA_free(rsa);
+		EVP_PKEY_free(pkey);
 
 		if (i) 
 			{
-			if ((s->peer_status == SSL_PEER_IN_SSL) &&
-				(s->peer != NULL))
-				X509_free(s->peer);
-			s->peer=x509;
-			s->peer_status=SSL_PEER_IN_SSL;
+			if (s->session->peer != NULL)
+				X509_free(s->session->peer);
+			s->session->peer=x509;
+			CS_BEGIN
+			x509->references++;
+			CS_END
 			ret=1;
-			goto end_no_x509_free;
+			goto end;
 			}
 		else
 			SSLerr(SSL_F_REQUEST_CERTIFICATE,SSL_R_BAD_CHECKSUM);
 		}
 end:
 	if (x509 != NULL) X509_free(x509);
-end_no_x509_free:
 	return(ret);
 	}
 
