@@ -1,7 +1,10 @@
 #!/usr/local/bin/perl
+#
+# Up the version numbers in the files.
+#
 
 @files=(
-	"apps/version.c",
+	"crypto/des/ecb_enc.c",
 	"crypto/idea/i_ecb.c",
 	"crypto/lhash/lhash.c",
 	"crypto/conf/conf.c",
@@ -11,7 +14,9 @@
 	"crypto/bn/bn_lib.c",
 	"crypto/dh/dh_lib.c",
 	"crypto/rc4/rc4_enc.org",
+	"crypto/rc2/rc2_ecb.c",
 	"crypto/rsa/rsa_lib.c",
+	"crypto/dsa/dsa_lib.c",
 	"crypto/sha/sha1dgst.c",
 	"crypto/sha/sha_dgst.c",
 	"crypto/asn1/asn1_lib.c",
@@ -20,6 +25,7 @@
 	"crypto/rand/md5_rand.c",
 	"crypto/stack/stack.c",
 	"crypto/txt_db/txt_db.c",
+	"crypto/cryptlib.c",
 	"ssl/ssl_lib.c",
 	"README",
 	);
@@ -36,11 +42,13 @@ foreach $file (@files)
 	{
 	open(IN,"<$file") || die "unable to open $file:$!\n";
 	open(OUT,">$file.new") || die "unable to open $file.new:$!\n";
+	$found=0;
 
 	while (<IN>)
 		{
-		if (s/SSLeay [^"]*(\")/SSLeay $ver $time\1/)
+		if (s/SSLeay [^"]*(\"|\s)/SSLeay $ver $time\1/)
 			{
+			$found++;
 			print OUT;
 			while (<IN>) { print OUT; }
 			last;
@@ -49,6 +57,7 @@ foreach $file (@files)
 		}
 	close(IN);
 	close(OUT);
+	(!$found) && die "unable to update the version number in $file\n";
 	rename($file,"$file.old") || die "unable to rename $file:$!\n";
 	rename("$file.new",$file) || die "unable to rename $file.new:$!\n";
 	}
